C  //////////////////////////////////////////////////////////////
C
C
C  PPPPP   EEEEEE  N    N  EEEEEE  L        OOOO   PPPPP   EEEEEE
C  P    P  E       NN   N  E       L       O    O  P    P  E
C  P    P  E       N N  N  E       L       O    O  P    P  E
C  PPPPP   EEEE    N  N N  EEEE    L       O    O  PPPPP   EEEE
C  P       E       N   NN  E       L       O    O  P       E
C  P       EEEEEE  N    N  EEEEEE  LLLLLL   OOOO   P       EEEEEE
C
C
C                                           (Version 2000.08.28).
C
C     Subroutine package for Monte Carlo simulation of electron-
C  photon showers.
C
C  //////////////////////////////////////////////////////////////
C
C  List of changes/amendments of the original (1996.02.29) code:
C
C  ** A minor error was found in subroutine SINCOH (the variable
C  NSM was undefined for energies larger than 5 MeV). Corrected
C  on January 26, 1998 by J.M. Fernandez-Varea and F. Salvat.
C  --------------------------
C
C  ** The following modifications were introduced in July, 2000
C  by F. Salvat:
C
C  1) The flexibility of the code was increased by defining the
C  simulation parameters EABS(3), C1, C2, WCC and WCR indepen-
C  dently for each material. This required multiple changes of
C  the source code. Notice that these parameters are now input
C  through the common block /SIMPAR/.
C
C  2) The structure of common TRACK was simplified by removing
C  several transport variables, whose values are now transferred
C  as arguments of subroutines JUMP and KNOCK. A label ILB was
C  added to provide information about the origin of secondary
C  radiations. The value ILB of a particle is inherited by its
C  direct descendants.
C
C  3) Subroutine JUMP(DSMAX,DS) was modified to make the simula-
C  tion of electron/positron tracks more flexible. We have added
C  the input argument DSMAX that defines the maximum allowed step
C  length and supersedes the old parameter HFPMAX, which has been
C  eliminated. The control of the step length is performed by
C  inserting delta interactions (which do nothing) along the
C  particle track. This modification has no practical effects
C  when DSMAX is much larger than the mean free path between hard
C  interactions. The output argument DS is the step length to the
C  following interaction event.
C
C  4) Subroutine DIRECT was recoded to avoid doing a permutation
C  of axes when the initial direction is parallel to (0,0,1).
C  This permutation was fine for the simulation of collisions,
C  but gave wrong results when used to generate the initial
C  direction of particles.
C
C  5) In the simulation of random hinges, the angular deflection
C  is sampled from the multiple scattering distribution corre-
C  sponding to the energy evaluated either at the beginning or at
C  the end of the step (i.e. after subtracting the energy loss
C  resulting from soft interactions), and the two options are
C  selected randomly with equal probabilities. This procedure,
C  which was suggested by Alex Bielajew, partially accounts for
C  the energy variation of the soft 'scattering power'.
C
C  6) We have introduced the following systematic correction to
C  account for the energy variation of the hard inverse mean free
C  paths (or interaction probabilities per unit path length, IP)
C  between successive hard interactions. Instead of sampling the
C  step length by using the total IP corresponding to the energy
C  at the beginning of the step, we use an 'augmented' energy-
C  independent IP that is larger than the actual IP in the energy
C  interval swept while the particle travels through the step.
C  The excess IP is compensated by means of delta interactions.
C
C  --------------------------



C  **************************************************************
C                       SUBROUTINE PEINIT
C  **************************************************************
      SUBROUTINE PEINIT(EMAX,NMAT,IPRINT)
C
C     INPUT OF MATERIAL DATA AND INITIALIZATION.
C
C     EACH MATERIAL IS DEFINED THROUGH THE INPUT FILE (UNIT=5),
C  WHICH IS CREATED BY THE PROGRAM 'MATERIAL' USING INFORMATION
C  CONTAINED IN THE DATABASE. THIS FILE CAN BE MODIFIED BY THE
C  USER IF MORE ACCURATE INFORMATION IS AVAILABLE. DATA FILES FOR
C  DIFFERENT MATERIALS MUST BE CONCATENATED IN A SINGLE INPUT
C  FILE, THE I-TH MATERIAL IN THIS FILE IS IDENTIFIED BY THE
C  INDEX M=I.
C
C     THE FORMAT OF THE DATA FILE FOR EACH MATERIAL IS:
C...+....1....+....2....+....3....+....4....+....5....+....6....+
C1   MATERIAL:(                      A48                       )
C2   MASS DENSITY=(   E11.4   )G/CM3
C3   NUMBER OF ELEMENTS IN THE MOLECULE=(I2) /.LE.15/
C4+  ATOMIC NUMBER=(I2),ATOMS/MOLECULE=(    E13.6    )
C5   MEAN EXCITATION ENERGY=(  F8.2  )EV
C6   NUMBER OF OSCILLATORS=(I2) /.LE.30/
C7+  (    E13.6    ,    E13.6    ,  E10.3   ,I2) OSCILLATOR DATA
C8   NUMBER OF INPUT ENERGIES=( I3) FOR ELECTRONS /.LE.180/
C9+  (E,MFP,1ST AND 2ND TRANS.MFP,COL.SP,RAD.SP) (EV,MTU)
C10  NUMBER OF INPUT ENERGIES=( I3) FOR POSITRONS /.LE.180/
C11+ (E,MFP,1ST AND 2ND TRANS.MFP,COL.SP,RAD.SP) (EV,MTU)
C12  NUMBER OF INPUT ENERGIES=( I3) FOR PHOTONS /.LE.400/
C13+ (E,RAYL,COMP,PHOT AND PAIRP AT.COEFFICIENTS) (EV,CM2/G)
C...+....1....+....2....+....3....+....4....+....5..E.+....6....+
C                                      NOTE: 1 MTU = 1.0D-6 G/CM2
C
C  - LINE C1: TEXT DESCRIBING THE MATERIAL.
C
C  - LINE C2: SAMPLE DENSITY (IN G/CM3).
C
C  - LINE C3: NUMBER OF DIFFERENT ELEMENTS IN THE MATERIAL
C       (NELEM), LESS THAN OR EQUAL TO 15.
C
C  - NELEM LINES C4: EACH ONE CONTAINING THE ATOMIC NUMBER AND
C       THE NUMBER OF ATOMS PER 'MOLECULE' OF AN ELEMENT IN THE
C       MATERIAL.
C
C  - LINE C5: MEAN EXCITATION ENERGY (IN EV). IF THE INPUT VALUE
C       IS LESS THAN 1 EV, IT IS COMPUTED FROM STORED VALUES FOR
C       SINGLE ELEMENT MATERIALS BY USING THE ADDITIVITY RULE.
C
C  - LINE C6: NUMBER OF OSCILLATORS (NOS), LESS THAN OR EQUAL TO
C       30.
C
C  - NOS LINES C7: OSCILLATOR DATA. THE FOLLOWING QUANTITIES IN
C       EACH LINE
C       - OSCILLATOR STRENGTH,
C       - IONIZATION ENERGY (EV),
C       - ONE-ELECTRON COMPTON PROFILE FOR P=0 (ATOMIC UNITS),
C       - IZ FOR THE K-SHELL OF THE ELEMENT OF ATOMIC NUMBER IZ,
C         0 FOR OTHER SHELLS.
C
C  - LINE C8: NUMBER OF INPUT ENERGY MESH POINTS FOR ELECTRON
C       SCATTERING DATA (NINPE), LESS THAN 180.
C
C  - NINPE LINES C9: EACH ONE CONTAINING THE FOLLOWING QUANTI-
C       TIES (IN THE GIVEN ORDER AND FREE FORMAT)
C       - ELECTRON KINETIC ENERGY (EV) IN INCREASING ORDER,
C       - ELASTIC MEAN FREE PATH (MTU),
C       - FIRST TRANSPORT MEAN FREE PATH (MTU),
C       - SECOND TRANSPORT MEAN FREE PATH (MTU),
C       - COLLISION STOPPING POWER (EV/MTU),
C       - BREMSSTRAHLUNG (RADIATIVE) STOPPING POWER (EV/MTU).
C
C  - LINE C10: NUMBER OF INPUT ENERGY MESH POINTS FOR POSITRON
C       SCATTERING DATA (NINPP), LESS THAN 180.
C
C  - NINPP LINES C11: EACH ONE CONTAINING THE SAME DATA AS LINES
C       C9 BUT FOR POSITRONS (THE ENERGIES CAN BE DIFFERENT FROM
C       THOSE ENTERED FOR ELECTRONS).
C
C  - LINE C12: NUMBER OF INPUT ENERGY MESH POINTS FOR PHOTON
C       ATTENUATION COEFFICIENTS (NINPG) LESS THAN 400.
C
C  - NINPG LINES C13: EACH ONE CONTAINING THE FOLLOWING DATA
C       (IN THE GIVEN ORDER AND FREE FORMAT)
C       - PHOTON ENERGY (EV) IN INCREASING ORDER,
C       - RAYLEIGH ATTENUATION COEFFICIENT (CM2/G),
C       - COMPTON ATTENUATION COEFFICIENT (CM2/G),
C       - PHOTOELECTRIC ATTENUATION COEFFICIENT (CM2/G),
C       - PAIR-PRODUCTION ATTENUATION COEFFICIENT (CM2/G).
C
C     THE PROGRAM IS ABORTED WHEN A CLEARLY INCORRECT INPUT DATUM
C  IS FOUND. THE INCORRECT QUANTITY APPEARS IN THE LAST LINE OF
C  THE OUTPUT FILE.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), CHARACTER*2 (L),
     1   INTEGER*4 (I-K,M-N)
      PARAMETER (PI=3.1415926535897932D0, FOURPI=4.0D0*PI)
      PARAMETER (A0B=5.29177249D-9, HREV=27.2113961D0,
     1 AVOG=6.0221367D23, SL=137.0359895D0)
      PARAMETER (MAXMAT=10, NO=30, NMP=200)
      DIMENSION F(NO),UI(NO),WI(NO),EEAB(3)
C  ****  MAIN-PENELOPE COMMONS.
      COMMON/MATER/AW(MAXMAT),DEN(MAXMAT),RANGE(3,MAXMAT)
      COMMON/simpar/EABS(3,MAXMAT),C1(MAXMAT),C2(MAXMAT),
     1              WCC(MAXMAT),WCR(MAXMAT)
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C  ****  ENERGY MESH POINTS.
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
C  ****  PEINIT-PECALC COMMONS.
      COMMON/COMP/ZAT(MAXMAT,15),STF(MAXMAT,15),ZT(MAXMAT),
     1 AT(MAXMAT),ZEQ(MAXMAT),AEQ(MAXMAT),RHO(MAXMAT),
     2 EXPOT(MAXMAT),NELEM(MAXMAT)
      COMMON/XHIN/FZ(MAXMAT,NO),BEN(MAXMAT,NO),WRI(MAXMAT,NO),
     1 WRI2(MAXMAT,NO),NOSC(MAXMAT)
      COMMON/XINCIA/FJ0(MAXMAT,NO),IZFL(MAXMAT,NO)
      COMMON/ERRSEC/IERSEC
C  ****  ELEMENT DATA.
      COMMON/DELEM/LAS(99)
      COMMON/CATWG/ATW(99)
      COMMON/CMEXP/EPX(99)
      IERSEC=0
C
      WRITE(6,2000)
 2000 FORMAT(1H1//3X,39('*'),/3X,'**   PENELOPE ',
     1  '(VERSION 2000.08.28)   **',/3X,39('*'))
C
C  ****  Lower limit of the energy grid.
C
      emin=1.0d35
      do m=1,nmat
        emin=dmin1(emin,eabs(1,m),eabs(2,m),eabs(3,m))
      enddo
      if(emin.lt.100.0d0) emin=100.0d0
C
C  ****  ENERGY MESH POINTS.
C
      EL=0.99999D0*EMIN
      EU=1.00001D0*EMAX
      DLFC=DLOG(EU/EL)/DFLOAT(NMP-1)
      DLEMP1=DLOG(EL)
      DLEMP(1)=DLEMP1
      DO I=2,NMP
        DLEMP(I)=DLEMP(I-1)+DLFC
      ENDDO
      DLFC=1.0D0/DLFC
      WRITE(6,2100) EMIN,EMAX
 2100 FORMAT(/3X,'EMIN =',1P,E11.4,' eV,  EMAX =',E11.4,' eV')
C
C  ************  INPUT OF MATERIAL DATA AND SCATTERING FUNCTIONS.
C
      IF(NMAT.GT.MAXMAT) THEN
        WRITE(6,2001) NMAT,MAXMAT,NMAT
 2001   FORMAT(3X,'*** PENELOPE CANNOT HANDLE ',I2,' DIFFER',
     1  'ENT MATERIALS.'/7X,'EDIT THE SOURCE FILE AND CHANGE ',
     2  'THE PARAMETER MAXMAT = ',I2,' TO MAXMAT = ',I2,'.')
        STOP
      ENDIF
C
      DO 11 M=1,NMAT
      READ(5,1001) (UI(I),I=1,12)
 1001 FORMAT(15X,12A4)
      IF(M.EQ.1) LIT='st'
      IF(M.EQ.2) LIT='nd'
      IF(M.EQ.3) LIT='rd'
      IF(M.GT.3) LIT='th'
      WRITE(6,2002) M,LIT,(UI(I),I=1,12)
 2002 FORMAT(//3X,22('*')/3X,'**  ',I2,A2,' MATERIAL   **'
     1  /3X,22('*')//3X,12A4)
C
C  ****  ENERGY LIMITS AND THRESHOLDS.
C
      WRITE(6,2003)
 2003 FORMAT(/3X,'*** SIMULATION PARAMETERS:')
      IF(EABS(1,m).LT.1.0D2) THEN
        EABS(1,m)=1.0D2
        WRITE(6,2004)
 2004 FORMAT(/3X,'*** WARNING: ELECTRON ABSORPTION ENERGY HAS ',
     1  'BEEN SET TO 100 eV.')
      ENDIF
      WRITE(6,2005) EABS(1,m)
 2005 FORMAT(7X,'ELECTRON ABSORPTION ENERGY =',1P,E11.4,' eV')
c
      IF(EABS(2,m).LT.1.0D3) THEN
        EABS(2,m)=1.0D3
        WRITE(6,2006)
 2006 FORMAT(/3X,'*** WARNING: PHOTON ABSORPTION ENERGY HAS ',
     1  'BEEN SET TO 1 keV.')
        EABS(2,m)=1.0D3
      ENDIF
      WRITE(6,2007) EABS(2,m)
 2007 FORMAT(7X,'  PHOTON ABSORPTION ENERGY =',1P,E11.4,' eV')
c
      IF(EABS(3,m).LT.1.0D2) THEN
        EABS(3,m)=1.0D2
        WRITE(6,2008)
 2008 FORMAT(/3X,'*** WARNING: POSITRON ABSORPTION ENERGY HAS ',
     1  'BEEN SET TO 100 eV.')
      ENDIF
      WRITE(6,2009) EABS(3,m)
 2009 FORMAT(7X,'POSITRON ABSORPTION ENERGY =',1P,E11.4,' eV')
C
      CC1=DMIN1(0.2D0,DABS(C1(m)))
      CC2=DMIN1(0.1D0,DABS(C2(m)))
      TDR(m)=DMIN1(DABS(WCC(m)),EMAX)
      THB(m)=0.0D0
      IF(WCR(m).GE.0.0D0)
     1   THB(m)=DMIN1(DMAX1(10.00001D0,WCR(m)),EMAX)
      WRITE(6,2010) CC1,CC2,TDR(m),THB(m)
 2010 FORMAT(8X,'C1 =',1P,E11.4,',       C2 =',E11.4,/7X,'WCC =',
     2  E11.4,' eV,   WCR =',E11.4,' eV')
      IF(THB(m).LT.10.0D0) WRITE(6,2011)
 2011 FORMAT(/3X,'*** WARNING: RADIATIVE LOSSES ARE SWITCHED',
     1  ' OFF.'/)
C
C  ****  DENSITY.
C
      READ(5,1002) RHO(M)
 1002 FORMAT(19X,E11.4)
      WRITE(6,2012) RHO(M)
 2012 FORMAT(/3X,'MASS DENSITY =',1P,E11.4,' g/cm3')
      IF(RHO(M).LE.0.0D0) STOP
      DEN(M)=RHO(M)
C
C  ****  COMPOSITION.
C
      READ(5,1003) NELEM(M)
 1003 FORMAT(41X,I2)
      WRITE(6,2013)
 2013 FORMAT(/3X,'*** COMPOSITION:')
      IF(NELEM(M).LT.1.OR.NELEM(M).GT.15) STOP
C
      ZT(M)=0.0D0
      AT(M)=0.0D0
      FACT=0.0D0
      EXPT=0.0D0
C
      NNELEM=NELEM(M)
      DO 3 I=1,NNELEM
      READ(5,1004) IZZ,STF(M,I)
 1004 FORMAT(20X,I2,18X,E13.6)
      WRITE(6,2014) LAS(IZZ),IZZ,STF(M,I)
 2014 FORMAT(3X,'ELEMENT: ',A2,' (Z=',I2,'),  ATOMS/MOLECULE =',
     1  1P,E13.6)
      IF(IZZ.LT.1) STOP
      IF(STF(M,I).LE.0.0D0) STOP
      ZT(M)=ZT(M)+IZZ*STF(M,I)
      AT(M)=AT(M)+ATW(IZZ)*STF(M,I)
      FACT=FACT+IZZ*ATW(IZZ)*STF(M,I)
      EXPT=EXPT+IZZ*DLOG(EPX(IZZ))*STF(M,I)
      ZAT(M,I)=IZZ
    3 CONTINUE
C
      EXPT=DEXP(EXPT/ZT(M))
      ZEQ(M)=FACT/AT(M)
      AEQ(M)=FACT/ZT(M)
      AW(M)=AT(M)
      VMOL=(AVOG*RHO(M)/AT(M))*A0B**3
C  ****  PLASMA RESONANCE ENERGY.
      OP2=FOURPI*ZT(M)*VMOL
      OMEGA2=OP2*HREV*HREV
C
C  ****  MEAN EXCITATION ENERGY.
C
      READ(5,1005) EXPOT(M)
 1005 FORMAT(29X,F8.2)
      IF(EXPOT(M).LT.1.0D0) EXPOT(M)=EXPT
      WRITE(6,2015) EXPOT(M)
 2015 FORMAT(/3X,'MEAN EXCITATION ENERGY =',1P,E11.4,' eV')
C
C  ****  OSCILLATOR DATA.
C
      READ(5,1006) NOS
 1006 FORMAT(28X,I2)
      WRITE(6,2016) NOS
 2016 FORMAT(/3X,'NUMBER OF OSCILLATORS = ',I2)
      IF(NOS.GT.NO) THEN
        WRITE(6,2017) NOS,NO,NOS
 2017   FORMAT(3X,'*** PENELOPE CANNOT DEAL WITH ',I3,' DIFFER',
     1  'ENT OSCILLATORS.'/7X,'EDIT THE SOURCE FILE AND ',
     2  'CHANGE THE PARAMETER NO = ',I3,' TO NO = ',I3,'.')
        STOP
      ENDIF
      IF(NOS.LT.1) STOP
C  ****  OPTICAL OSCILLATOR STRENGTH AND ONE-ELECTRON COMPTON
C        PROFILES.
      WRITE(6,2018)
 2018 FORMAT(6X,'STRENGTH',8X,'Ui(eV)',9X,'Wi(eV)',6X,
     1 'Ji0(a.u.)',3X,'IZ')
      DO 4 I=1,NOS
      READ(5,1007) F(I),UI(I),FJ0(M,I),IZFL(M,I)
 1007 FORMAT(6X,E13.6,1X,E13.6,1X,E10.3,1X,I2)
      IF((F(I).LT.1.0D-6).OR.(DABS(UI(I)).LT.1.0D-1).OR.
     1   (FJ0(M,I).LT.1.0D-16).OR.(IZFL(M,I).LT.0)) THEN
        BEN(M,I)=DMAX1(UI(I),0.0D0)
        WRITE(6,2019) F(I),BEN(M,I),UI(I),FJ0(M,I),IZFL(M,I)
        STOP
      ENDIF
    4 CONTINUE
 2019 FORMAT(3X,1P,E13.6,2X,E13.6,2X,E13.6,2X,E10.3,3X,I2)
C  ****  OSCILLATORS ARE SORTED BY INCREASING ENERGIES.
      FT=0.0D0
      IF(NOS.GT.1) THEN
        DO 6 I=1,NOS-1
        DO 5 J=I+1,NOS
        IF(UI(I).LT.UI(J)) GO TO 5
        SAVE=UI(I)
        UI(I)=UI(J)
        UI(J)=SAVE
        SAVE=F(I)
        F(I)=F(J)
        F(J)=SAVE
        SAVE=FJ0(M,I)
        FJ0(M,I)=FJ0(M,J)
        FJ0(M,J)=SAVE
        ISAVE=IZFL(M,I)
        IZFL(M,I)=IZFL(M,J)
        IZFL(M,J)=ISAVE
    5   CONTINUE
        FT=FT+F(I)
    6   CONTINUE
      ELSE
        UI(1)=DABS(UI(1))
      ENDIF
      FT=FT+F(NOS)
      IF(DABS(FT-ZT(M)).GT.1.0D-4) THEN
        DO 7 I=1,NOS
        BEN(M,I)=DMAX1(UI(I),0.0D0)
        WRITE(6,2019) F(I),BEN(M,I),UI(I),FJ0(M,I),IZFL(M,I)
    7   CONTINUE
        STOP
      ENDIF
C
C  ****  STERNHEIMER ADJUSTMENT FACTOR.
C
      IF(NOS.GT.1) THEN
        TST=ZT(M)*DLOG(EXPOT(M))
        AAL=0.5D0
        AAU=10.0D0
    8   AA=0.5D0*(AAL+AAU)
        SUM=0.0D0
        DO 9 I=1,NOS
        IF(UI(I).GT.0.0D0) THEN
        WI2=(AA*UI(I))**2+0.666666666666666D0*(F(I)/ZT(M))*OMEGA2
          WI(I)=DSQRT(WI2)
        ELSE
          WI(I)=DABS(UI(I))
        ENDIF
        SUM=SUM+F(I)*DLOG(WI(I))
    9   CONTINUE
        IF(SUM.LT.TST) THEN
          AAL=AA
        ELSE
          AAU=AA
        ENDIF
        IF(AAU-AAL.GT.1.0D-6*AA) GO TO 8
      ELSE
        UI(1)=DABS(UI(1))
        WI(1)=EXPOT(M)
      ENDIF
C  ****  QUANTITIES STORED IN COMMON BLOCKS.
      TST=0.0D0
      DO 10 I=1,NOS
      FZ(M,I)=F(I)/FT
      BEN(M,I)=DMAX1(UI(I),0.0D0)
      WRI(M,I)=WI(I)
      WRI2(M,I)=WRI(M,I)**2
      WI(I)=WI(I)/HREV
      WRITE(6,2019) F(I),BEN(M,I),WRI(M,I),FJ0(M,I),IZFL(M,I)
      FJ0(M,I)=FJ0(M,I)*SL
      TST=TST+FZ(M,I)*DLOG(WRI(M,I))
   10 CONTINUE
      NOSC(M)=NOS
C
      EXPT=DEXP(TST)
      IF(DABS(EXPT-EXPOT(M)).GT.1.0D-6*EXPOT(M)) THEN
        WRITE(6,2020) EXPT
 2020   FORMAT(/3X,'*** INCONSISTENT OPTICAL OSCILLATOR STRENG',
     1  'TH PARAMETERS.'/7X,'CALCULATED MEAN EXCITATION ENERGY',
     2  ' =',1P,E11.4,' eV')
        STOP
      ENDIF
      TST=DSQRT(OMEGA2)
      WRITE(6,2021) TST
 2021 FORMAT(3X,'PLASMA ENERGY =',1P,E11.4,' eV')
      WRITE(6,2022) AA
 2022 FORMAT(3X,'STERNHEIMER ADJUSTMENT FACTOR =',1P,E11.4)
C
      EEAB(1)=EABS(1,m)
      EEAB(2)=EABS(2,m)
      EEAB(3)=EABS(3,m)
      CALL PECALC(EEAB,CC1,CC2,F,WI,OP2,NOS,IPRINT)
C
   11 CONTINUE
C
      WRITE(6,'(/3X,''***  END OF PEINIT  ***'',//)')
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PECALC
C  **************************************************************
      SUBROUTINE PECALC(EEAB,CC1,CC2,F,WI,OP2,NOS,IWR)
C
C     CALCULATION OF INTERPOLATION ARRAYS FOR ENERGY-DEPENDENT
C  QUANTITIES.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      CHARACTER*2 LCUTEL
      PARAMETER (PI=3.1415926535897932D0)
      PARAMETER (A0B=5.29177249D-9, HREV=27.2113961D0,
     1 SL=137.0359895D0, SL2=SL*SL, REV=5.1099906D5,
     2 AVOG=6.0221367D23)
      PARAMETER (MAXMAT=10, NO=30, NMP=200, NMS=400)
C  ****  ENERGY MESH POINTS.
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
C
C  ************  INTERPOLATION ARRAYS.
C
C  ELECTRONS AND POSITRONS:
C     - INVERSE MFP FOR HARD ELASTIC COLLISIONS (SHE),
C     - INVERSE MFP FOR ELASTIC COLLISIONS (SEL),
C     - INVERSE MFP FOR HARD INELASTIC COLLISIONS (SIN),
C     - INVERSE MFP FOR HARD BREMSSTRAHLUNG EMISSION (SBR),
C     - INVERSE MFP FOR POSITRON ANNIHILATION (SAN),
C     - TOTAL INVERSE MFP FOR HARD EVENTS (PTOT).
C
C     - ELASTIC DISTRIBUTION PARAMETERS (AEL,BEL),
C     - STOPPING POWER DUE TO SOFT STOPPING INTERACTIONS (W1),
C     - STRAGGLING DUE TO SOFT STOPPING INTERACTIONS (W2),
C     - BREMSSTRAHLUNG CROSS SECTION PARAMETERS (F0,BCB).
C
C  PHOTONS:
C     - INVERSE MFP FOR RAYLEIGH SCATTERING (SRA),
C     - INVERSE MFP FOR COMPTON SCATTERING (SCO),
C     - INVERSE MFP FOR PHOTOELECTRIC ABSORPTION (SPH;EPHT,SPHT),
C     - INVERSE MFP FOR ELECTRON-POSITRON PAIR PRODUCTION (SPA).
C
      COMMON/DELE/SHEE(MAXMAT,NMP),SELE(MAXMAT,NMP),
     1 SINE(MAXMAT,NMP),SBRE(MAXMAT,NMP),AELE(MAXMAT,NMP),
     2 BELE(MAXMAT,NMP),W1E(MAXMAT,NMP),W2E(MAXMAT,NMP),
     3 DEL(MAXMAT,NMP),PTOTE(MAXMAT,NMP),ISOFT(maxmat)
      COMMON/DPOS/SHEP(MAXMAT,NMP),SELP(MAXMAT,NMP),
     1 SINP(MAXMAT,NMP),SBRP(MAXMAT,NMP),SAN(MAXMAT,NMP),
     2 AELP(MAXMAT,NMP),BELP(MAXMAT,NMP),W1P(MAXMAT,NMP),
     3 W2P(MAXMAT,NMP),PTOTP(MAXMAT,NMP)
      COMMON/DGAM/SRA(MAXMAT,NMP),SCO(MAXMAT,NMP),
     1 SPH(MAXMAT,NMP),EPHT(MAXMAT,NMS),SPHT(MAXMAT,NMS),
     2 SPA(MAXMAT,NMP),MPH0(MAXMAT),INTPH0(MAXMAT)
C  ****  MAIN-PENELOPE COMMONS.
      COMMON/MATER/AW(MAXMAT),DEN(MAXMAT),RANGE(3,MAXMAT)
      COMMON/TRACK/E,HX,HY,HZ,U,V,W,WGHT,KPAR,IBODY,M,ILB
      COMMON/JMODES/MODE,MODEE,MODEI,IEXCD
      COMMON/JUMCOL/P(5),ST,DST,DS1,W1,W2,XEL,XE,XEK,K
C  ****  PEINIT-PECALC COMMONS.
      COMMON/COMP/ZAT(MAXMAT,15),STF(MAXMAT,15),ZT(MAXMAT),
     1 AT(MAXMAT),ZEQ(MAXMAT),AEQ(MAXMAT),RHO(MAXMAT),
     2 EXPOT(MAXMAT),NELEM(MAXMAT)
      COMMON/XBRE/F0(MAXMAT,3),BCB(MAXMAT)
C  ****  OUTPUT FROM PHYSICAL SUBROUTINES.
      COMMON/COLIN/DELTA,CSD(NO),CSC(NO)
      COMMON/BREMS/CBCOR,CGAM,CEPCUT,CF0(4),CBCB
C  **** STORAGE.
      COMMON/SECST/ET(NMS),FPE(NMS),TFP1(NMS),TFP2(NMS),SPC(NMS),
     1   SPR(NMS),R(NMS),AUX(NMS),KS(NMS),IBODYS(NMS),MS(NMS),
     2   ILBS(NMS),NSEC
      DIMENSION X(NMS),Y1(NMS),A1(NMS),B1(NMS),C1(NMS),D1(NMS),
     2  Y2(NMS),A2(NMS),B2(NMS),C2(NMS),D2(NMS)
C
      DIMENSION F(NO),WI(NO),EEAB(3),ICUTOF(NMP),LCUTEL(3)
      DATA LCUTEL/'FP','C1','C2'/
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C  ************  ELECTRON AND POSITRON CROSS SECTIONS.
C
C  ****  CONVERSION FACTORS FROM BOHR**2 TO 1/MTU.
      FCMTU=1.0D-6*AVOG*A0B**2/AT(M)
      FCMTUE=1.0D-6*AVOG*A0B**2/AEQ(M)
C  ****  CONVERSION FACTORS FROM BOHR**2 TO 1/CM.
      FCCMI=AVOG*RHO(M)*A0B**2/AT(M)
      FCCMIE=AVOG*RHO(M)*A0B**2/AEQ(M)
      ZZT=ZT(M)
      ZZEQ=ZEQ(M)
C
      KPAR=1
    1 CONTINUE
      IELEC=KPAR-2
      READ(5,1001) NINP
 1001 FORMAT(31X,I3)
C
      IF(NINP.GT.NMS-1.OR.NINP.LT.4) THEN
        WRITE(6,2001) NINP
 2001   FORMAT(/3X,'NUMBER OF INPUT ENERGIES = ',I4,'. STOP.')
        STOP
      ENDIF
C
      DO 2 I=1,NINP
      READ(5,*) ET(I),FPE(I),TFP1(I),TFP2(I),SPC(I),SPR(I)
    2 CONTINUE
C
C  ****  PRINT INPUT TABLES IF IWR.GT.0.
C
      IF(IWR.GT.0) THEN
      IF(KPAR.EQ.1) WRITE(6,2002)
 2002 FORMAT(//3X,'******  ELECTRON SCATTERING FUNCTIONS  ***')
      IF(KPAR.EQ.3) WRITE(6,2003)
 2003 FORMAT(//3X,'******  POSITRON SCATTERING FUNCTIONS  ***')
      WRITE(6,2007)
 2007 FORMAT(/4X,'ENERGY',5X,'EL MFP',4X,'1st TMFP',3X,
     1  '2nd TMFP',3X,'COL STOP',3X,'RAD STOP',5X,'RANGE')
      WRITE(6,2008)
 2008 FORMAT(5X,'(eV)',6X,'(mtu)',6X,'(mtu)',6X,'(mtu)',5X,
     2  '(eV/mtu)',3X,'(eV/mtu)',5X,'(cm)'/3X,75('-'))
      ENDIF
C
C  ****  RANGE AS A FUNCTION OF ENERGY.
C
      DO 11 I=1,NINP
      IF(THB(m).LT.10.0D0) SPR(I)=0.0D0
      Y1(I)=1.0D0/(SPC(I)+SPR(I))
   11 CONTINUE
C
      CALL SPLINE(ET,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      RANGI=0.0D0
      R(1)=0.0D0
      DO 12 I=2,NINP
      XL=ET(I-1)
      XU=ET(I)
      CALL INTEG(ET,A1,B1,C1,D1,XL,XU,DR,NINP)
      RANGI=RANGI+DR
      R(I)=RANGI
   12 CONTINUE
C
C  ****  TABLE OF MEAN FREE PATHS, STOPPING POWERS AND RANGE.
C
      DO 13 I=1,NINP
      TL=R(I)/(1.0D6*RHO(M))
      IF(IWR.GT.0) WRITE(6,2009)
     1  ET(I),FPE(I),TFP1(I),TFP2(I),SPC(I),SPR(I),TL
      IF(DMIN1(ET(I),FPE(I),TFP1(I),TFP2(I),SPC(I)).LE.0.0D0)
     1 STOP
      X(I)=DLOG(ET(I))
   13 CONTINUE
 2009 FORMAT(1X,1P,7E11.3)
C
C  ****  CSDA RANGE.
C
      NINP1=NINP+1
      ET(NINP1)=DMAX1(EU,2.0D0*ET(NINP))
      X(NINP1)=DLOG(ET(NINP1))
      CALL SPLINE(ET,R,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      A1(NINP)=A1(NINP-1)+ET(NINP)*(B1(NINP-1)+ET(NINP)
     1        *(C1(NINP-1)+ET(NINP)*D1(NINP-1)))
      B1(NINP)=B1(NINP-1)+ET(NINP)*(2.0D0*C1(NINP-1)
     1        +ET(NINP)*3.0D0*D1(NINP-1))
      A1(NINP)=A1(NINP)-B1(NINP)*ET(NINP)
      C1(NINP)=0.0D0
      D1(NINP)=0.0D0
      CALL FINDI(ET,EEAB(KPAR),NINP1,K)
      RANGL=A1(K)+EL*(B1(K)+EL*(C1(K)+EL*D1(K)))
      CALL FINDI(ET,EU,NINP1,K)
      RANGU=A1(K)+EU*(B1(K)+EU*(C1(K)+EU*D1(K)))
      RNG=RANGU-RANGL
      RANGE(KPAR,M)=RNG/(1.0D6*RHO(M))
      IF(KPAR.EQ.1) WRITE(6,2010) RNG,RANGE(KPAR,M)
 2010 FORMAT(/3X,'ELECTRON CSDA RANGE (EMAX,EABS) = ',1P,E11.4,
     1  ' mtu = ',E11.4,' cm')
      IF(KPAR.EQ.3) WRITE(6,2011) RNG,RANGE(KPAR,M)
 2011 FORMAT(/3X,'POSITRON CSDA RANGE (EMAX,EABS) = ',1P,E11.4,
     1  ' mtu = ',E11.4,' cm')
C
C  ************  INTERPOLATION ARRAYS.
C
C  ****  INELASTIC SCATTERING AND BREMSSTRAHLUNG.
C
      ISOFT(m)=1
      TDRH=TDR(m)/HREV
      THBH=THB(m)/HREV
      IF(TDRH.LT.WI(1).AND.THB(m).LT.10.0D0) ISOFT(m)=0
C
      DO 14 I=1,NINP
      Y1(I)=DLOG(SPC(I))
      Y2(I)=DLOG(DMAX1(SPR(I),1.0D-35))
   14 CONTINUE
C
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      A1(NINP)=A1(NINP-1)+X(NINP)*(B1(NINP-1)+X(NINP)*(C1(NINP-1)
     1        +X(NINP)*D1(NINP-1)))
      B1(NINP)=B1(NINP-1)+X(NINP)*(2.0D0*C1(NINP-1)
     1        +X(NINP)*3.0D0*D1(NINP-1))
      A1(NINP)=A1(NINP)-B1(NINP)*X(NINP)
      C1(NINP)=0.0D0
      D1(NINP)=0.0D0
      CALL SPLINE(X,Y2,A2,B2,C2,D2,0.0D0,0.0D0,NINP)
      A2(NINP)=A2(NINP-1)+X(NINP)*(B2(NINP-1)+X(NINP)*(C2(NINP-1)
     1        +X(NINP)*D2(NINP-1)))
      B2(NINP)=B2(NINP-1)+X(NINP)*(2.0D0*C2(NINP-1)
     1        +X(NINP)*3.0D0*D2(NINP-1))
      A2(NINP)=A2(NINP)-B2(NINP)*X(NINP)
      C2(NINP)=0.0D0
      D2(NINP)=0.0D0
      DO 15 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINP1,K)
      SPCI=DEXP(A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K))))
      SPRI=DEXP(A2(K)+XE*(B2(K)+XE*(C2(K)+XE*D2(K))))
C  ****  INELASTIC COLLISIONS.
      E=DEXP(XE)/HREV
      STP=SPCI/(HREV*FCMTU)
      CALL PEINEL(ZZT,F,WI,OP2,0.0D0,E,CS0,CS1,CS2,NOS,IELEC)
      TL=CS1
      TU=CS2
      FF1=STP/CS1
      CALL PEINEL(ZZT,F,WI,OP2,TDRH,E,CS0,CS1,CS2,NOS,IELEC)
      S3I=FCCMI*FF1*CS0
      W1I=FCCMI*HREV*FF1*(TL-CS1)
      W2I=FCCMI*HREV*HREV*FF1*(TU-CS2)
C  ****  BREMSSTRAHLUNG EMISSION.
      IF(THB(m).GE.10.0D0) THEN
        STP=SPRI/(HREV*FCMTUE)
        CALL PEBREM(ZZEQ,THBH,E,S0D,S1D,S2D,S1T,S2T,IELEC)
        FF2=STP/S1T
        S4I=FCCMIE*FF2*S0D
        W1I=FCCMIE*HREV*FF2*(S1T-S1D)+W1I
        W2I=FCCMIE*HREV*HREV*FF2*(S2T-S2D)+W2I
      ELSE
        S4I=0.0D0
      ENDIF
      DEL(M,I)=DLOG(DMAX1(DELTA,1.0D-35))
      IF(KPAR.EQ.1) THEN
        SINE(M,I)=DLOG(DMAX1(S3I,1.0D-35))
        SBRE(M,I)=DLOG(DMAX1(S4I,1.0D-35))
        W1E(M,I)=DLOG(DMAX1(W1I,1.0D-35))
        W2E(M,I)=DLOG(DMAX1(W2I,1.0D-35))
      ELSE
        SINP(M,I)=DLOG(DMAX1(S3I,1.0D-35))
        SBRP(M,I)=DLOG(DMAX1(S4I,1.0D-35))
        W1P(M,I)=DLOG(DMAX1(W1I,1.0D-35))
        W2P(M,I)=DLOG(DMAX1(W2I,1.0D-35))
      ENDIF
   15 CONTINUE
C
      CALL PEBREM(ZZEQ,1.0D0,1.0D5,S0D,S1D,S2D,S1T,S2T,IELEC)
      F0(M,1)=CF0(1)
      F0(M,2)=CF0(2)
      F0(M,3)=CF0(3)
      BCB(M)=CBCB
C
C  ****  ELASTIC SCATTERING.
C
      DO 16 I=1,NINP
      FPEI=1.0D6*RHO(M)/FPE(I)
      TFP1I=1.0D6*RHO(M)/TFP1(I)
      TFP2I=1.0D6*RHO(M)/TFP2(I)
C  ****  CONTRIBUTION OF SOFT INELASTIC COLLISIONS.
      E=ET(I)/HREV
      STP=SPC(I)/(HREV*FCMTU)
      CALL PEINEL(ZZT,F,WI,OP2,0.0D0,E,CS0,CS1,CS2,NOS,IELEC)
      FF1=STP/CS1
      CALL PESCIN(F,WI,TDRH,E,CS0,CS1,CS2,NOS,IELEC)
      FPEI=FPEI+FF1*CS0*FCCMI
      TFP1I=TFP1I+FF1*CS1*FCCMI
      TFP2I=TFP2I+FF1*CS2*FCCMI
      FPE(I)=1.0D6*RHO(M)/FPEI
      TFP1(I)=1.0D6*RHO(M)/TFP1I
      TFP2(I)=1.0D6*RHO(M)/TFP2I
C
      A2(I)=1.0D0/FPEI
      B2(I)=1.0D0/TFP1I
      C2(I)=1.0D0/TFP2I
      D2(I)=(ET(I)/(SPC(I)+SPR(I)))/(1.0D6*RHO(M))
   16 CONTINUE
C
      DO 17 I=1,NINP
      Y1(I)=DLOG(A2(I))
   17 CONTINUE
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      A1(NINP)=A1(NINP-1)+X(NINP)*(B1(NINP-1)+X(NINP)*(C1(NINP-1)
     1        +X(NINP)*D1(NINP-1)))
      B1(NINP)=B1(NINP-1)+X(NINP)*(2.0D0*C1(NINP-1)
     1        +X(NINP)*3.0D0*D1(NINP-1))
      A1(NINP)=A1(NINP)-B1(NINP)*X(NINP)
      C1(NINP)=0.0D0
      D1(NINP)=0.0D0
      DO 18 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINP1,K)
      A2(I)=DEXP(A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K))))
   18 CONTINUE
C
      DO 19 I=1,NINP
      Y1(I)=DLOG(B2(I))
   19 CONTINUE
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      A1(NINP)=A1(NINP-1)+X(NINP)*(B1(NINP-1)+X(NINP)*(C1(NINP-1)
     1        +X(NINP)*D1(NINP-1)))
      B1(NINP)=B1(NINP-1)+X(NINP)*(2.0D0*C1(NINP-1)
     1        +X(NINP)*3.0D0*D1(NINP-1))
      A1(NINP)=A1(NINP)-B1(NINP)*X(NINP)
      C1(NINP)=0.0D0
      D1(NINP)=0.0D0
      DO 20 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINP1,K)
      B2(I)=DEXP(A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K))))
   20 CONTINUE
C
      DO 21 I=1,NINP
      Y1(I)=DLOG(C2(I))
   21 CONTINUE
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      A1(NINP)=A1(NINP-1)+X(NINP)*(B1(NINP-1)+X(NINP)*(C1(NINP-1)
     1        +X(NINP)*D1(NINP-1)))
      B1(NINP)=B1(NINP-1)+X(NINP)*(2.0D0*C1(NINP-1)
     1        +X(NINP)*3.0D0*D1(NINP-1))
      A1(NINP)=A1(NINP)-B1(NINP)*X(NINP)
      C1(NINP)=0.0D0
      D1(NINP)=0.0D0
      DO 22 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINP1,K)
      C2(I)=DEXP(A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K))))
   22 CONTINUE
C
      DO 23 I=1,NINP
      Y1(I)=DLOG(D2(I))
   23 CONTINUE
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINP)
      A1(NINP)=A1(NINP-1)+X(NINP)*(B1(NINP-1)+X(NINP)*(C1(NINP-1)
     1        +X(NINP)*D1(NINP-1)))
      B1(NINP)=B1(NINP-1)+X(NINP)*(2.0D0*C1(NINP-1)
     1        +X(NINP)*3.0D0*D1(NINP-1))
      A1(NINP)=A1(NINP)-B1(NINP)*X(NINP)
      C1(NINP)=0.0D0
      D1(NINP)=0.0D0
      DO 24 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINP1,K)
      D2(I)=DEXP(A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K))))
   24 CONTINUE
C
      DO 25 I=1,NMP
      FPEI=A2(I)
      TFP1I=B2(I)
      TFP2I=C2(I)
      EOSI=D2(I)
      CALL ELINIT(FPEI,TFP1I,TFP2I,AI,BI)
      HEFPI=DMAX1(FPEI,DMIN1(CC1*TFP1I,CC2*EOSI))
C
      IF(FPEI.GT.DMIN1(CC1*TFP1I,CC2*EOSI)) THEN
        ICUTOF(I)=1
      ELSE IF(CC1*TFP1I.LT.CC2*EOSI) THEN
        ICUTOF(I)=2
      ELSE
        ICUTOF(I)=3
      ENDIF
      IF(HEFPI.LT.0.5D0*FPEI) ICUTOF(I)=1
C
      IF(KPAR.EQ.1) THEN
        SELE(M,I)=DLOG(1.0D0/FPEI)
        SHEE(M,I)=DLOG(1.0D0/HEFPI)
        AELE(M,I)=DLOG(AI)
        BELE(M,I)=BI
      ELSE
        SELP(M,I)=DLOG(1.0D0/FPEI)
        SHEP(M,I)=DLOG(1.0D0/HEFPI)
        AELP(M,I)=DLOG(AI)
        BELP(M,I)=BI
      ENDIF
   25 CONTINUE
C
C  ****  POSITRON ANNIHILATION (HEITLER CROSS SECTION).
C
      IF(KPAR.EQ.3) THEN
        SGA=FCCMIE*ZZEQ*PI/SL2**2
        DO 26 I=1,NMP
        E=DEXP(DLEMP(I))
        GAM=1.0D0+(E+1.0D0)/REV
        GAM2=GAM*GAM
        F2=GAM2-1.0D0
        F1=DSQRT(F2)
        SAN(M,I)=DLOG(SGA*((GAM2+4.0D0*GAM+1.0D0)*DLOG(GAM+F1)/F2
     1          -(GAM+3.0D0)/F1)/(GAM+1.0D0))
   26 CONTINUE
      ENDIF
C
C  ****  Total interaction probabilities per unit path length.
C
      IF(KPAR.EQ.1) THEN
        DO K=1,NMP
          PP1=DEXP(SELE(M,K))
          PP2=DEXP(SHEE(M,K))
          IF(PP2.GE.0.5D0*PP1) PP2=PP1
          IF(E.GT.TDR(m)) THEN
            PP3=DEXP(SINE(M,K))
          ELSE
            PP3=0.0D0
          ENDIF
          IF(E.GT.THB(m).AND.THB(m).GE.10.0D0) THEN
            PP4=DEXP(SBRE(M,K))
          ELSE
            PP4=0.0D0
          ENDIF
          PTOTE(M,K)=DLOG(PP2+PP3+PP4)
        ENDDO
      ELSE IF(KPAR.EQ.3) THEN
        DO K=1,NMP
          PP1=DEXP(SELE(M,K))
          PP2=DEXP(SHEE(M,K))
          IF(PP2.GE.0.5D0*PP1) PP2=PP1
          IF(E.GT.TDR(m)) THEN
            PP3=DEXP(SINE(M,K))
          ELSE
            PP3=0.0D0
          ENDIF
          IF(E.GT.THB(m).AND.THB(m).GE.10.0D0) THEN
            PP4=DEXP(SBRE(M,K))
          ELSE
            PP4=0.0D0
          ENDIF
          PP5=DEXP(SAN(M,K))
          PTOTP(M,K)=DLOG(PP2+PP3+PP4+PP5)
        ENDDO
      ENDIF
C
C  ************  Print look-up tables if IWR.gt.1.
C
      IF(IWR.GT.1) THEN
      WRITE(6,5000)
 5000 FORMAT(//3X,'***  LOOK-UP TABLES OF ENERGY-DEPENDENT',
     1  ' QUANTITIES.')
        IF(KPAR.EQ.1) THEN
          WRITE(6,5001)
 5001     FORMAT(/4X,'ENERGY',5X,'HEL MFP',7X,'HIN MFP',4X,
     1      'HBR MFP',3X,'HARD MFP',3X,'SOFT STP',3X,'SOFT STR')
          WRITE(6,5002)
 5002     FORMAT(5X,'(eV)',7X,'(cm)',10X,'(cm)',7X,'(cm)',7X,
     2      '(cm)',6X,'(eV/cm)',3X,'(eV^2/cm)',/3X,78('-'))
 5003     FORMAT(1X,1P,2E11.3,1X,A2,8E11.3)
          DO K=1,NMP
            EE=DEXP(DLEMP(K))
            FP1=1.0D0/DEXP(SHEE(M,K))
            FP2=1.0D0/DEXP(SINE(M,K))
            FP3=1.0D0/DEXP(SBRE(M,K))
            FP4=1.0D0/DEXP(PTOTE(M,K))
            STP1=DEXP(W1E(M,K))
            STP2=DEXP(W2E(M,K))
            WRITE(6,5003) EE,FP1,LCUTEL(ICUTOF(K)),
     1          FP2,FP3,FP4,STP1,STP2
          ENDDO
        ELSE IF(KPAR.EQ.3) THEN
          WRITE(6,5004)
 5004     FORMAT(/4X,'ENERGY',5X,'HEL MFP',7X,'HIN MFP',4X,
     1      'HBR MFP',4X,'ANN MFP',3X,'HARD MFP',3X,
     2      'SOFT STP',3X,'SOFT STR')
          WRITE(6,5005)
 5005     FORMAT(5X,'(eV)',7X,'(cm)',10X,'(cm)',7X,'(cm)',7X,
     1      '(cm)',7X,'(cm)',6X,'(eV/cm)',3X,'(eV^2/cm)',
     2      /3X,89('-'))
          DO K=1,NMP
            EE=DEXP(DLEMP(K))
            FP1=1.0D0/DEXP(SHEP(M,K))
            FP2=1.0D0/DEXP(SINP(M,K))
            FP3=1.0D0/DEXP(SBRP(M,K))
            FP4=1.0D0/DEXP(SAN(M,K))
            FP5=1.0D0/DEXP(PTOTP(M,K))
            STP1=DEXP(W1P(M,K))
            STP2=DEXP(W2P(M,K))
            WRITE(6,5003) EE,FP1,LCUTEL(ICUTOF(K)),
     1               FP2,FP3,FP4,FP5,STP1,STP2
          ENDDO
        ENDIF
      ENDIF
C
C  ************  INTERPOLATION TEST.
C
      IF(IWR.GT.2) THEN
      WRITE(6,2012)
 2012 FORMAT(//3X,'***  PERCENTAGE ERRORS IN THE INTERPOLATED',
     1  ' FUNCTIONS.')
      WRITE(6,2013)
 2013 FORMAT(/4X,'ENERGY',5X,'EL MFP',3X,'1st TMFP',2X,
     1  '2nd TMFP',2X,'STOPPING',2X,'MAXIMUM'/3X,59('-'))
      ENDIF
C
      TST=0.0D0
      DO 27 I=1,NINP
      E=ET(I)
      XE=1.0D0+(DLOG(E)-DLEMP1)*DLFC
      K=XE
      IF(K.LT.1.OR.K.GE.NMP) GO TO 27
      XEK=XE-K
      IF(KPAR.EQ.1) THEN
        WL=DEXP(W1E(M,K)+(W1E(M,K+1)-W1E(M,K))*XEK)
        TA=DEXP(AELE(M,K)+(AELE(M,K+1)-AELE(M,K))*XEK)
        TB=BELE(M,K)+(BELE(M,K+1)-BELE(M,K))*XEK
        P2=DEXP(SELE(M,K)+(SELE(M,K+1)-SELE(M,K))*XEK)
        P3=DEXP(SINE(M,K)+(SINE(M,K+1)-SINE(M,K))*XEK)
        P4=DEXP(SBRE(M,K)+(SBRE(M,K+1)-SBRE(M,K))*XEK)
      ELSE
        WL=DEXP(W1P(M,K)+(W1P(M,K+1)-W1P(M,K))*XEK)
        TA=DEXP(AELP(M,K)+(AELP(M,K+1)-AELP(M,K))*XEK)
        TB=BELP(M,K)+(BELP(M,K+1)-BELP(M,K))*XEK
        P2=DEXP(SELP(M,K)+(SELP(M,K+1)-SELP(M,K))*XEK)
        P3=DEXP(SINP(M,K)+(SINP(M,K+1)-SINP(M,K))*XEK)
        P4=DEXP(SBRP(M,K)+(SBRP(M,K+1)-SBRP(M,K))*XEK)
      ENDIF
C
      FPEI=1.0D6*RHO(M)/P2
      T0=100.0D0*DABS(FPEI/FPE(I)-1.0D0)
C
      IF(TB.GT.0.0D0) THEN
        TBB=TB
        TC=TA*(TA+1.0D0)*DLOG((1.0D0+TA)/TA)-TA
      ELSE
        TBB=-TB
        TC=0.8D0
      ENDIF
      RMU1W=TA*(TA+1.0D0)*DLOG((1.0D0+TA)/TA)-TA
      RMU2W=TA*(1.0D0-2.0D0*RMU1W)
      RMU1=(1.0D0-TBB)*RMU1W+TBB*TC
      RMU2=(1.0D0-TBB)*RMU2W+TBB*TC*TC
      TFP1I=FPEI/(2.0D0*RMU1)
      T1=100.0D0*DABS(TFP1I/TFP1(I)-1.0D0)
      TFP2I=FPEI/(6.0D0*(RMU1-RMU2))
      T2=100.0D0*DABS(TFP2I/TFP2(I)-1.0D0)
C
      STPI=WL/(1.0D6*RHO(M))
      ERH=E/HREV
      CALL PEINEL(ZZT,F,WI,OP2,TDRH,ERH,CS0,CS1,CS2,NOS,IELEC)
      IF(CS0.GT.0.0D0) STPI=STPI+P3*(CS1/CS0)*HREV/(1.0D6*RHO(M))
      IF(THB(m).GE.10.0D0) THEN
        CALL PEBREM(ZZEQ,THBH,ERH,S0D,S1D,S2D,S1T,S2T,IELEC)
        IF(S0D.GT.0.0D0)
     1    STPI=STPI+P4*(S1D/S0D)*HREV/(1.0D6*RHO(M))
        T3=100.0D0*DABS(STPI/(SPC(I)+SPR(I))-1.0D0)
      ELSE
        T3=100.0D0*DABS(STPI/SPC(I)-1.0D0)
      ENDIF
C
      TST=DMAX1(TST,T0,T1,T2,T3)
C  ****  INTERPOLATION TEST PRINTED IF IWR.gt.2.
      IF(IWR.GT.2) WRITE(6,2014) ET(I),T0,T1,T2,T3,TST
 2014 FORMAT(2X,1P,E10.3,6E10.2)
   27 CONTINUE
C
      IF(IWR.GT.2.OR.TST.GT.1.0D0) WRITE(6,2015) TST
 2015 FORMAT(/3X,'MAXIMUM INTERPOLATION ERROR = ',1P,E10.3,
     1  ' PER CENT.')
      IF(TST.GT.1.0D0) THEN
        WRITE(6,2016)
 2016   FORMAT(/3X,'*** WARNING: INTERPOLATION ERRORS LARGER',
     1  ' THAN 1.0 PER CENT.')
      ENDIF
C
      IF(KPAR.EQ.1) THEN
        KPAR=3
        GO TO 1
      ENDIF
C
C  ************  PHOTON CROSS SECTIONS.
C
      KPAR=2
C
C  ****  PRINTED OUTPUT IF IWR.gt.0.
C
      IF(IWR.GT.0) THEN
      WRITE(6,4001)
 4001 FORMAT(//3X,'******  PHOTON ATTENUATION COEFFICIENTS  ***')
      WRITE(6,4002)
 4002 FORMAT(/4X,'ENERGY',4X,'RAYLEIGH',4X,'COMPTON',3X,
     1  'PHOTOELEC.',3X,'PAIR',7X,'TOTAL')
      WRITE(6,4003)
 4003 FORMAT(5X,'(eV)',5X,'(cm2/g)',5X,'(cm2/g)',4X,'(cm2/g)',4X,
     2  '(cm2/g)',5X,'(1/cm)'/3X,64('-'))
      ENDIF
C
      READ(5,3001) NINP
 3001 FORMAT(31X,I3)
C
      IF(NINP.GT.NMS-1.OR.NINP.LT.4) THEN
        WRITE(6,4004) NINP
 4004   FORMAT(/3X,'NUMBER OF INPUT ENERGIES = ',I4,'. STOP.')
        STOP
      ENDIF
C
      DO 101 I=1,NINP
      READ(5,*) ET(I),FPE(I),TFP1(I),TFP2(I),SPC(I)
      IF(SPC(I).LT.1.0D-35) SPC(I)=1.0D-35
      ATCOEF=(FPE(I)+TFP1(I)+TFP2(I)+SPC(I))*RHO(M)
      IF(IWR.GT.0) WRITE(6,4005)
     1  ET(I),FPE(I),TFP1(I),TFP2(I),SPC(I),ATCOEF
      EPHT(M,I)=DLOG(ET(I))
      SPHT(M,I)=DLOG(TFP2(I)*RHO(M))
      SPHT(M,I)=DMAX1(SPHT(M,I),-80.59048D0)
  101 CONTINUE
 4005 FORMAT(1X,1P,7E11.3)
      NINP1=NINP+1
      ET(NINP1)=DMAX1(EU,2.0D0*ET(NINP))
C
C  ************  INVERSE MEAN FREE PATHS.
C
C  ****  RAYLEIGH SCATTERING.
C
      NINQ=1
      IEDGE=1
      X(1)=EPHT(M,1)
      Y1(1)=DLOG(FPE(1)*RHO(M))
      Y2(1)=DLOG(TFP1(1)*RHO(M))
      DO 102 I=2,NINP
      IF(ET(I).GT.ET(I-1)+1.0D0) THEN
        NINQ=NINQ+1
        X(NINQ)=EPHT(M,I)
        Y1(NINQ)=DLOG(FPE(I)*RHO(M))
        Y2(NINQ)=DLOG(TFP1(I)*RHO(M))
      ELSE
        IEDGE=I+1
      ENDIF
  102 CONTINUE
      IF(IEDGE.GT.NINP) IEDGE=NINP
      NINQ1=NINQ+1
      X(NINQ1)=DLOG(ET(NINP1))
C
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINQ)
      A1(NINQ)=A1(NINQ-1)+X(NINQ)*(B1(NINQ-1)+X(NINQ)*(C1(NINQ-1)
     1        +X(NINQ)*D1(NINQ-1)))
      B1(NINQ)=B1(NINQ-1)+X(NINQ)*(2.0D0*C1(NINQ-1)
     1        +X(NINQ)*3.0D0*D1(NINQ-1))
      A1(NINQ)=A1(NINQ)-B1(NINQ)*X(NINQ)
      C1(NINQ)=0.0D0
      D1(NINQ)=0.0D0
C
      DO 103 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINQ1,K)
      SRA(M,I)=A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K)))
      SRA(M,I)=DMAX1(SRA(M,I),-80.59048D0)
  103 CONTINUE
C
C  ****  COMPTON SCATTERING.
C
      CALL SPLINE(X,Y2,A1,B1,C1,D1,0.0D0,0.0D0,NINQ)
      A1(NINQ)=A1(NINQ-1)+X(NINQ)*(B1(NINQ-1)+X(NINQ)*(C1(NINQ-1)
     1        +X(NINQ)*D1(NINQ-1)))
      B1(NINQ)=B1(NINQ-1)+X(NINQ)*(2.0D0*C1(NINQ-1)
     1        +X(NINQ)*3.0D0*D1(NINQ-1))
      A1(NINQ)=A1(NINQ)-B1(NINQ)*X(NINQ)
      C1(NINQ)=0.0D0
      D1(NINQ)=0.0D0
C
      DO 104 I=1,NMP
      XE=DLEMP(I)
      CALL FINDI(X,XE,NINQ1,K)
      SCO(M,I)=A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K)))
      SCO(M,I)=DMAX1(SCO(M,I),-80.59048D0)
  104 CONTINUE
C
C  ****  PHOTOELECTRIC EFFECT.
C
      NINQ=0
      DO 105 I=IEDGE,NINP
      NINQ=NINQ+1
      X(NINQ)=EPHT(M,I)
      Y1(NINQ)=SPHT(M,I)
  105 CONTINUE
      IF(NINQ.LT.4) STOP
      NINQ1=NINQ+1
      X(NINQ1)=DLOG(ET(NINP1))
C
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINQ)
      A1(NINQ)=A1(NINQ-1)+X(NINQ)*(B1(NINQ-1)+X(NINQ)*(C1(NINQ-1)
     1        +X(NINQ)*D1(NINQ-1)))
      B1(NINQ)=B1(NINQ-1)+X(NINQ)*(2.0D0*C1(NINQ-1)
     1        +X(NINQ)*3.0D0*D1(NINQ-1))
      A1(NINQ)=A1(NINQ)-B1(NINQ)*X(NINQ)
      C1(NINQ)=0.0D0
      D1(NINQ)=0.0D0
C
      XECUT=EPHT(M,IEDGE)
      INTPH=NMP
      DO 107 I=1,NMP
      XEL=DLEMP(I)
      IF(XEL.LT.XECUT) THEN
        II=1
        II1=IEDGE
  106   IIT=(II+II1)/2
        IF(XEL.GT.EPHT(M,IIT)) THEN
          II=IIT
        ELSE
          II1=IIT
        ENDIF
        IF(II1-II.GT.1) GO TO 106
        DEE=EPHT(M,II+1)-EPHT(M,II)
        IF(DEE.GT.1.0D-15) THEN
          SPH(M,I)=SPHT(M,II)+(SPHT(M,II+1)-SPHT(M,II))
     1            *(XEL-EPHT(M,II))/DEE
        ELSE
          SPH(M,I)=SPHT(M,II)
        ENDIF
      ELSE
        CALL FINDI(X,XEL,NINQ1,K)
        SPH(M,I)=A1(K)+XEL*(B1(K)+XEL*(C1(K)+XEL*D1(K)))
        SPH(M,I)=DMAX1(SPH(M,I),-80.59048D0)
        IF(INTPH.EQ.NMP) INTPH=I
      ENDIF
  107 CONTINUE
      INTPH0(M)=INTPH
      XECUT=DLEMP(INTPH)
      DO 108 I=1,NINP
      IF(EPHT(M,I).LE.XECUT) GO TO 108
      MPH0(M)=I+2
      GO TO 109
  108 CONTINUE
      MPH0(M)=NINP
  109 CONTINUE
      IF(MPH0(M).GT.NINP) MPH0(M)=NINP
C
C  ****  ELECTRON-POSITRON PAIR PRODUCTION.
C
      NINQ=0
      DO 110 I=1,NINP
      IF(ET(I).LT.1.023D6) GO TO 110
      NINQ=NINQ+1
      X(NINQ)=EPHT(M,I)
      FACT=(ET(I)/(ET(I)-1.022D6))**3
      Y1(NINQ)=DLOG(FACT*SPC(I)*RHO(M))
  110 CONTINUE
      IF(NINQ.LT.4) STOP
      NINQ1=NINQ+1
      X(NINQ1)=DLOG(ET(NINP1))
C
      CALL SPLINE(X,Y1,A1,B1,C1,D1,0.0D0,0.0D0,NINQ)
      A1(NINQ)=A1(NINQ-1)+X(NINQ)*(B1(NINQ-1)+X(NINQ)*(C1(NINQ-1)
     1        +X(NINQ)*D1(NINQ-1)))
      B1(NINQ)=B1(NINQ-1)+X(NINQ)*(2.0D0*C1(NINQ-1)
     1        +X(NINQ)*3.0D0*D1(NINQ-1))
      A1(NINQ)=A1(NINQ)-B1(NINQ)*X(NINQ)
      C1(NINQ)=0.0D0
      D1(NINQ)=0.0D0
C
      DO 111 I=1,NMP
      XE=DLEMP(I)
      EE=DEXP(XE)
      IF(EE.LT.1.023D6) THEN
        SPA(M,I)=-80.59048D0
      ELSE
        CALL FINDI(X,XE,NINQ1,K)
        FACT=(EE/(EE-1.022D6))**3
        SPA(M,I)=A1(K)+XE*(B1(K)+XE*(C1(K)+XE*D1(K)))-DLOG(FACT)
        SPA(M,I)=DMAX1(SPA(M,I),-80.59048D0)
      ENDIF
  111 CONTINUE
C
      E=EU*0.9999D0
      CALL JUMP(1.0D30,DUMMY)
      ATCOEF=ST
      RANGE(KPAR,M)=5.0D0/ATCOEF
      WRITE(6,4006) RANGE(KPAR,M)
 4006 FORMAT(/3X,'PHOTON RANGE (E=EMAX, 5 MEAN FREE PATHS) = ',
     1  1P,E11.4,' cm')
C
C  ************  INTERPOLATION TEST.
C
      IF(IWR.GT.2) THEN
      WRITE(6,4007)
 4007 FORMAT(//3X,'***  PERCENTAGE ERRORS IN THE INTERPOLATED',
     1  ' FUNCTIONS.')
      WRITE(6,4008)
 4008 FORMAT(/4X,'ENERGY',4X,'RAYLEIGH',2X,'COMPTON',3X,
     1  'PHOTOEL.',3X,'PAIR',5X,'MAXIMUM'/3X,59('-'))
      ENDIF
C
      TST=0.0D0
      DO 112 I=1,NINP
      E=ET(I)
      IF(E.LT.EL.OR.E.GT.EU*0.9999D0) GO TO 112
      IF(I.LT.NINP) THEN
        IF(E.GT.ET(I+1)-1.0D-2) E=ET(I)-1.0D-5
      ENDIF
      IF(I.GT.1) THEN
        IF(E.LT.ET(I-1)+1.0D-2) E=ET(I)+1.0D-5
      ENDIF
      MODE=0
      CALL JUMP(1.0D30,DUMMY)
      RACS=P(1)/RHO(M)
      COCS=P(2)/RHO(M)
      PHCS=P(3)/RHO(M)
      PACS=P(4)/RHO(M)
      T1=100.0D0*DABS(RACS/FPE(I)-1.0D0)
      T2=100.0D0*DABS(COCS/TFP1(I)-1.0D0)
      T3=100.0D0*DABS(PHCS/TFP2(I)-1.0D0)
      IF(E.GT.7.5D5) THEN
        T3P=0.1D0*T3
      ELSE
        T3P=T3
      ENDIF
      IF(E.GT.1.023D6) THEN
        T4=100.0D0*DABS(PACS/SPC(I)-1.0D0)
      ELSE
        T4=0.0D0
      ENDIF
      IF(E.GT.4.0D6) THEN
        T4P=T4
      ELSE
        T4P=0.0D0
      ENDIF
      TST=DMAX1(TST,T1,T2,T3P,T4P)
C  ****  INTERPOLATION TEST PRINTED IF IWR.gt.2.
      IF(IWR.GT.2) WRITE(6,4009) ET(I),T1,T2,T3,T4,TST
 4009 FORMAT(2X,1P,E10.3,6E10.2)
  112 CONTINUE
C
      IF(IWR.GT.2.OR.TST.GT.1.0D0) WRITE(6,4010) TST
 4010 FORMAT(/3X,'MAXIMUM INTERPOLATION ERROR = ',1P,E10.3,
     1  ' PER CENT.')
      IF(TST.GT.1.0D0) THEN
        WRITE(6,4011)
 4011   FORMAT(/3X,'*** WARNING: INTERPOLATION ERRORS LARGER',
     1  ' THAN 1.0 PER CENT.')
      ENDIF
C
      CALL SCOHE0(M)
C
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PESCIN
C  **************************************************************
      SUBROUTINE PESCIN(F,WI,WC,E,CS0,CS1,CS2,NOS,IELEC)
C
C     RESTRICTED CROSS SECTION FOR SCATTERING OF ELECTRONS AND
C  POSITRONS BY ATOMIC ELECTRONS, COMPUTED FROM THE STERNHEIMER-
C  LILJEQUIST GOS (ONLY COLLISIONS WITH ENERGY LOSS LESS THAN WC
C  CONTRIBUTE TO THE CROSS SECTIONS).
C
C     NOTICE THAT THE DENSITY EFFECT CORRECTION, DELTA, IS ENTE-
C  RED THROUGH COMMON/COLIN/. THIS QUANTITY IS COMPUTED BY SUB-
C  ROUTINE PEINEL, WHICH MUST BE CALLED IMMEDIATELY BEFORE CALL-
C  ING PESCIN.
C
C  INPUT ARGUMENTS:
C     F(I) .... OSCILLATOR STRENGTHS,
C     W(I) .... EXCITATION ENERGIES,
C     WC ...... CUTOFF ENERGY LOSS,
C     E ....... ELECTRON OR POSITRON KINETIC ENERGY,
C     NOS ..... NUMBER OF OSCILLATORS,
C     IELEC ... -1 FOR ELECTRONS, +1 FOR POSITRONS.
C
C  OUTPUT ARGUMENTS: CROSS SECTIONS (PER ATOM OR MOLECULE) FOR
C  ENERGY LOSSES LESS THAN WC.
C     CS0 ..... TOTAL CROSS SECTION,
C     CS1 ..... FIRST TRANSPORT CROSS SECTION,
C     CS2 ..... SECOND TRANSPORT CROSS SECTION.
C
C                               (ALL QUANTITIES IN ATOMIC UNITS).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0)
      PARAMETER (SL=137.0359895D0, SL2=SL*SL, TSL2=2.0D0*SL2)
      PARAMETER (NO=30)
      DIMENSION F(NO),WI(NO)
      COMMON/COLIN/DELTA,CSD(NO),CSC(NO)
      CS0=0.0D0
      CS1=0.0D0
      CS2=0.0D0
C
      WMAXD=DMIN1(E,WC)
      IF(IELEC.EQ.-1) THEN
        WMAXC=DMIN1(0.5D0*E,WC)
      ELSE
        WMAXC=WMAXD
      ENDIF
C  ****  CONSTANTS.
      GAM=1.0D0+E/SL2
      GAM2=GAM*GAM
      BETA2=(GAM2-1.0D0)/GAM2
      CPA=DSQRT(E*(E+TSL2))
      CONSD=2.0D0*PI/(SL2*BETA2)
      CONSC=PI/(CPA*CPA*BETA2)
      CSDT0=DMAX1(DLOG(GAM2)-BETA2-DELTA,0.0D0)
C
      DO 1 I=1,NOS
      W=WI(I)
      IF(WMAXD.LT.W) GO TO 1
C  ****  DISTANT COLLISIONS.
      CPP=DSQRT((E-W)*(E-W+TSL2))
      A=4.0D0*CPA*CPP
      B=(CPA-CPP)**2
      RMU1=(W*(W+TSL2)-B)/A
      CS0D=DLOG((A*RMU1+B)/B)
      CS1D=RMU1-(B/A)*CS0D
      CS2D=CS0D*(B/A)**2+(RMU1/(2.0D0*A))*(A*RMU1-2.0D0*B)
      FDELTA=(CS0D+CSDT0)/CS0D
C  ****  CLOSE COLLISIONS.
      IF(WMAXC.GT.W) THEN
        IF(E.GT.WMAXC+1.0D-5) THEN
          CPM=DSQRT((E-WMAXC)*(E-WMAXC+TSL2))
          RMU2=(WMAXC*(WMAXC+TSL2)-(CPA-CPM)**2)/(4.0D0*CPA*CPM)
        ELSE
          RMU2=0.5D0
        ENDIF
        CS0C=(1.0D0/RMU1)-(1.0D0/RMU2)
        CS1C=DLOG(RMU2/RMU1)
        CS2C=RMU2-RMU1
      ELSE
        CS0C=0.0D0
        CS1C=0.0D0
        CS2C=0.0D0
      ENDIF
C
      CS0=CS0+F(I)*((CONSD/W)*CS0D*FDELTA+CONSC*CS0C)
      CS1=CS1+F(I)*((CONSD/W)*CS1D*FDELTA+CONSC*CS1C)
      CS2=CS2+F(I)*((CONSD/W)*CS2D*FDELTA+CONSC*CS2C)
    1 CONTINUE
C
      CS2=6.0D0*(CS1-CS2)
      CS1=2.0D0*CS1
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE ELINIT
C  **************************************************************
      SUBROUTINE ELINIT(FP,TFP1,TFP2,A,B)
C
C     EVALUATION OF THE ANGULAR DISTRIBUTION PARAMETERS FOR SIN-
C  GLE ELASTIC SCATTERING. W2D MODEL.
C
C  INPUT ARGUMENTS:
C     FP ........ ELASTIC MEAN FREE PATH,
C     TFP1 ...... FIRST TRANSPORT MEAN FREE PATH,
C     TFP2 ...... SECOND TRANSPORT MEAN FREE PATH.
C
C  OUTPUT VALUES:
C     A, B ...... ANGULAR DISTRIBUTION PARAMETERS.
C
C                      ALL LENGTHS IN THE SAME (ARBITRARY) UNITS.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      IF(FP.LE.0.0D0.OR.TFP1.LE.0.0D0) THEN
        WRITE(6,1001) FP,TFP1,TFP2
 1001 FORMAT(/3X,'*** THE ARGUMENTS IN SUBROUTINE ELINIT ARE IN',
     1  'CONSISTENT.'/7X,'FP = ',1P,E12.5,', TFP1 = ',E12.5,
     2  ', TFP2 = ',E12.5)
        STOP
      ENDIF
      RMU1=FP/(2.0D0*TFP1)
      RMU2=RMU1-FP/(6.0D0*TFP2)
      IF(RMU1.LT.0.0D0.OR.RMU1.GT.0.5D0) THEN
        WRITE(6,1001) FP,TFP1,TFP2
        STOP
      ENDIF
C
      A=RMU1*RMU1
    1 SAVE=DLOG((1.0D0+A)/A)
      FA=A*(A+1.0D0)*SAVE-A-RMU1
      FAP=(1.0D0+A+A)*SAVE-2.0D0
      AO=A
      A=AO-FA/FAP
      IF(DABS(AO-A).GT.1.0D-11*A) GO TO 1
      IF(RMU2.LT.RMU1*RMU1) THEN
        B=1.0D0
        RETURN
      ENDIF
      RMU1W=A*(A+1.0D0)*DLOG((1.0D0+A)/A)-A
      RMU2W=A*(1.0D0-2.0D0*RMU1W)
      IF(RMU2W.GT.RMU2) THEN
        B=(RMU2W-RMU2)/(RMU2W-RMU1W*RMU1W)
        RETURN
      ENDIF
C
      C=0.8D0
      F0=(C*RMU1-RMU2)/(C-RMU1)
      AL=1.0D-24
      AU=A
    2 A=0.5D0*(AL+AU)
      RMU1W=A*(A+1.0D0)*DLOG((1.0D0+A)/A)-A
      RMU2W=A*(1.0D0-2.0D0*RMU1W)
      F=(C*RMU1W-RMU2W)/(C-RMU1W)
      IF(F.LT.F0) THEN
        AL=A
      ELSE
        AU=A
      ENDIF
      IF(AU-AL.GT.1.0D-14*A) GO TO 2
      B=(RMU1W-RMU1)/(C-RMU1W)
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SCOHE0
C  **************************************************************
      SUBROUTINE SCOHE0(M)
C
C  INITIALIZATION OF RANDOM SAMPLING FOR COHERENT SCATTERING.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10)
      COMMON/COMP/ZAT(MAXMAT,15),STF(MAXMAT,15),ZT(MAXMAT),
     1 AT(MAXMAT),ZEQ(MAXMAT),AEQ(MAXMAT),RHO(MAXMAT),
     2 EXPOT(MAXMAT),NELEM(MAXMAT)
      COMMON/CFORM/A1(95),A2(95),A3(95),A4(95),A5(95)
      COMMON/XCOH/X2COH(MAXMAT,241),PDCOH(MAXMAT,241),FLCOH
      COMMON/SCOIN0/SI(15),A1F(15),A2F(15),A3F(15),A4F(15),
     1 A5F(15),IZZ(15),NZ
      EXTERNAL DCOH
C
      NZ=NELEM(M)
      DO 1 I=1,NZ
      IZ=ZAT(M,I)+0.5D0
      IZZ(I)=IZ
      SI(I)=STF(M,I)
      A1F(I)=A1(IZ)
      A2F(I)=A2(IZ)
      A3F(I)=A3(IZ)
      A4F(I)=A4(IZ)
      A5F(I)=A5(IZ)
    1 CONTINUE
C
      SUM=0.0D0
      XL=0.0D0
      XU=1.0D-4
      FACT=(1.0D6/XU)**(1.0D0/240.0D0)
      CALL GABQ(DCOH,XL,XU,SUM,1.0D-12,IER)
      X2COH(M,1)=XU
      PDCOH(M,1)=SUM
      DO 2 I=2,241
      XL=XU
      XU=XL*FACT
      CALL GABQ(DCOH,XL,XU,SUM,1.0D-12,IER)
      X2COH(M,I)=XU
      PDCOH(M,I)=PDCOH(M,I-1)+SUM
    2 CONTINUE
      DO 3 I=1,241
      X2COH(M,I)=DLOG(X2COH(M,I))
      PDCOH(M,I)=DLOG(PDCOH(M,I))
    3 CONTINUE
      FLCOH=DLOG(FACT)
      RETURN
      END
C  **************************************************************
      DOUBLE PRECISION FUNCTION DCOH(X2)
C
C     SQUARED MOLECULAR FORM FACTOR (ADDITIVITY RULE).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (SL=137.0359895D0)
      COMMON/SCOIN0/SI(15),A1F(15),A2F(15),A3F(15),A4F(15),
     1 A5F(15),IZZ(15),NZ
      X=DSQRT(X2)
      DCOH=0.0D0
      DO 1 I=1,NZ
      IZ=IZZ(I)
C  ****  ATOMIC FORM FACTORS.
      FA=(1.0D0+X2*(A1F(I)+X*(A2F(I)+X*A3F(I))))
     1  /(1.0D0+X2*(A4F(I)+X2*A5F(I)))**2
      IF(IZ.GT.10.AND.FA*IZ.LT.2.0D0) THEN
        PA=(IZ-0.3125D0)/SL
        PG=DSQRT(1.0D0-PA*PA)
        PQ=2.426311D-2*X/PA
        FB=DSIN(2.0D0*PG*DATAN(PQ))/(IZ*PG*PQ*(1.0D0+PQ*PQ)**PG)
        FA=DMAX1(FA,FB)
      ENDIF
C
      DCOH=DCOH+SI(I)*(IZ*FA)**2
    1 CONTINUE
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE CLEANS
C  **************************************************************
      SUBROUTINE CLEANS
C
C     THIS SUBROUTINE INITIALIZES THE SECONDARY STACK. IT MUST BE
C  CALLED BEFORE STARTING THE SIMULATION OF EACH PRIMARY TRACK.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NMS=400)
      COMMON/SECST/ES(NMS),XS(NMS),YS(NMS),ZS(NMS),US(NMS),
     1   VS(NMS),WS(NMS),WGHTS(NMS),KS(NMS),IBODYS(NMS),MS(NMS),
     2   ILBS(NMS),NSEC
      NSEC=0
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE START
C  **************************************************************
      SUBROUTINE START
C
C     THIS SUBROUTINE FORCES THE NEXT EVENT TO BE AN ARTIFICIAL
C  SOFT EVENT. IT MUST BE CALLED WHEN A NEW (PRIMARY OR SECOND-
C  ARY) ELECTRON OR POSITRON TRACK IS STARTED AND WHEN IT CROSSES
C  AN INTERFACE.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10, NMP=200)
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
      COMMON/JMODES/MODE,MODEE,MODEI,IEXCD
C
      IF(E.LT.1.000001D0*EL.OR.E.GT.0.999999D0*EU) THEN
        WRITE(6,1000) KPAR,E,EL,EU
 1000   FORMAT(/3X,'*** ENERGY OUT OF RANGE. KPAR = ',I2,
     1  ', E = ',1P,E12.5,' eV'/7X,'EMIN = ',E12.5,' eV, ',
     2  'EMAX = ',E12.5,' eV.'/7X,'CHECK THE VALUES OF',
     3  ' EABS(KPAR,m) AND EMAX.')
        STOP
      ENDIF
      IF(KPAR.EQ.2.AND.E.LT.1.0D3) THEN
        WRITE(6,1001) E
 1001   FORMAT(/3X,'*** ENERGY OUT OF RANGE. KPAR = ',I2,
     1  ', E = ',1P,E12.5,' eV.'/7X,'PHOTONS WITH ENERGY LESS',
     2  ' THAN 1 keV CANNOT BE SIMULATED.'/7X,'CHECK THE',
     3  ' VALUE OF EABS(2,m).')
        STOP
      ENDIF
      MODE=0
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE JUMP
C  **************************************************************
      SUBROUTINE JUMP(DSMAX,DS)
C
C     CALCULATION OF THE FREE PATH FROM THE STARTING POINT TO THE
C  POSITION OF THE NEXT EVENT AND OF THE PROBABILITIES OF OCCUR-
C  RENCE OF DIFFERENT EVENTS.
C
C  Arguments:
C    DSMAX ... maximum allowed step length (input),
C    DS ...... step length (output).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10, NMP=200)
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
      COMMON/DELE/SHEE(MAXMAT,NMP),SELE(MAXMAT,NMP),
     1 SINE(MAXMAT,NMP),SBRE(MAXMAT,NMP),AELE(MAXMAT,NMP),
     2 BELE(MAXMAT,NMP),W1E(MAXMAT,NMP),W2E(MAXMAT,NMP),
     3 DEL(MAXMAT,NMP),PTOTE(MAXMAT,NMP),ISOFT(maxmat)
      COMMON/DPOS/SHEP(MAXMAT,NMP),SELP(MAXMAT,NMP),
     1 SINP(MAXMAT,NMP),SBRP(MAXMAT,NMP),SAN(MAXMAT,NMP),
     2 AELP(MAXMAT,NMP),BELP(MAXMAT,NMP),W1P(MAXMAT,NMP),
     3 W2P(MAXMAT,NMP),PTOTP(MAXMAT,NMP)
      COMMON/JMODES/MODE,MODEE,MODEI,IEXCD
      COMMON/JUMCOL/P(5),ST,DST,DS1,W1,W2,XEL,XE,XEK,K
C  ****  MAIN-PENELOPE COMMONS.
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
      COMMON/simpar/EABS(3,MAXMAT),C1(MAXMAT),C2(MAXMAT),
     1              WCC(MAXMAT),WCR(MAXMAT)
      EXTERNAL RAND
C
      IF(KPAR.EQ.2) THEN
        GO TO 3
      ELSE IF(KPAR.EQ.3) THEN
        GO TO 2
      ENDIF
C
C  ************  ELECTRONS (KPAR=1).
C
    1 CONTINUE
      IF(MODE.EQ.1) THEN
        DS=DS1
        CALL IMFPSE
        RETURN
      ENDIF
C  ****  Energy index.
      XE=1.0D0+(DLOG(E)-DLEMP1)*DLFC
      K=XE
      XEK=XE-K
C  ****  Inverse hard mean free path (interaction probability per
C        unit path length).
      ST=DEXP(PTOTE(M,K)+(PTOTE(M,K+1)-PTOTE(M,K))*XEK)
      DSMAXP=DSMAX
      ISRAND=0
C
C  ****  Soft stopping interactions.
C     MODEI=1, soft stopping is active,
C     MODEI=0, soft stopping is not active.
C
      IF(ISOFT(m).EQ.1) THEN
        W1=DEXP(W1E(M,K)+(W1E(M,K+1)-W1E(M,K))*XEK)
        IF(W1.GT.1.0D-15) THEN
          W2=DEXP(W2E(M,K)+(W2E(M,K+1)-W2E(M,K))*XEK)
          MODEI=1
C  ****  The maximum step length, DSMAXP, is tentatively set
C  equal to 3 mean free paths. If the input DSMAX is less than
C  this value, DSMAXP is randomized to eliminate dose artifacts
C  at the end of the first step.
          DSMC=3.0D0/ST
          IF(DSMAXP.GT.DSMC) THEN
            DSMAXP=DSMC
          ELSE IF(DSMAXP.GT.1.0D-10) THEN
             DSMAXP=0.5D0*(RAND(1.0D0)+RAND(2.0D0))*DSMAXP
          ELSE
C  ****  The external maximum step length control is switched off
C  when the input value of DSMAX is less than 1.0D-10.
            DSMAXP=DSMC
          ENDIF
          ISRAND=1
C  ****  Upper bound for the interaction probability along the
C        step (including artificial energy straggling).
          DEMAX=DSMAXP*W1
          W21=W2/W1
          IF(DEMAX.GT.3.0D0*W21) THEN
            ELW=DMAX1(E-(DEMAX+DSQRT(3.0D0*W2*DSMAXP)),EABS(1,M))
          ELSE
            ELW=DMAX1(E-1.501D0*(DEMAX+W21),EABS(1,M))
          ENDIF
          XE1=1.0D0+(DLOG(ELW)-DLEMP1)*DLFC
          K1=XE1
          XEK1=XE1-K1
          STLW=DEXP(PTOTE(M,K1)+(PTOTE(M,K1+1)-PTOTE(M,K1))*XEK1)
          ST=DMAX1(ST,STLW)
        ELSE
          MODEI=0
        ENDIF
      ELSE
        MODEI=0
      ENDIF
C
C  ****  Soft elastic scattering.
C     MODEE=1, soft scattering is active,
C     MODEE=0, soft scattering is not active.
C
      P(1)=DEXP(SELE(M,K)+(SELE(M,K+1)-SELE(M,K))*XEK)
      P(2)=DEXP(SHEE(M,K)+(SHEE(M,K+1)-SHEE(M,K))*XEK)
      IF(P(2).GE.0.5D0*P(1)) THEN
        P(2)=P(1)
        MODEE=0
      ELSE
        MODEE=1
        IF(ISRAND.EQ.0.AND.DSMAXP.LT.1.0D20) THEN
C  ****  The maximum step length, DSMAXP, is tentatively set
C  equal to 3 mean free paths. If the input DSMAX is less than
C  this value, DSMAXP is randomized to eliminate dose artifacts
C  at the end of the first step.
          DSMC=3.0D0/ST
          IF(DSMAXP.GT.DSMC) THEN
            DSMAXP=DSMC
          ELSE IF(DSMAXP.GT.1.0D-10) THEN
            DSMAXP=0.5D0*(RAND(1.0D0)+RAND(2.0D0))*DSMAXP
          ELSE
            DSMAXP=DSMC
          ENDIF
        ENDIF
      ENDIF
C
      DST=-DLOG(RAND(3.0D0))/ST
C  ****  IEXCD=0, hard interaction,
C  ****  IEXCD=1, delta interaction,
      IF(DST.LT.DSMAXP) THEN
        IEXCD=0
      ELSE
        DST=DSMAXP
        IEXCD=1
      ENDIF
C
      IF(MODEE+MODEI.EQ.0) THEN
        DS=DST
        DS1=0.0D0
        MODE=1
        CALL IMFPSE
      ELSE
        DS=DST*RAND(4.0D0)
        DS1=DST-DS
      ENDIF
      RETURN
C
C  ************  POSITRONS (KPAR=3).
C
    2 CONTINUE
      IF(MODE.EQ.1) THEN
        DS=DS1
        CALL IMFPSP
        RETURN
      ENDIF
C  ****  Energy index.
      XE=1.0D0+(DLOG(E)-DLEMP1)*DLFC
      K=XE
      XEK=XE-K
C  ****  Inverse hard mean free path (interaction probability per
C        unit path length).
      ST=DEXP(PTOTP(M,K)+(PTOTP(M,K+1)-PTOTP(M,K))*XEK)
      DSMAXP=DSMAX
      ISRAND=0
C
C  ****  Soft stopping interactions.
C     MODEI=1, soft stopping is active,
C     MODEI=0, soft stopping is not active.
C
      IF(ISOFT(m).EQ.1) THEN
        W1=DEXP(W1P(M,K)+(W1P(M,K+1)-W1P(M,K))*XEK)
        IF(W1.GT.1.0D-15) THEN
          W2=DEXP(W2P(M,K)+(W2P(M,K+1)-W2P(M,K))*XEK)
          MODEI=1
C  ****  The maximum step length, DSMAXP, is tentatively set
C  equal to 3 mean free paths. If the input DSMAX is less than
C  this value, DSMAXP is randomized to eliminate dose artifacts
C  at the end of the first step.
          DSMC=3.0D0/ST
          IF(DSMAXP.GT.DSMC) THEN
            DSMAXP=DSMC
          ELSE IF(DSMAXP.GT.1.0D-10) THEN
             DSMAXP=0.5D0*(RAND(1.0D0)+RAND(2.0D0))*DSMAXP
          ELSE
            DSMAXP=DSMC
          ENDIF
          ISRAND=1
C  ****  Upper bound for the interaction probability along the
C        step (including artificial energy straggling).
          DEMAX=DSMAXP*W1
          W21=W2/W1
          IF(DEMAX.GT.3.0D0*W21) THEN
            ELW=DMAX1(E-(DEMAX+DSQRT(3.0D0*W2*DSMAXP)),EABS(3,M))
          ELSE
            ELW=DMAX1(E-1.501D0*(DEMAX+W21),EABS(3,M))
          ENDIF
          XE1=1.0D0+(DLOG(ELW)-DLEMP1)*DLFC
          K1=XE1
          XEK1=XE1-K1
          STLW=DEXP(PTOTP(M,K1)+(PTOTP(M,K1+1)-PTOTP(M,K1))*XEK1)
          ST=DMAX1(ST,STLW)
        ELSE
          MODEI=0
        ENDIF
      ELSE
        MODEI=0
      ENDIF
C
C  ****  Soft elastic scattering.
C     MODEE=1, soft scattering is active,
C     MODEE=0, soft scattering is not active.
C
      P(1)=DEXP(SELP(M,K)+(SELP(M,K+1)-SELP(M,K))*XEK)
      P(2)=DEXP(SHEP(M,K)+(SHEP(M,K+1)-SHEP(M,K))*XEK)
      IF(P(2).GE.0.5D0*P(1)) THEN
        P(2)=P(1)
        MODEE=0
      ELSE
        MODEE=1
        IF(ISRAND.EQ.0.AND.DSMAXP.LT.1.0D20) THEN
C  ****  The maximum step length, DSMAXP, is tentatively set
C  equal to 3 mean free paths. If the input DSMAX is less than
C  this value, DSMAXP is randomized to eliminate dose artifacts
C  at the end of the first step.
          DSMC=3.0D0/ST
          IF(DSMAXP.GT.DSMC) THEN
            DSMAXP=DSMC
          ELSE IF(DSMAXP.GT.1.0D-10) THEN
            DSMAXP=0.5D0*(RAND(1.0D0)+RAND(2.0D0))*DSMAXP
          ELSE
            DSMAXP=DSMC
          ENDIF
        ENDIF
      ENDIF
C
      DST=-DLOG(RAND(3.0D0))/ST
      IF(DST.LT.DSMAXP) THEN
        IEXCD=0
      ELSE
        DST=DSMAXP
        IEXCD=1
      ENDIF
C
      IF(MODEE+MODEI.EQ.0) THEN
        DS=DST
        DS1=0.0D0
        MODE=1
        CALL IMFPSP
      ELSE
        DS=DST*RAND(4.0D0)
        DS1=DST-DS
      ENDIF
      RETURN
C
C  ************  PHOTONS (KPAR=2).
C
    3 CONTINUE
C  ****  Energy index.
      XEL=DLOG(E)
      XE=1.0D0+(XEL-DLEMP1)*DLFC
      K=XE
      XEK=XE-K
C
      CALL IMFPSG
      ST=P(1)+P(2)+P(3)+P(4)
      DS=-DLOG(RAND(1.0D0))/ST
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE KNOCK
C  **************************************************************
      SUBROUTINE KNOCK(DE,ICOL)
C
C     Simulation of random hinges and hard interaction events.
C
C  Output arguments:
C    DE ..... energy deposited by the particle in the material.
C             It is equal to the difference between the energies
C             before and after the interaction.
C    ICOL ... kind of interaction suffered by the particle.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (REV=5.1099906D5, TREV=2.0D0*REV)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      PARAMETER (MAXMAT=10, NO=30, NMP=200)
      DIMENSION PHOT(15)
C  ****  ENERGY MESH POINTS.
      COMMON/COMP/ZAT(MAXMAT,15),STF(MAXMAT,15),ZT(MAXMAT),
     1 AT(MAXMAT),ZEQ(MAXMAT),AEQ(MAXMAT),RHO(MAXMAT),
     2 EXPOT(MAXMAT),NELEM(MAXMAT)
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
      COMMON/DELE/SHEE(MAXMAT,NMP),SELE(MAXMAT,NMP),
     1 SINE(MAXMAT,NMP),SBRE(MAXMAT,NMP),AELE(MAXMAT,NMP),
     2 BELE(MAXMAT,NMP),W1E(MAXMAT,NMP),W2E(MAXMAT,NMP),
     3 DEL(MAXMAT,NMP),PTOTE(MAXMAT,NMP),ISOFT(maxmat)
      COMMON/DPOS/SHEP(MAXMAT,NMP),SELP(MAXMAT,NMP),
     1 SINP(MAXMAT,NMP),SBRP(MAXMAT,NMP),SAN(MAXMAT,NMP),
     2 AELP(MAXMAT,NMP),BELP(MAXMAT,NMP),W1P(MAXMAT,NMP),
     3 W2P(MAXMAT,NMP),PTOTP(MAXMAT,NMP)
      COMMON/CBIND/IB(25,92),NOC(25)
      COMMON/CRELX/PRK1(92),PRK2(92),PRK3(92),PRK4(92),PRK5(92),
     1       PRK6(92),PRK7(92)
      COMMON/XINCIA/FJ0(MAXMAT,NO),IZFL(MAXMAT,NO)
      COMMON/JMODES/MODE,MODEE,MODEI,IEXCD
      COMMON/JUMCOL/P(5),ST,DST,DS1,W1,W2,XEL,XE,XEK,K
C  ****  MAIN-PENELOPE COMMONS.
      COMMON/simpar/EABS(3,MAXMAT),C1(MAXMAT),C2(MAXMAT),
     1              WCC(MAXMAT),WCR(MAXMAT)
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C
      EXTERNAL RAND
C
      IF(KPAR.EQ.2) GO TO 2000
C
C  ************  ELECTRONS (KPAR=1) AND POSITRONS (KPAR=3).
C
 1000 CONTINUE
      IF(MODE.EQ.1) GO TO 1100
C
C  ****  ARTIFICIAL SOFT EVENT (ICOL=1).
C
      ICOL=1
      MODE=1
C
      IF(MODEI.EQ.0) THEN
        DE=0.0D0
      ELSE
        EDE=W1*DST
        VDE=W2*DST
C  ****  GENERATION OF RANDOM VALUES DE DISTRIBUTED FROM 0 TO
C        INFINITY WITH MEAN EDE AND VARIANCE VDE.
        RU=RAND(1.0D0)
        EDE2=EDE*EDE
        VDE3=3.0D0*VDE
        IF(EDE2.LT.VDE3) THEN
          PNULL=(VDE3-EDE2)/(VDE3+3.0D0*EDE2)
          IF(RU.LT.PNULL) THEN
            DE=0.0D0
          ELSE
            DE=1.5D0*(EDE+VDE/EDE)*(RU-PNULL)/(1.0D0-PNULL)
          ENDIF
        ELSE
          DE=EDE+(2.0D0*RU-1.0D0)*DSQRT(VDE3)
        ENDIF
      ENDIF
C
      E=E-DE
      IF(E.LT.EABS(KPAR,m)) THEN
        DE=E+DE
        E=0.0D0
        IF(KPAR.EQ.1) RETURN
C
C  ****  SLOW POSITRONS ANNIHILATE IN SITU.
C
        DE=DE+TREV
        IF(REV.GT.EABS(2,m)) THEN
          WS=-1.0D0+2.0D0*RAND(3.0D0)
          SDTS=DSQRT(1.0D0-WS*WS)
          DF=TWOPI*RAND(4.0D0)
          US=DCOS(DF)*SDTS
          VS=DSIN(DF)*SDTS
          CALL STORES(REV,US,VS,WS,WGHT,2)
          US=-US
          VS=-VS
          WS=-WS
          CALL STORES(REV,US,VS,WS,WGHT,2)
        ENDIF
        RETURN
      ENDIF
C
C  ****  Bielajew's randomly alternate hinge.
C
      IF(RAND(2.0D0).GT.0.5D0.AND.DE.GT.1.0D-10) THEN
        XE=1.0D0+(DLOG(E)-DLEMP1)*DLFC
        K=XE
        XEK=XE-K
        IF(KPAR.EQ.1) THEN
          TA=DEXP(AELE(M,K)+(AELE(M,K+1)-AELE(M,K))*XEK)
          TB=BELE(M,K)+(BELE(M,K+1)-BELE(M,K))*XEK
          P(1)=DEXP(SELE(M,K)+(SELE(M,K+1)-SELE(M,K))*XEK)
          P(2)=DEXP(SHEE(M,K)+(SHEE(M,K+1)-SHEE(M,K))*XEK)
        ELSE
          TA=DEXP(AELP(M,K)+(AELP(M,K+1)-AELP(M,K))*XEK)
          TB=BELP(M,K)+(BELP(M,K+1)-BELP(M,K))*XEK
          P(1)=DEXP(SELP(M,K)+(SELP(M,K+1)-SELP(M,K))*XEK)
          P(2)=DEXP(SHEP(M,K)+(SHEP(M,K+1)-SHEP(M,K))*XEK)
        ENDIF
        IF(P(2).LT.0.5D0*P(1)) THEN
          FPE=1.0D0/P(1)
          FPEH=1.0D0/P(2)
          CALL ELSOFT(FPE,FPEH,TA,TB,DST,U,V,W)
        ENDIF
      ELSE
        IF(MODEE.EQ.1) THEN
          IF(KPAR.EQ.1) THEN
            TA=DEXP(AELE(M,K)+(AELE(M,K+1)-AELE(M,K))*XEK)
            TB=BELE(M,K)+(BELE(M,K+1)-BELE(M,K))*XEK
          ELSE
            TA=DEXP(AELP(M,K)+(AELP(M,K+1)-AELP(M,K))*XEK)
            TB=BELP(M,K)+(BELP(M,K+1)-BELP(M,K))*XEK
          ENDIF
          FPE=1.0D0/P(1)
          FPEH=1.0D0/P(2)
          CALL ELSOFT(FPE,FPEH,TA,TB,DST,U,V,W)
        ENDIF
        IF(DE.GT.1.0D-10) THEN
          XE=1.0D0+(DLOG(E)-DLEMP1)*DLFC
          K=XE
          XEK=XE-K
        ENDIF
      ENDIF
      RETURN
C
C  ************  HARD EVENT.
C
 1100 CONTINUE
      MODE=0
C  ****  A delta interaction (ICOL=6) occurs when the maximum
C        allowed step length is exceeded.
      IF(IEXCD.EQ.1) THEN
        ICOL=6
        DE=0.0D0
        RETURN
      ENDIF
C  ****  Interaction probabilities.
      IF(KPAR.EQ.1) THEN
        STNOW=P(2)+P(3)+P(4)
      ELSE
        STNOW=P(2)+P(3)+P(4)+P(5)
      ENDIF
C  ****  Random sampling of the interaction type.
      STS=DMAX1(STNOW,ST)*RAND(5.0D0)
      SS=P(2)
      IF(SS.GT.STS) GO TO 1200
      SS=SS+P(3)
      IF(SS.GT.STS) GO TO 1300
      SS=SS+P(4)
      IF(SS.GT.STS) GO TO 1400
      IF(KPAR.EQ.3) THEN
        SS=SS+P(5)
        IF(SS.GT.STS) GO TO 1500
      ENDIF
C  ****  A delta interaction (ICOL=6) may occur when the total
C        interaction probability per unit pathlength, ST, is
C        larger than STNOW.
      ICOL=6
      DE=0.0D0
      RETURN
C
C  ****  HARD ELASTIC COLLISION (ICOL=2).
C
 1200 ICOL=2
      IF(KPAR.EQ.1) THEN
        TA=DEXP(AELE(M,K)+(AELE(M,K+1)-AELE(M,K))*XEK)
        TB=BELE(M,K)+(BELE(M,K+1)-BELE(M,K))*XEK
      ELSE
        TA=DEXP(AELP(M,K)+(AELP(M,K+1)-AELP(M,K))*XEK)
        TB=BELP(M,K)+(BELP(M,K+1)-BELP(M,K))*XEK
      ENDIF
      FPE=1.0D0/P(1)
      IF(MODEE.EQ.1) THEN
        FPEH=DMAX1(FPE,1.0D0/P(2))
      ELSE
        FPEH=FPE
      ENDIF
      CALL ELHARD(FPE,FPEH,TA,TB,CDT)
      DF=TWOPI*RAND(6.0D0)
      CALL DIRECT(CDT,DF,U,V,W)
      DE=0.0D0
      RETURN
C
C  ****  HARD INELASTIC COLLISION (ICOL=3).
C
 1300 ICOL=3
      DELTA=DEXP(DEL(M,K)+(DEL(M,K+1)-DEL(M,K))*XEK)
      tdrarg=TDR(m)
      CALL SHINEL(E,tdrarg,DELTA,DE,EP,CDT,ES,CDTS,M,KPAR)
C  ****  SCATTERING ANGLES (PRIMARY PARTICLE).
      DF=TWOPI*RAND(7.0D0)
C  ****  DELTA RAY.
      IF(ES.GT.EABS(1,m)) THEN
        DFS=DF+PI
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDTS,DFS,US,VS,WS)
        CALL STORES(ES,US,VS,WS,WGHT,1)
      ENDIF
C  ****  NEW ENERGY AND DIRECTION.
      IF(EP.GT.EABS(KPAR,m)) THEN
        E=EP
        CALL DIRECT(CDT,DF,U,V,W)
        RETURN
      ENDIF
      DE=E
      E=0.0D0
      IF(KPAR.EQ.1) RETURN
      DE=DE+TREV
      GO TO 2500
C
C  ****  HARD BREMSSTRAHLUNG EMISSION (ICOL=4).
C
 1400 ICOL=4
      thbarg=THB(m)
      CALL SBREMS(E,thbarg,DE,M)
C  ****  BREMSSTRAHLUNG PHOTON.
      IF(DE.GT.EABS(2,m)) THEN
        BETA=DSQRT(E*(E+TREV))/(E+REV)
        CDTS=2.0D0*RAND(8.0D0)-1.0D0
        IF(RAND(9.0D0).GT.0.75D0) THEN
          IF(CDTS.LT.0.0D0) THEN
            CDTS=-(-CDTS)**0.333333333333333D0
          ELSE
            CDTS=CDTS**0.333333333333333D0
          ENDIF
        ENDIF
        CDTS=(CDTS+BETA)/(1.0D0+BETA*CDTS)
        DFS=TWOPI*RAND(10.0D0)
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDTS,DFS,US,VS,WS)
        CALL STORES(DE,US,VS,WS,WGHT,2)
      ENDIF
C  ****  NEW ENERGY.
      E=E-DE
      IF(E.GT.EABS(KPAR,m)) RETURN
      DE=E+DE
      E=0.0D0
      IF(KPAR.EQ.1) RETURN
      DE=DE+TREV
      GO TO 2500
C
C  ****  POSITRON ANNIHILATION IN FLIGHT (ICOL=5).
C
 1500 ICOL=5
      DE=E+TREV
C  ****  SECONDARY PHOTONS.
      GAM=1.0D0+(E+1.0D0)/REV
      GAM21=DSQRT(GAM*GAM-1.0D0)
      ANI=1.0D0+GAM
      TAUMIN=1.0D0/(ANI+GAM21)
      TAUMAX=1.0D0-TAUMIN
      GT0=ANI*ANI+2.0D0*(GAM-ANI)
 1501 CONTINUE
      TAU=TAUMIN*(TAUMAX/TAUMIN)**RAND(11.0D0)
      GTAU=ANI*ANI*(1.0D0-TAU)+GAM+GAM-1.0D0/TAU
      IF(RAND(12.0D0)*GT0.GT.GTAU) GO TO 1501
      IF(TAU.LT.0.5D0) THEN
        TAUH=1.0D0-TAU
      ELSE
        TAUH=TAU
        TAU=1.0D0-TAU
      ENDIF
C
      DF=TWOPI*RAND(13.0D0)
      ES=TAU*DE
      IF(ES.GT.EABS(2,m)) THEN
        CDTS=(ANI-1.0D0/TAU)/GAM21
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDTS,DF,US,VS,WS)
        CALL STORES(ES,US,VS,WS,WGHT,2)
      ENDIF
C
      ES=TAUH*DE
      IF(ES.GT.EABS(2,m)) THEN
        DFS=DF+PI
        CDTS=(ANI-1.0D0/TAUH)/GAM21
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDTS,DFS,US,VS,WS)
        CALL STORES(ES,US,VS,WS,WGHT,2)
      ENDIF
C
      E=0.0D0
      RETURN
C
C  ************  PHOTONS (KPAR=2).
C
 2000 CONTINUE
C
      STS=ST*RAND(14.0D0)
      SS=P(2)
      IF(SS.GT.STS) GO TO 2200
      SS=SS+P(3)
      IF(SS.GT.STS) GO TO 2300
      SS=SS+P(4)
      IF(SS.GT.STS) GO TO 2400
C
C  ****  RAYLEIGH SCATTERING (ICOL=1).
C
      ICOL=1
      DE=0.0D0
      CALL SCOHER(E,CDT,M)
      DF=TWOPI*RAND(15.0D0)
      CALL DIRECT(CDT,DF,U,V,W)
      RETURN
C
C  ****  COMPTON SCATTERING (ICOL=2).
C
 2200 ICOL=2
      CALL SINCOH(E,EP,CDT,ISH,M)
      DF=TWOPI*RAND(16.0D0)
      DE=E-EP
      ES=DE
      IFLUOR=0
      IZ=IZFL(M,ISH)
      IF(IZ.GT.0) THEN
        EKS=IB(1,IZ)
        IF(EKS.GT.EABS(1,m).OR.EKS.GT.EABS(2,m)) THEN
          IFLUOR=1
          ES=DE-EKS
        ENDIF
      ENDIF
C  ****  COMPTON ELECTRON.
      IF(ES.GT.EABS(1,m)) THEN
        Q2=E*E+EP*(EP-2.0D0*E*CDT)
        IF(Q2.GT.1.0D-12) THEN
           CDTS=(E-EP*CDT)/DSQRT(Q2)
        ELSE
           CDTS=1.0D0
        ENDIF
        DFS=DF+PI
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDTS,DFS,US,VS,WS)
        CALL STORES(ES,US,VS,WS,WGHT,1)
      ENDIF
C  ****  NEW DIRECTION AND ENERGY.
      IF(EP.LT.EABS(2,m)) THEN
        DE=E
        E=0.0D0
      ELSE
        CALL DIRECT(CDT,DF,U,V,W)
        E=EP
      ENDIF
      IF(IFLUOR.EQ.1) GO TO 2350
      RETURN
C
C  ****  PHOTOELECTRIC ABSORPTION (ICOL=3).
C
 2300 ICOL=3
      NNELEM=NELEM(M)
      SUMPH=0.0D0
      DO 2301 I=1,NNELEM
      IZ=ZAT(M,I)+0.5D0
      CALL PEPHOT(IZ,E,CS0)
      PHOT(I)=STF(M,I)*CS0
      SUMPH=SUMPH+PHOT(I)
 2301 CONTINUE
C
      STS=SUMPH*RAND(17.0D0)
      SS=0.0D0
      DO 2302 I=1,NNELEM
      SS=SS+PHOT(I)
      IF(SS.LT.STS) GO TO 2302
      J=I
      GO TO 2303
 2302 CONTINUE
      J=NNELEM
 2303 CONTINUE
C
      ES=E
      IFLUOR=0
      IZ=ZAT(M,J)
      EKS=IB(1,IZ)
      IF((IZ.GT.6).AND.(E.GT.EKS)) THEN
        PK=(1.0D0+IZ*(8.76D-2-IZ*7.35D-4))
     1    /(9.65D-1+IZ*(1.07D-1-IZ*8.39D-4))
        IF(RAND(18.0D0).LT.PK) THEN
          IF(EKS.GT.EABS(1,m).OR.EKS.GT.EABS(2,m)) THEN
            IFLUOR=1
            ES=E-EKS
          ENDIF
        ENDIF
      ENDIF
C  ****  SECONDARY PHOTOELECTRON.
      IF(ES.GT.EABS(1,m)) THEN
        CALL SAUTER(ES,CDTS)
        DFS=TWOPI*RAND(19.0D0)
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDTS,DFS,US,VS,WS)
        CALL STORES(ES,US,VS,WS,WGHT,1)
      ENDIF
C
      DE=E
      E=0.0D0
      IF(IFLUOR.EQ.0) RETURN
C
C  ****  K-SHELL FLUORESCENT RADIATION.
C
 2350 RU=RAND(20.0D0)
      TST=PRK1(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-IB(3,IZ)
        GO TO 2351
      ENDIF
      TST=TST+PRK2(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-IB(4,IZ)
        GO TO 2351
      ENDIF
      TST=TST+PRK3(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-IB(6,IZ)
        GO TO 2351
      ENDIF
      TST=TST+PRK4(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-IB(7,IZ)
        GO TO 2351
      ENDIF
      TST=TST+PRK5(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-IB(11,IZ)
        GO TO 2351
      ENDIF
      TST=TST+PRK6(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-IB(12,IZ)
        GO TO 2351
      ENDIF
      TST=TST+PRK7(IZ)
      IF(RU.LT.TST) THEN
        KPARS=2
        ES=EKS-5.0D0
        GO TO 2351
      ENDIF
      KPARS=1
      ES=EKS-IB(4,IZ)
C
 2351 CONTINUE
      IF(ES.GT.EABS(KPARS,m)) THEN
        WS=2.0D0*RAND(21.0D0)-1.0D0
        DF=TWOPI*RAND(22.0D0)
        FNT=DSQRT(1.0D0-WS*WS)
        US=FNT*DCOS(DF)
        VS=FNT*DSIN(DF)
        CALL STORES(ES,US,VS,WS,WGHT,KPARS)
      ENDIF
      RETURN
C
C  ****  ELECTRON-POSITRON PAIR PRODUCTION (ICOL=4).
C
 2400 ICOL=4
      ZZ=ZEQ(M)
      CALL SPAIRP(E,ZZ,EPS,M)
      DE=E
C  ****  ELECTRON.
      ES=EPS*E-REV
      IF(ES.GT.EABS(1,m)) THEN
        CDT=2.0D0*RAND(23.0D0)-1.0D0
        A1=ES+REV
        A2=DSQRT(ES*(ES+TREV))
        CDT=(CDT*A1+A2)/(A1+CDT*A2)
        DF=TWOPI*RAND(24.0D0)
        US=U
        VS=V
        WS=W
        CALL DIRECT(CDT,DF,US,VS,WS)
        CALL STORES(ES,US,VS,WS,WGHT,1)
      ENDIF
C  ****  POSITRON.
      ES=(1.0D0-EPS)*E-REV
      IF(ES.GT.EABS(3,m)) THEN
        CDT=2.0D0*RAND(25.0D0)-1.0D0
        A1=ES+REV
        A2=DSQRT(ES*(ES+TREV))
        CDT=(CDT*A1+A2)/(A1+CDT*A2)
        DF=TWOPI*RAND(26.0D0)
        CALL DIRECT(CDT,DF,U,V,W)
        CALL STORES(ES,U,V,W,WGHT,3)
C  ****  THE POSITRON CARRIES A 'LATENT' ENERGY OF 1022 KEV.
        DE=DE-TREV
        E=0.0D0
        RETURN
      ENDIF
      E=0.0D0
C
C  ****  SLOW POSITRONS ANNIHILATE IN SITU.
C
 2500 CONTINUE
      IF(REV.GT.EABS(2,m)) THEN
        WS=-1.0D0+2.0D0*RAND(27.0D0)
        SDTS=DSQRT(1.0D0-WS*WS)
        DF=TWOPI*RAND(28.0D0)
        US=DCOS(DF)*SDTS
        VS=DSIN(DF)*SDTS
        CALL STORES(REV,US,VS,WS,WGHT,2)
        US=-US
        VS=-VS
        WS=-WS
        CALL STORES(REV,US,VS,WS,WGHT,2)
      ENDIF
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE IMFPSE
C  **************************************************************
      SUBROUTINE IMFPSE
C
C     This subroutine computes inverse mean free paths of hard
C  interactions for electrons with the current energy.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10, NMP=200)
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
      COMMON/DELE/SHEE(MAXMAT,NMP),SELE(MAXMAT,NMP),
     1 SINE(MAXMAT,NMP),SBRE(MAXMAT,NMP),AELE(MAXMAT,NMP),
     2 BELE(MAXMAT,NMP),W1E(MAXMAT,NMP),W2E(MAXMAT,NMP),
     3 DEL(MAXMAT,NMP),PTOTE(MAXMAT,NMP),ISOFT(maxmat)
      COMMON/JUMCOL/P(5),ST,DST,DS1,W1,W2,XEL,XE,XEK,K
C  ****  MAIN-PENELOPE COMMON.
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C
      P(1)=DEXP(SELE(M,K)+(SELE(M,K+1)-SELE(M,K))*XEK)
      P(2)=DEXP(SHEE(M,K)+(SHEE(M,K+1)-SHEE(M,K))*XEK)
      IF(P(2).GE.0.5D0*P(1)) P(2)=P(1)
      IF(E.GT.TDR(m)) THEN
        P(3)=DEXP(SINE(M,K)+(SINE(M,K+1)-SINE(M,K))*XEK)
      ELSE
        P(3)=0.0D0
      ENDIF
      IF(E.GT.THB(m)) THEN
        P(4)=DEXP(SBRE(M,K)+(SBRE(M,K+1)-SBRE(M,K))*XEK)
      ELSE
        P(4)=0.0D0
      ENDIF
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE IMFPSP
C  **************************************************************
      SUBROUTINE IMFPSP
C
C     This subroutine computes inverse mean free paths of hard
C  interactions for positrons with the current energy.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10, NMP=200)
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
      COMMON/DPOS/SHEP(MAXMAT,NMP),SELP(MAXMAT,NMP),
     1 SINP(MAXMAT,NMP),SBRP(MAXMAT,NMP),SAN(MAXMAT,NMP),
     2 AELP(MAXMAT,NMP),BELP(MAXMAT,NMP),W1P(MAXMAT,NMP),
     3 W2P(MAXMAT,NMP),PTOTP(MAXMAT,NMP)
      COMMON/JUMCOL/P(5),ST,DST,DS1,W1,W2,XEL,XE,XEK,K
C  ****  MAIN-PENELOPE COMMON.
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C
      P(1)=DEXP(SELP(M,K)+(SELP(M,K+1)-SELP(M,K))*XEK)
      P(2)=DEXP(SHEP(M,K)+(SHEP(M,K+1)-SHEP(M,K))*XEK)
      IF(P(2).GE.0.5D0*P(1)) P(2)=P(1)
      IF(E.GT.TDR(m)) THEN
        P(3)=DEXP(SINP(M,K)+(SINP(M,K+1)-SINP(M,K))*XEK)
      ELSE
        P(3)=0.0D0
      ENDIF
      IF(E.GT.THB(m)) THEN
        P(4)=DEXP(SBRP(M,K)+(SBRP(M,K+1)-SBRP(M,K))*XEK)
      ELSE
        P(4)=0.0D0
      ENDIF
      P(5)=DEXP(SAN(M,K)+(SAN(M,K+1)-SAN(M,K))*XEK)
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE IMFPSG
C  **************************************************************
      SUBROUTINE IMFPSG
C
C     This subroutine computes inverse mean free paths of hard
C  interactions for photons (gammas) with the current energy.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10, NMP=200, NMS=400)
      COMMON/EMESH/EL,EU,TDR(maxmat),THB(maxmat),DLFC,DLEMP1,
     1             DLEMP(NMP)
      COMMON/DGAM/SRA(MAXMAT,NMP),SCO(MAXMAT,NMP),
     1 SPH(MAXMAT,NMP),EPHT(MAXMAT,NMS),SPHT(MAXMAT,NMS),
     2 SPA(MAXMAT,NMP),MPH0(MAXMAT),INTPH0(MAXMAT)
      COMMON/JUMCOL/P(5),ST,DST,DS1,W1,W2,XEL,XE,XEK,K
C  ****  MAIN-PENELOPE COMMON.
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C
      P(1)=DEXP(SRA(M,K)+(SRA(M,K+1)-SRA(M,K))*XEK)
      P(2)=DEXP(SCO(M,K)+(SCO(M,K+1)-SCO(M,K))*XEK)
      IF(K.LT.INTPH0(M)) THEN
        II=1
        II1=MPH0(M)
  100   IIT=(II+II1)/2
        IF(XEL.GT.EPHT(M,IIT)) THEN
          II=IIT
        ELSE
          II1=IIT
        ENDIF
        IF(II1-II.GT.1) GO TO 100
        DEE=EPHT(M,II+1)-EPHT(M,II)
        IF(DEE.GT.1.0D-7) THEN
          P(3)=DEXP(SPHT(M,II)+(SPHT(M,II+1)-SPHT(M,II))
     1        *(XEL-EPHT(M,II))/DEE)
        ELSE
          P(3)=DEXP(SPHT(M,II))
        ENDIF
      ELSE
        P(3)=DEXP(SPH(M,K)+(SPH(M,K+1)-SPH(M,K))*XEK)
      ENDIF
      IF(E.LT.1.023D6) THEN
        P(4)=0.0D0
      ELSE
        P(4)=DEXP(SPA(M,K)+(SPA(M,K+1)-SPA(M,K))*XEK)
      ENDIF
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE ELHARD
C  **************************************************************
      SUBROUTINE ELHARD(FP,FPH,A,B,CDT)
C
C     SIMULATION OF HARD ELASTIC EVENTS. W2D MODEL.
C
C  INPUT ARGUMENTS:
C     FP ........ TRUE ELASTIC MEAN FREE PATH,
C     FPH ....... MEAN FREE PATH BETWEEN HARD COLLISIONS,
C     A, B ...... ANGULAR DISTRIBUTION PARAMETERS.
C
C  OUTPUT VALUE:
C     CDT ....... COSINE OF THE POLAR SCATTERING ANGLE.
C
C                       FP AND FPH IN THE SAME (ARBITRARY) UNITS.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      EXTERNAL RAND
C
      A1=A+1.0D0
      IF(B.GT.0.0D0) THEN
        BB=B
        C=A*A1*DLOG(A1/A)-A
      ELSE
        BB=-B
        C=0.8D0
      ENDIF
      B1=1.0D0-BB
      RUC=1.0D0-FP/FPH
      RUT=RUC+RAND(1.0D0)*(1.0D0-RUC)
C
      RU1=B1*C*A1/(A+C)
      IF(RUT.LT.RU1) THEN
        RUU=RUT/B1
        CDT=1.0D0-2.0D0*A*RUU/(A1-RUU)
      ELSE IF(RUT.GT.RU1+BB) THEN
        RUU=(RUT-BB)/B1
        CDT=1.0D0-2.0D0*A*RUU/(A1-RUU)
      ELSE
        CDT=1.0D0-2.0D0*C
      ENDIF
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE ELSOFT
C  **************************************************************
      SUBROUTINE ELSOFT(FP,FPH,A,B,S,U,V,W)
C
C     SIMULATION OF THE ANGULAR DEFLECTION DUE TO SOFT ELASTIC
C  EVENTS ALONG A TRACK SEGMENT. W2D MODEL.
C
C  INPUT ARGUMENTS:
C     FP ........ TRUE ELASTIC MEAN FREE PATH,
C     FPH ....... MEAN FREE PATH BETWEEN HARD COLLISIONS,
C     A, B ...... ANGULAR DISTRIBUTION PARAMETERS,
C     S ......... LENGTH OF THE TRACK SEGMENT,
C     U, V, W ... INITIAL DIRECTION COSINES.
C
C  OUTPUT VALUES:
C     U, V, W ... FINAL DIRECTION COSINES.
C
C                    FP, FPH AND S IN THE SAME (ARBITRARY) UNITS.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      EXTERNAL RAND
C
      A1=A+1.0D0
      IF(B.GT.0.0D0) THEN
        BB=B
        C=A*A1*DLOG(A1/A)-A
      ELSE
        BB=-B
        C=0.8D0
      ENDIF
      B1=1.0D0-BB
      RUC=1.0D0-FP/FPH
C  ****  CUTOFF ANGULAR DEFLECTION.
      RU1=B1*C*A1/(A+C)
      IF(RUC.LT.RU1) THEN
        RUU=RUC/B1
        RMUC=A*RUU/(A1-RUU)
      ELSE IF(RUC.GT.RU1+BB) THEN
        RUU=(RUC-BB)/B1
        RMUC=A*RUU/(A1-RUU)
      ELSE
        RMUC=C
      ENDIF
C  ****  FIRST AND SECOND TRANSPORT MEAN FREE PATHS FOR SOFT
C        EVENTS.
      IF(RUC.LT.RU1) THEN
        FRAC=RMUC*A1/(A+RMUC)
        S1=A*(A1*DLOG((A+RMUC)/A)-FRAC)
        S2=A*(RMUC*FRAC-(S1+S1))
        S1=B1*S1
        S2=B1*S2
      ELSE IF(RUC.GT.RU1+BB) THEN
        FRAC=RMUC*A1/(A+RMUC)
        S1=A*(A1*DLOG((A+RMUC)/A)-FRAC)
        S2=A*(RMUC*FRAC-(S1+S1))
        S1=B1*S1+BB*C
        S2=B1*S2+BB*C*C
      ELSE
        FRAC=C*A1/(A+C)
        S1=A*(A1*DLOG((A+C)/A)-FRAC)
        S2=A*(C*FRAC-(S1+S1))
        S1=B1*S1+(RUC-RU1)*C
        S2=B1*S2+(RUC-RU1)*C*C
      ENDIF
C  ****  1ST AND 2ND MOMENTS OF THE FINAL ANGULAR DISTRIBUTION.
      TFP1I=2.0D0*S1/FP
      EMU1=0.5D0*(1.0D0-DEXP(-S*TFP1I))
      IF(EMU1.LT.1.0D-3) THEN
        CDT=1.0D0-2.0D0*EMU1
        DF=TWOPI*RAND(1.0D0)
        CALL DIRECT(CDT,DF,U,V,W)
        RETURN
      ENDIF
C
      TFP2I=6.0D0*(S1-S2)/FP
      EMU2=EMU1-(1.0D0-DEXP(-S*TFP2I))/6.0D0
C  ****  SAMPLE A RANDOM VALUE FROM A DISTRIBUTION WITH THESE
C        MOMENTS.
      AMU=DSQRT((EMU2-EMU1**2)/(EMU1*(1.0D0-EMU1)))
      IF(RAND(2.0D0).LT.EMU1) THEN
        CDT=1.0D0-2.0D0*(EMU1*(1.0D0-AMU)+AMU)
      ELSE
        CDT=1.0D0-2.0D0*EMU1*(1.0D0-AMU)
      ENDIF
      DF=TWOPI*RAND(3.0D0)
      CALL DIRECT(CDT,DF,U,V,W)
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SHINEL
C  **************************************************************
      SUBROUTINE SHINEL(E,WC,DEL,DE,EP,CDT,ES,CDTS,M,KPAR)
C
C     RANDOM SIMULATION OF HARD INELASTIC COLLISIONS.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (REV=5.1099906D5, TREV=2.0D0*REV)
      PARAMETER (MAXMAT=10, NO=30)
      DIMENSION QM(NO),CS(2*NO)
      COMMON/XHIN/FZ(MAXMAT,NO),UI(MAXMAT,NO),WI(MAXMAT,NO),
     1 WI2(MAXMAT,NO),NOSC(MAXMAT)
      EXTERNAL RAND
      NOS=NOSC(M)
C  ****  CONSTANTS.
      RB=E+TREV
      GAM=1.0D0+E/REV
      GAM2=GAM*GAM
      BETA2=(GAM2-1.0D0)/GAM2
      G12=(GAM+1.0D0)**2
      RMOL1=((GAM-1.0D0)/GAM)**2
      IF(KPAR.EQ.3) THEN
        RBHA1=RMOL1*(2.0D0*G12-1.0D0)/(GAM2-1.0D0)
        RBHA2=RMOL1*(3.0D0+1.0D0/G12)
        RBHA3=RMOL1*2.0D0*GAM*(GAM-1.0D0)/G12
        RBHA4=RMOL1*(GAM-1.0D0)**2/G12
      ENDIF
C
C  ****  HARD CLOSE COLLISIONS ABOVE WC THRESHOLD.
C
      WCC=0.0D0
      CSC0=0.0D0
      IF(WC.GT.1.0D-1) THEN
        WCC=WC
        WL=WCC/E
        WL1=1.0D0-WL
C  ****  ELECTRONS (MOLLER'S CROSS SECTION).
        IF(KPAR.NE.3) THEN
        IF(WL.LT.0.5D0) CSC0=(RMOL1*(0.5D0-WL)+1.0D0/WL-1.0D0/WL1
     1    -(1.0D0-RMOL1)*DLOG(WL1/WL))/E
C  ****  POSITRONS (BHABHA'S CROSS SECTION).
        ELSE
        IF(WL.LT.1.0D0) CSC0=((1.0D0/WL-1.0D0)+RBHA1*DLOG(WL)
     1    +RBHA2*WL1+(RBHA3/2.0D0)*(WL**2-1.0D0)+(RBHA4/3.0D0)
     2    *(1.0D0-WL**3))/E
        ENDIF
       ENDIF
C
C  ************  CROSS SECTIONS FOR THE DIFFERENT EXCITATIONS.
C
C  ****  DISTANT EXCITATIONS.
      CSDT0=DMAX1(DLOG(GAM2)-BETA2-DEL,0.0D0)
      CPA=DSQRT(E*RB)
      CST=0.0D0
      DO 3 I=1,NOS
      WII=WI(M,I)
C  ****  TRANSVERSE EXCITATIONS.
      IF(WII.GT.E.OR.WII.LT.WC) GO TO 1
      IF(WII.GT.1.0D-6*E) THEN
        CPP=DSQRT((E-WII)*(E-WII+TREV))
        QM(I)=DSQRT((CPA-CPP)**2+REV*REV)-REV
      ELSE
        QM(I)=WI2(M,I)/(BETA2*TREV)
      ENDIF
      IF(QM(I).GE.WII) GO TO 1
C  ****  LONGITUDINAL EXCITATIONS.
      CSDL=DLOG(WII*(QM(I)+TREV)/(QM(I)*(WII+TREV)))
      CSD=FZ(M,I)*(CSDL+CSDT0)/WII
      CST=CST+CSD
    1 CS(I)=CST
C  ****  CLOSE COLLISIONS.
      IF(WII.LE.WCC) THEN
        CSC=FZ(M,I)*CSC0
      ELSE
        WL=WII/E
        WL1=1.0D0-WL
C  ****  ELECTRONS (MOLLER'S CROSS SECTION).
        IF(KPAR.NE.3) THEN
          IF(WL.GE.0.5D0) GO TO 2
          CSC=FZ(M,I)*(RMOL1*(0.5D0-WL)+1.0D0/WL-1.0D0/WL1
     1       -(1.0D0-RMOL1)*DLOG(WL1/WL))/E
C  ****  POSITRONS (BHABHA'S CROSS SECTION).
        ELSE
          IF(WL.GE.1.0D0) GO TO 2
          CSC=FZ(M,I)*((1.0D0/WL-1.0D0)+RBHA1*DLOG(WL)+RBHA2*WL1
     1       +(RBHA3/2.0D0)*(WL**2-1.0D0)+(RBHA4/3.0D0)*(1.0D0
     2       -WL**3))/E
        ENDIF
      ENDIF
      CST=CST+CSC
    2 CS(NOS+I)=CST
    3 CONTINUE
C
C  ****  SELECTION OF THE COLLISION TYPE.
C
      IF(CST.LE.0.0D0) THEN
        DE=0.0D0
        EP=E
        CDT=1.0D0
        ES=0.0D0
        CDTS=0.0D0
        RETURN
      ENDIF
      TST=CST*RAND(1.0D0)
      DO 4 I=1,NOS
      J=I
      IF(CS(I).GT.TST) GO TO 200
      IF(CS(NOS+I).GT.TST) GO TO 100
    4 CONTINUE
C
C  ****  CLOSE COLLISION.
C
  100 CONTINUE
      ER=DMAX1(WI(M,J),WC)/E
      IF(KPAR.EQ.3) GO TO 102
C  ****  ELECTRONS.
      A=5.0D0*RMOL1
      AER=A*0.5D0*ER
  101 CONTINUE
      FB=(1.0D0+AER)*RAND(2.0D0)
      IF(FB.LT.1.0D0) THEN
        EPS=ER/(1.0D0-FB*(1.0D0-(ER+ER)))
      ELSE
        EPS=ER+(FB-1.0D0)*(0.5D0-ER)/AER
      ENDIF
      EPS2=EPS*EPS
      EPST=EPS/(1.0D0-EPS)
      PHI=1.0D0+EPST**2-EPST+RMOL1*(EPS2+EPST)
      IF(RAND(3.0D0)*(1.0D0+A*EPS2).GT.PHI) GO TO 101
      GO TO 103
C  ****  POSITRONS.
  102 CONTINUE
      EPS=ER/(1.0D0-RAND(4.0D0)*(1.0D0-ER))
      PHI=1.0D0-EPS*(RBHA1-EPS*(RBHA2-EPS*(RBHA3-RBHA4*EPS)))
      IF(RAND(5.0D0).GT.PHI) GO TO 102
C  ****  ENERGY AND SCATTERING ANGLE (PRIMARY PARTICLE).
  103 DE=EPS*E
      EP=E-DE
      CDT=DSQRT(EP*RB/(E*(RB-DE)))
C  ****  ENERGY AND SCATTERING ANGLE (DELTA RAY).
      ES=DE-UI(M,J)
      CDTS=DSQRT(DE*RB/(E*(DE+TREV)))
      RETURN
C
C  ****  DISTANT COLLISION.
C
  200 CONTINUE
      DE=WI(M,J)
      EP=E-DE
      QS=QM(J)/(1.0D0+QM(J)/TREV)
      Q=QS/(((QS/DE)*(1.0D0+DE/TREV))**RAND(6.0D0)-(QS/TREV))
      QTREV=Q*(Q+TREV)
      CPP2=EP*(EP+TREV)
      CDT=(CPP2+CPA*CPA-QTREV)/(2.0D0*CPA*DSQRT(CPP2))
      IF(CDT.GT.1.0D0) CDT=1.0D0
C  ****  ENERGY AND SCATTERING ANGLE (DELTA RAY).
      ES=DE-UI(M,J)
      CDTS=0.5D0*(DE*(E+RB-DE)+QTREV)/DSQRT(E*RB*QTREV)
      IF(CDTS.GT.1.0D0) CDTS=1.0D0
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SBREMS
C  **************************************************************
      SUBROUTINE SBREMS(E,THB,DE,M)
C
C     RANDOM SAMPLING OF HARD BREMSSTRAHLUNG EMISSION.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      PARAMETER (REV=5.1099906D5)
      PARAMETER (MAXMAT=10)
      COMMON/XBRE/F0(MAXMAT,3),BCB(MAXMAT)
      EXTERNAL RAND
C
      TOTEN=E+REV
      GAM=TOTEN/REV
      PA=BCB(M)*GAM
      EM=E/TOTEN
      IF(EM.GT.9.9999D-1) EM=9.9999D-1
C  ****  REJECTION FUNCTIONS.
      ED=(E-5.0D0*REV)/TOTEN
      EC=THB/TOTEN
      EC2=EC*EC
      EMECS=EM*EM-EC2
C  ****  LOW ENERGY CORRECTION.
      XX=E/REV
      F2LE=(4.650D0-F0(M,3)*(6.005D0-F0(M,3)*2.946D0))/XX
     1    -(3.242D1-F0(M,3)*(6.708D1+F0(M,3)*3.906D0))/(1+XX)**2
     2    +(2.033D1+F0(M,3)*(2.338D1-F0(M,3)*7.742D1))/(1+XX)**3
C
      IF(EC.LT.ED) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      F00=F0(M,IC)+F2LE
      EC1=1.0D0-EC
      BEC=EC/(PA*EC1)
      CALL SCHIFF(BEC,F1,F2)
      F1C=F1+F00
      F2C=(F2+F00)*EC1
      IF(EC.LT.ED) THEN
        F00=F0(M,1)+F2LE
        ED1=1.0D0-ED
        BED=ED/(PA*ED1)
        CALL SCHIFF(BED,F1,F2)
        F1D=F1+F00
        F2D=(F2+F00)*EC1
        IF(F1D.GT.F1C) F1C=F1D
        IF(F2D.GT.F2C) F2C=F2D
      ENDIF
      PA1=EMECS*F1C
      PA2=2.66666666666666D0*DLOG(EM/EC)*F2C
C
    1 CONTINUE
      IF(RAND(1.0D0)*(PA1+PA2).GT.PA1) GO TO 2
      EPS=DSQRT(EC2+RAND(2.0D0)*EMECS)
      PB=EPS/(PA*(1.0D0-EPS))
      F1=2.0D0-2.0D0*DLOG(1.0D0+PB*PB)
      IF(PB.LT.1.0D-8) THEN
        F1=F1-PB*TWOPI
      ELSE
        F1=F1-4.0D0*PB*DATAN2(1.0D0,PB)
      ENDIF
      IF(EPS.LT.ED) THEN
        F00=F0(M,2)+F2LE
      ELSE
        F00=F0(M,1)+F2LE
      ENDIF
      IF(RAND(3.0D0)*F1C.GT.F1+F00) GO TO 1
      GO TO 3
    2 CONTINUE
      EPS=EC*(EM/EC)**RAND(4.0D0)
      EPS1=1.0D0-EPS
      PB=EPS/(PA*EPS1)
      PB2=PB*PB
      F1=2.0D0-2.0D0*DLOG(1.0D0+PB2)
      F2=F1-6.666666666666666D-1
      IF(PB.LT.1.0D-8) THEN
        F1=F1-PB*TWOPI
      ELSE
        PA0=4.0D0*PB*DATAN2(1.0D0,PB)
        F1=F1-PA0
        F2=F2+2.0D0*PB2*(4.0D0-PA0-3.0D0*DLOG((1.0D0+PB2)/PB2))
      ENDIF
      F2=0.5D0*(3.0D0*F1-F2)
      IF(EPS.LT.ED) THEN
        F00=F0(M,2)+F2LE
      ELSE
        F00=F0(M,1)+F2LE
      ENDIF
      IF(RAND(5.0D0)*F2C.GT.EPS1*(F2+F00)) GO TO 1
    3 DE=EPS*TOTEN
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SCOHER
C  **************************************************************
      SUBROUTINE SCOHER(E,CDT,M)
C
C     RANDOM SAMPLING OF COHERENT (RAYLEIGH) SCATTERING.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (REV=5.1099906D5)
      PARAMETER (MAXMAT=10)
      COMMON/XCOH/X2COH(MAXMAT,241),PDCOH(MAXMAT,241),FLCOH
      EXTERNAL RAND
C
      X2MAX=2.0D0*DLOG(41.2148D0*E/REV)
      IF(X2MAX.LT.X2COH(M,2)) THEN
        JM=1
      ELSE IF(X2MAX.GE.X2COH(M,240)) THEN
        JM=240
      ELSE
        JM=1+(X2MAX-X2COH(M,1))/FLCOH
      ENDIF
      RUMAX=PDCOH(M,JM)+(PDCOH(M,JM+1)-PDCOH(M,JM))
     1     *(X2MAX-X2COH(M,JM))/(X2COH(M,JM+1)-X2COH(M,JM))
C
    1 RU=RUMAX+DLOG(RAND(1.0D0))
C  ****  BINARY SEARCH.
      J=1
      JU=JM+1
    2 JT=(J+JU)/2
      IF(RU.GT.PDCOH(M,JT)) THEN
        J=JT
      ELSE
        JU=JT
      ENDIF
      IF(JU-J.GT.1) GO TO 2
C
      DENOM=PDCOH(M,J+1)-PDCOH(M,J)
      IF(DENOM.GT.1.0D-12) THEN
        X2RAT=X2COH(M,J)+((X2COH(M,J+1)-X2COH(M,J))
     1       *(RU-PDCOH(M,J))/DENOM)-X2MAX
      ELSE
        X2RAT=X2COH(M,J)-X2MAX
      ENDIF
      CDT=1.0D0-2.0D0*DEXP(X2RAT)
C  ****  REJECTION.
      G=0.5D0*(1.0D0+CDT*CDT)
      IF(RAND(2.0D0).GT.G) GO TO 1
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SINCOH
C  **************************************************************
      SUBROUTINE SINCOH(E,EP,CDT,ISH,M)
C
C     RANDOM SAMPLING OF INCOHERENT (COMPTON) SCATTERING. IMPULSE
C  APPROXIMATION WITH ANALYTICAL ONE-ELECTRON COMPTON PROFILES.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (MAXMAT=10, NO=30)
      PARAMETER (REV=5.1099906D5)
      PARAMETER (D1=0.707107D0, D2=1.414214D0, D12=D1*D1)
      DIMENSION RN(NO),PAC(NO)
      COMMON/COMP/ZAT(MAXMAT,15),STF(MAXMAT,15),ZT(MAXMAT),
     1 AT(MAXMAT),ZEQ(MAXMAT),AEQ(MAXMAT),RHO(MAXMAT),
     2 EXPOT(MAXMAT),NELEM(MAXMAT)
      COMMON/XHIN/FZ(MAXMAT,NO),BEN(MAXMAT,NO),WRI(MAXMAT,NO),
     1 WRI2(MAXMAT,NO),NOSC(MAXMAT)
      COMMON/XINCIA/FJ0(MAXMAT,NO),IZFL(MAXMAT,NO)
      EXTERNAL RAND
C
      EK=E/REV
      EK2=EK+EK+1.0D0
      EK3=EK*EK
      EK1=EK3-EK2-1.0D0
      TAU0=1.0D0/EK2
      A1=DLOG(EK2)
      A2=A1+2.0D0*EK*(1.0D0+EK)/(EK2*EK2)
      NSM=NOSC(M)
      IF(E.GT.5.0D6) GO TO 7
C
C  ****  INCOHERENT SCATTERING FUNCTION FOR THETA=PI.
C
      S0=0.0D0
      DO 1 I=1,NSM
      IF(BEN(M,I).LT.E) THEN
        AUX=E*(E-BEN(M,I))*2.0D0
        PZOMC=FJ0(M,I)*(AUX-REV*BEN(M,I))
     1       /(REV*DSQRT(AUX+AUX+BEN(M,I)**2))
        IF(PZOMC.GT.0.0D0) THEN
          RNI=1.0D0-0.5D0*DEXP(D12-(D1+D2*PZOMC)**2)
        ELSE
          RNI=0.5D0*DEXP(D12-(D1-D2*PZOMC)**2)
        ENDIF
        S0=S0+FZ(M,I)*RNI
      ENDIF
    1 CONTINUE
C
C  ****  SAMPLING TAU.
C
    2 CONTINUE
      IF(RAND(1.0D0)*A2.LT.A1) THEN
        TAU=TAU0**RAND(2.0D0)
      ELSE
        TAU=DSQRT(1.0D0+RAND(3.0D0)*(TAU0*TAU0-1.0D0))
      ENDIF
      CDT1=(1.0D0-TAU)/(EK*TAU)
C  ****  INCOHERENT SCATTERING FUNCTION.
      S=0.0D0
      DO 3 I=1,NSM
      IF(BEN(M,I).LT.E) THEN
        AUX=E*(E-BEN(M,I))*CDT1
        PZOMC=FJ0(M,I)*(AUX-REV*BEN(M,I))
     1       /(REV*DSQRT(AUX+AUX+BEN(M,I)**2))
        IF(PZOMC.GT.0.0D0) THEN
          RN(I)=1.0D0-0.5D0*DEXP(D12-(D1+D2*PZOMC)**2)
        ELSE
          RN(I)=0.5D0*DEXP(D12-(D1-D2*PZOMC)**2)
        ENDIF
        S=S+FZ(M,I)*RN(I)
        PAC(I)=S
      ELSE
        PAC(I)=S-1.0D-6
      ENDIF
    3 CONTINUE
C  ****  REJECTION FUNCTION.
      TST=S*(1.0D0+TAU*(EK1+TAU*(EK2+TAU*EK3)))
     1   /(EK3*TAU*(1.0D0+TAU*TAU))
      IF(RAND(4.0D0)*S0.GT.TST) GO TO 2
      CDT=1.0D0-CDT1
C
C  ****  TARGET ELECTRON SHELL.
C
    4 CONTINUE
      TST=S*RAND(5.0D0)
      DO 5 I=1,NSM
      IF(PAC(I).GT.TST) THEN
        ISH=I
        GO TO 6
      ENDIF
    5 CONTINUE
      ISH=NSM
    6 CONTINUE
C
C  ****  PROJECTED MOMENTUM OF THE TARGET ELECTRON.
C
      A=RAND(6.0D0)*RN(ISH)
      IF(A.LT.0.5D0) THEN
        PZOMC=(D1-DSQRT(D12-DLOG(A+A)))/(D2*FJ0(M,ISH))
      ELSE
        PZOMC=(DSQRT(D12-DLOG(2.0D0-A-A))-D1)/(D2*FJ0(M,ISH))
      ENDIF
      IF(PZOMC.LT.-1.0D0) GO TO 4
C
C  ****  F(EP) REJECTION.
C
      XQC=1.0D0+TAU*(TAU-2.0D0*CDT)
      AF=DSQRT(XQC)*(1.0D0+TAU*(TAU-CDT)/XQC)
      IF(AF.GT.0.0D0) THEN
        FPZMAX=1.0D0+AF*0.2D0
      ELSE
        FPZMAX=1.0D0-AF*0.2D0
      ENDIF
      FPZ=1.0D0+AF*DMAX1(DMIN1(PZOMC,0.2D0),-0.2D0)
      IF(RAND(7.0D0)*FPZMAX.GT.FPZ) GO TO 4
C
C  ****  ENERGY OF THE SCATTERED PHOTON.
C
      T=PZOMC**2
      B1=1.0D0-T*TAU*TAU
      B2=1.0D0-T*TAU*CDT
      IF(PZOMC.GT.0.0D0) THEN
        EP=E*(TAU/B1)*(B2+DSQRT(DABS(B2*B2-B1*(1.0D0-T))))
      ELSE
        EP=E*(TAU/B1)*(B2-DSQRT(DABS(B2*B2-B1*(1.0D0-T))))
      ENDIF
      RETURN
C
C  ****  NO DOPPLER BROADENING FOR E GREATER THAN 5 MeV.
C
    7 CONTINUE
      IF(RAND(8.0D0)*A2.LT.A1) THEN
        TAU=TAU0**RAND(9.0D0)
      ELSE
        TAU=DSQRT(1.0D0+RAND(10.0D0)*(TAU0*TAU0-1.0D0))
      ENDIF
C  ****  REJECTION FUNCTION.
      TST=(1.0D0+TAU*(EK1+TAU*(EK2+TAU*EK3)))
     1   /(EK3*TAU*(1.0D0+TAU*TAU))
      IF(RAND(11.0D0).GT.TST) GO TO 7
      EP=TAU*E
      CDT=1.0D0-(1.0D0-TAU)/(EK*TAU)
C
C  ****  TARGET ELECTRON SHELL.
C
    8 CONTINUE
      TST=ZT(M)*RAND(12.0D0)
      S=0.0D0
      DO 9 I=1,NSM
      S=S+FZ(M,I)
      IF(S.GT.TST) THEN
        ISH=I
        GO TO 10
      ENDIF
    9 CONTINUE
      ISH=NSM
   10 CONTINUE
      IF(EP.GT.E-BEN(M,ISH)) GO TO 7
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SAUTER
C  **************************************************************
      SUBROUTINE SAUTER(ES,CDTS)
C
C     RANDOM SAMPLING OF THE INITIAL DIRECTION OF PHOTOELECTRONS
C  FROM SAUTER'S DISTRIBUTION.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (REV=5.1099906D5)
      EXTERNAL RAND
C
      IF(ES.GT.1.0D9) THEN
        CDTS=1.0D0
        RETURN
      ENDIF
      GAM=1.0D0+ES/REV
      GAM2=GAM*GAM
      BETA=DSQRT((GAM2-1.0D0)/GAM2)
      AC=1.0D0/BETA-1.0D0
      A1=0.5D0*BETA*GAM*(GAM-1.0D0)*(GAM-2.0D0)
      A2=AC+2.0D0
      GTMAX=2.0D0*(A1+1.0D0/AC)
    1 CONTINUE
      RU=RAND(1.0D0)
      TSAM=2.0D0*AC*(2.0D0*RU+A2*DSQRT(RU))/(A2*A2-4.0D0*RU)
      GTR=(2.0D0-TSAM)*(A1+1.0D0/(AC+TSAM))
      IF(RAND(2.0D0)*GTMAX.GT.GTR) GO TO 1
      CDTS=1.0D0-TSAM
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SPAIRP
C  **************************************************************
      SUBROUTINE SPAIRP(E,Z,EPS,M)
C
C     RANDOM SAMPLING OF EPS (REDUCED ENERGY) IN ELECTRON-POSI-
C  TRON PAIR PRODUCTION.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (SL=137.0359895D0, REV=5.1099906D5)
      PARAMETER (MAXMAT=10)
      COMMON/XBRE/F0(MAXMAT,3),BCB(MAXMAT)
      EXTERNAL RAND
C
      EKI=REV/E
      IF(E.LT.1.1D6) THEN
        EPS=EKI+(1.0D0-2.0D0*EKI)*RAND(1.0D0)
        RETURN
      ENDIF
C  ****  LOW-ENERGY AND COULOMB CORRECTIONS.
      ALZ=Z/SL
      T=DSQRT(2.0D0*EKI)
      F00=(-1.774D0-1.210D1*ALZ+1.118D1*ALZ*ALZ)*T
     1    +(8.523D0+7.326D1*ALZ-4.441D1*ALZ*ALZ)*T**2
     2    -(1.352D1+1.211D2*ALZ-9.641D1*ALZ*ALZ)*T**3
     3    +(8.946D0+6.205D1*ALZ-6.341D1*ALZ*ALZ)*T**4
      G0=F0(M,2)+F00
      BMIN=4.0D0*EKI/BCB(M)
      CALL SCHIFP(BMIN,G1,G2)
      G1MIN=G1+G0
      G2MIN=G2+G0
      XR=0.5D0-EKI
      A1=6.666666666666666D-1*G1MIN*XR**2
      P1=A1/(A1+G2MIN)
C  ****  RANDOM SAMPLING OF EPS.
    1 CONTINUE
      IF(RAND(2.0D0).GT.P1) GO TO 2
      RU2M1=2.0D0*RAND(3.0D0)-1.0D0
      IF(RU2M1.LT.0.0D0) THEN
        EPS=0.5D0-XR*DABS(RU2M1)**3.333333333333333D-1
      ELSE
        EPS=0.5D0+XR*RU2M1**3.333333333333333D-1
      ENDIF
      B=EKI/(BCB(M)*EPS*(1.0D0-EPS))
      CALL SCHIFP(B,G1,G2)
      G1=DMAX1(G1+G0,0.0D0)
      IF(RAND(4.0D0)*G1MIN.GT.G1) GO TO 1
      RETURN
    2 CONTINUE
      EPS=EKI+2.0D0*XR*RAND(5.0D0)
      B=EKI/(BCB(M)*EPS*(1.0D0-EPS))
      CALL SCHIFP(B,G1,G2)
      G2=DMAX1(G2+G0,0.0D0)
      IF(RAND(6.0D0)*G2MIN.GT.G2) GO TO 1
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE STORES
C  **************************************************************
      SUBROUTINE STORES(EI,UI,VI,WI,WGHTI,KPARI)
C
C     THIS SUBROUTINE STORES THE INITIAL STATE OF A NEW SECONDARY
C  PARTICLE IN THE SECONDARY STACK. THE INPUT VALUES ARE:
C     EI ........... INITIAL ENERGY,
C     UI, VI, WI ... INITIAL DIRECTION COSINES,
C     WGHTI ........ WEIGHT (=1 IN ANALOGUE SIMULATION),
C     KPARI ........ KIND OF PARTICLE (1: ELECTRON, 2: PHOTON,
C                    3: POSITRON).
C
C     THE PARAMETER NMS FIXES THE MAXIMUM NUMBER OF SECONDARIES
C  WHICH CAN BE STORED. IF THIS NUMBER IS EXCEEDED, A WARNING
C  MESSAGE IS PRINTED ON UNIT 6. IN CASE OF EXHAUSTING THE MEMORY
C  STORAGE, EACH NEW SECONDARY IS STORED ON THE POSITION OF THE
C  LESS ENERGETIC SECONDARY ELECTRON OR PHOTON ALREADY PRODUCED,
C  WHICH IS THUS DISCARDED.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NMS=400)
C  ****  MAIN-PENELOPE COMMON.
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C  ****  SECONDARY STACK.
      COMMON/SECST/ES(NMS),XS(NMS),YS(NMS),ZS(NMS),US(NMS),
     1   VS(NMS),WS(NMS),WGHTS(NMS),KS(NMS),IBODYS(NMS),MS(NMS),
     2   ILBS(NMS),NSEC
      COMMON/ERRSEC/IERSEC
C
      IF(NSEC.LT.NMS) THEN
        NSEC=NSEC+1
        IS=NSEC
      ELSE
        IF(IERSEC.EQ.0) THEN
          WRITE(6,1001)
 1001   FORMAT(/3X,'*** WARNING: NOT ENOUGH STORAGE FOR SECOND',
     1    'ARIES.'/16X,'EABS(KPAR,m) OR THE PARAMETER NMS SHOU',
     2    'LD BE ENLARGED.')
          IERSEC=1
        ENDIF
        NSEC=NMS
        EME=1.0D35
        EMG=1.0D35
        IE=0
        IG=0
        DO 1 I=1,NMS
        IF(KS(I).EQ.1) THEN
          IF(ES(I).LT.EME) THEN
            EME=ES(I)
            IE=I
          ENDIF
        ELSE IF(KS(I).EQ.2) THEN
          IF (ES(I).LT.EMG) THEN
            EMG=ES(I)
            IG=I
          ENDIF
        ENDIF
    1   CONTINUE
C
        IF(IE.GT.0) THEN
          IS=IE
        ELSE IF(IG.GT.0) THEN
          IS=IG
        ELSE
          WRITE(6,1002)
 1002     FORMAT(/3X,'*** NOT ENOUGH STORAGE FOR SECONDARY POS',
     1      'ITRONS.'/7X,'JOB ABORTED.')
          STOP
        ENDIF
      ENDIF
C
      ES(IS)=EI
      XS(IS)=X
      YS(IS)=Y
      ZS(IS)=Z
      US(IS)=UI
      VS(IS)=VI
      WS(IS)=WI
      WGHTS(IS)=WGHTI
      KS(IS)=KPARI
      IBODYS(IS)=IBODY
      MS(IS)=M
      ILBS(IS)=ILB
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SECPAR
C  **************************************************************
      SUBROUTINE SECPAR(LEFT)
C
C     THIS SUBROUTINE DELIVERS THE INITIAL STATE OF A SECONDARY
C  PARTICLE PRODUCED DURING THE PREVIOUS SIMULATION OF THE SHO-
C  WER. THIS PARTICLE IS REMOVED FROM THE SECONDARY STACK, SO
C  THAT IT WILL BE LOST IF A NEW CALL TO 'SECPAR' IS PERFORMED
C  BEFORE SIMULATING ITS TRAJECTORY UP TO THE END.
C     'LEFT' IS THE NUMBER OF PARTICLES IN THE SECONDARY STACK AT
C  THE CALLING TIME. WHEN LEFT=0, THE SIMULATION OF THE SHOWER
C  HAS BEEN COMPLETED.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (NMS=400)
C  ****  MAIN-PENELOPE COMMON.
      COMMON/TRACK/E,X,Y,Z,U,V,W,WGHT,KPAR,IBODY,M,ILB
C  ****  SECONDARY STACK.
      COMMON/SECST/ES(NMS),XS(NMS),YS(NMS),ZS(NMS),US(NMS),
     1   VS(NMS),WS(NMS),WGHTS(NMS),KS(NMS),IBODYS(NMS),MS(NMS),
     2   ILBS(NMS),NSEC
C
      IF(NSEC.GT.0) THEN
        LEFT=NSEC
        E=ES(NSEC)
        X=XS(NSEC)
        Y=YS(NSEC)
        Z=ZS(NSEC)
        U=US(NSEC)
        V=VS(NSEC)
        W=WS(NSEC)
        WGHT=WGHTS(NSEC)
        KPAR=KS(NSEC)
        IBODY=IBODYS(NSEC)
        M=MS(NSEC)
        ILB=ILBS(NSEC)
        NSEC=NSEC-1
      ELSE
        LEFT=0
      ENDIF
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                  ANALYTICAL CROSS SECTIONS.                   C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  **************************************************************
C                       SUBROUTINE PEINEL
C  **************************************************************
      SUBROUTINE PEINEL(Z,F,WI,OP2,WC,E,CS0,CS1,CS2,NOS,IELEC)
C
C     COLLISION INELASTIC CROSS SECTIONS FOR ELECTRONS AND POSI-
C  TRONS. STERNHEIMER-LILJEQUIST BETHE SURFACE MODEL.
C
C  INPUT ARGUMENTS:
C     Z ....... NUMBER OF ELECTRONS PER ATOM OR MOLECULE,
C     F(I) .... OSCILLATOR STRENGTHS,
C     W(I) .... EXCITATION ENERGIES,
C     OP2 ..... PLASMA ENERGY (SQUARED),
C     WC ...... CUTOFF ENERGY LOSS,
C     E ....... ELECTRON OR POSITRON KINETIC ENERGY,
C     NOS ..... NUMBER OF OSCILLATORS,
C     IELEC ... -1 FOR ELECTRONS, +1 FOR POSITRONS.
C
C  OUTPUT ARGUMENTS: CROSS SECTIONS (PER ATOM OR MOLECULE) FOR
C  ENERGY LOSSES LARGER THAN WC.
C     CS0 ..... TOTAL INELASTIC CROSS SECTION,
C     CS1 ..... STOPPING CROSS SECTION,
C     CS2 ..... STRAGGLING CROSS SECTION.
C
C                               (ALL QUANTITIES IN ATOMIC UNITS).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      PARAMETER (SL=137.0359895D0, SL2=SL*SL, TSL2=2.0D0*SL2)
      PARAMETER (NO=30)
      DIMENSION F(NO),FZ(NO),WI(NO),WI2(NO)
      COMMON/COLIN/DELTA,CSD(NO),CSC(NO)
      COMMON/BCLOSE/RMOL1,RBHA1,RBHA2,RBHA3,RBHA4
      CS0=0.0D0
      CS1=0.0D0
      CS2=0.0D0
C
      DO 1 I=1,NOS
      FZ(I)=F(I)/Z
    1 WI2(I)=WI(I)**2
C  ****  CONSTANTS.
      GAM=1.0D0+E/SL2
      GAM2=GAM*GAM
      BETA2=(GAM2-1.0D0)/GAM2
      ZCON=TWOPI*Z/(SL2*BETA2)
C
      G12=(GAM+1.0D0)**2
      RMOL1=((GAM-1.0D0)/GAM)**2
      RBHA1=RMOL1*(2.0D0*G12-1.0D0)/(GAM2-1.0D0)
      RBHA2=RMOL1*(3.0D0+1.0D0/G12)
      RBHA3=RMOL1*2.0D0*GAM*(GAM-1.0D0)/G12
      RBHA4=RMOL1*(GAM-1.0D0)**2/G12
C  ************  DISTANT EXCITATIONS.
C  ****  STERNHEIMER'S ENERGY (=L**2).
      TST=1.0D0/(GAM2*OP2)
      WL2=0.0D0
      FDEL=0.0D0
      DO 2 I=1,NOS
    2 FDEL=FDEL+FZ(I)/(WI2(I)+WL2)
      IF(FDEL.LT.TST) GO TO 7
      WL2=WI2(NOS)
    3 WL2=WL2+WL2
      FDEL=0.0D0
      DO 4 I=1,NOS
    4 FDEL=FDEL+FZ(I)/(WI2(I)+WL2)
      IF(FDEL.GT.TST) GO TO 3
      WL2L=0.0D0
      WL2U=WL2
    5 WL2=0.5D0*(WL2L+WL2U)
      FDEL=0.0D0
      DO 6 I=1,NOS
    6 FDEL=FDEL+FZ(I)/(WI2(I)+WL2)
      IF(FDEL.GT.TST) THEN
      WL2L=WL2
      ELSE
      WL2U=WL2
      ENDIF
      IF(WL2U-WL2L.GT.1.0D-8*WL2) GO TO 5
    7 CONTINUE
C  ****  DENSITY EFFECT CORRECTION (DELTA).
      DELTA=0.0D0
      DO 8 I=1,NOS
    8 DELTA=DELTA+FZ(I)*(DLOG(1.0D0+WL2/WI2(I))-WL2/(WL2+WI2(I)))
C  ****  TRANSVERSE EXCITATIONS.
      CSDT0=DMAX1(DLOG(GAM2)-BETA2-DELTA,0.0D0)
      CPA=DSQRT(E*(E+TSL2))
      DO 9 I=1,NOS
      CSD(I)=0.0D0
      W=WI(I)
      IF(W.GE.E.OR.W.LE.WC) GO TO 9
      W2=W*W
      IF(W.GT.1.0D-6*E) THEN
        CPP=DSQRT((E-W)*(E-W+TSL2))
        QM=DSQRT((CPA-CPP)**2+SL2*SL2)-SL2
      ELSE
        QM=W2/(BETA2*TSL2)
      ENDIF
      IF(QM.GE.W) GO TO 9
C  ****  LONGITUDINAL EXCITATIONS.
      CSDL=DLOG(W*(QM+TSL2)/(QM*(W+TSL2)))
      CSDI=ZCON*FZ(I)*(CSDL+CSDT0)
      CSD(I)=CSDI/W
      CS0=CS0+CSD(I)
      CS1=CS1+CSDI
      CS2=CS2+CSDI*W
    9 CONTINUE
C  ************  CLOSE COLLISIONS.
      DO 10 I=1,NOS
      CSC(I)=0.0D0
      W=WI(I)
      IF(E.LE.W) GO TO 10
      WCR=DMAX1(W,WC,1.0D-5)
      CALL CLOSEC(E,WCR,CSC0,CSC1,CSC2,IELEC)
      CSC(I)=ZCON*FZ(I)*CSC0
      CS0=CS0+CSC(I)
      CS1=CS1+ZCON*FZ(I)*CSC1
      CS2=CS2+ZCON*FZ(I)*CSC2
   10 CONTINUE
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE CLOSEC
C  **************************************************************
      SUBROUTINE CLOSEC(E,WC,CSC0,CSC1,CSC2,IELEC)
C
C     INTEGRATED CROSS SECTIONS (PER ELECTRON) FOR CLOSE COLLI-
C  SIONS WITH ENERGY TRANSFERS LARGER THAN WC.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      COMMON/BCLOSE/RMOL1,RBHA1,RBHA2,RBHA3,RBHA4
      CSC0=0.0D0
      CSC1=0.0D0
      CSC2=0.0D0
      WL=WC/E
      WL1=1.0D0-WL
C  ****  ELECTRONS (MOLLER'S CROSS SECTION).
      IF(IELEC.EQ.-1) THEN
      IF(WL.GE.0.5D0) RETURN
      CSC0=(RMOL1*(0.5D0-WL)+1.0D0/WL-1.0D0/WL1
     1    -(1.0D0-RMOL1)*DLOG(WL1/WL))/E
      CSC1=1.0D0-(3.0D0-RMOL1)*DLOG(2.0D0)
     1    +0.5D0*RMOL1*(0.25D0-WL*WL)-WL/WL1
     2    -(2.0D0-RMOL1)*DLOG(WL1)-DLOG(WL)
      CSC2=(1.0D0+(3.0D0-RMOL1)*(0.5D0-WL)
     1    +RMOL1*(0.125D0-WL**3)/3.0D0-WL/WL1
     2    -(3.0D0-RMOL1)*DLOG(2.0D0*WL1))*E
C  ****  POSITRONS (BHABHA'S CROSS SECTION).
      ELSE
      IF(WL.GE.1.0D0) RETURN
      CSC0=((1.0D0/WL-1.0D0)+RBHA1*DLOG(WL)+RBHA2*WL1
     1    +(RBHA3/2.0D0)*(WL**2-1.0D0)+(RBHA4/3.0D0)*(1.0D0
     2    -WL**3))/E
      CSC1=-DLOG(WL)-RBHA1*WL1+(RBHA2/2.0D0)*(1.0D0-WL**2)
     1    +(RBHA3/3.0D0)*(WL**3-1.0D0)+(RBHA4/4.0D0)*(1.0D0
     2    -WL**4)
      CSC2=(WL1+(RBHA1/2.0D0)*(WL**2-1.0D0)
     1    +(RBHA2/3.0D0)*(1.0D0-WL**3)+(RBHA3/4.0D0)*(WL**4
     2    -1.0D0)+(RBHA4/5.0D0)*(1.0D0-WL**5))*E
      ENDIF
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PEBREM
C  **************************************************************
      SUBROUTINE PEBREM(Z,EC,E,S0D,S1D,S2D,S1,S2,IELEC)
C
C     CROSS SECTION (PER ATOM) FOR BREMSSTRAHLUNG EMISSION BY
C  FAST ELECTRONS OR POSITRONS. BETHE-HEITLER DIFFERENTIAL CROSS
C  SECTION WITH EXPONENTIAL SCREENING AND COULOMB AND LOW-ENERGY
C  CORRECTIONS.
C
C  INPUT ARGUMENTS:
C     Z ........ ATOMIC NUMBER,
C     EC ....... CUTOFF ENERGY FOR DISCRETE PHOTON EMISSION,
C     E ........ ELECTRON OR POSITRON KINETIC ENERGY,
C     IELEC .... -1 FOR ELECTRONS, +1 FOR POSITRONS.
C
C  OUTPUT PARAMETERS:
C     S0D ...... CROSS SECTION FOR DISCRETE PHOTON EMISSION,
C     S1D ...... STOPPING CROSS SECTION FOR DISCRETE PHOTON
C                EMISSION,
C     S2D ...... STRAGGLING CROSS SECTION FOR DISCRETE PHOTON
C                EMISSION,
C     S1 ....... TOTAL RADIATIVE STOPPING CROSS SECTION,
C     S2 ....... TOTAL RADIATIVE STRAGGLING CROSS SECTION.
C
C                               (ALL QUANTITIES IN ATOMIC UNITS).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (SL=137.0359895D0, SL2=SL*SL)
      COMMON/CPAIR/RS(100),ETA(100)
      COMMON/BREMS/BCOR,GAM,EPCUT,F0(4),BCB
      EXTERNAL DBHSC0,DBHSC1,DBHSC2
C
      IZ=Z+0.25D0
      IF(IZ.LE.0) IZ=1
      IF(IZ.GT.100) IZ=100
C  ****  DBM COULOMB CORRECTION.
      ALZ=Z/SL
      A=ALZ*ALZ
      FC=A*(0.202059D0-A*(0.03693D0-A*(0.00835D0-A*(0.00201D0-A*
     1 (0.00049D0-A*(0.00012D0-A*0.00003D0)))))+1.0D0/(A+1.0D0))
C  ****  ELECTRON CONTRIBUTION.
      ETAE=((E/SL2)**0.8D0/(2.43D0+(E/SL2)**0.8D0))*ETA(IZ)
C  ****  SCREENING FUNCTIONS.
      BCB=2.0D0/RS(IZ)
      F0(1)=4.0D0*DLOG(RS(IZ))
      F0(2)=F0(1)-4.0D0*FC
C  ****  LOW-ENERGY CORRECTION.
      F0(3)=ALZ
      XX=E/SL2
      F0(4)=(4.650D0-F0(3)*(6.005D0-F0(3)*2.946D0))/XX
     1     -(3.242D1-F0(3)*(6.708D1+F0(3)*3.906D0))/(1+XX)**2
     2     +(2.033D1+F0(3)*(2.338D1-F0(3)*7.742D1))/(1+XX)**3
C  ****  CONSTANTS.
      GAM=1.0D0+E/SL2
      EPCUT=(E-5.0D0*SL2)/(E+SL2)
      IC=1
      IF(EPCUT.GT.0.0D0) IC=2
      EPC=EC/(E+SL2)
      EPM=E/(E+SL2)
      IF(EPM.GT.9.9999D-1) EPM=9.9999D-1
      CALL SCHIFF(0.0D0,F1,F2)
      BCOR=F2+F0(IC)+F0(4)
      BFACT=Z*(Z+ETAE)/SL**5
C  ****  CORRECTION FOR POSITRONS.
      IF(IELEC.EQ.1) THEN
        T=DLOG(1.0D0+1.0D6*E/(SL*Z)**2)
        FPOS=1.0D0-DEXP(-T*(1.2359D-1-T*(6.1274D-2-T*(3.1516D-2-T
     1   *(7.7446D-3-T*(1.0595D-3-T*(7.0568D-5-T*1.8080D-6)))))))
        BFACT=FPOS*BFACT
      ENDIF
C  ****  INTEGRATED CROSS SECTIONS.
      IF(EPM.GT.EPC) THEN
        CALL GABQ(DBHSC0,EPC,EPM,S0D,1.0D-5,IER)
        S0D=S0D+1.333333333333333D0*BCOR*(DLOG(EPM/EPC)-EPM+EPC)
        CALL GABQ(DBHSC1,0.0D0,EPC,S1C,1.0D-5,IER)
        CALL GABQ(DBHSC1,EPC,EPM,S1D,1.0D-5,IER)
        CALL GABQ(DBHSC2,0.0D0,EPC,S2C,1.0D-5,IER)
        CALL GABQ(DBHSC2,EPC,EPM,S2D,1.0D-5,IER)
      ELSE
        S0D=0.0D0
        S1D=0.0D0
        S2D=0.0D0
        CALL GABQ(DBHSC1,0.0D0,EPM,S1C,1.0D-5,IER)
        CALL GABQ(DBHSC2,0.0D0,EPM,S2C,1.0D-5,IER)
      ENDIF
C
      S0D=BFACT*S0D
      S1=BFACT*(S1C+S1D)*(E+SL2)
      S1D=BFACT*S1D*(E+SL2)
      S2=BFACT*(S2C+S2D)*(E+SL2)**2
      S2D=BFACT*S2D*(E+SL2)**2
      RETURN
      END
C  **************************************************************
      DOUBLE PRECISION FUNCTION DBHSC0(EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      COMMON/BREMS/BCOR,GAM,EPCUT,F0(4),BCB
      EPS1=1.0D0-EPS
      B=EPS/(BCB*GAM*EPS1)
      IF(EPS.LT.EPCUT) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      CALL SCHIFF(B,F1,F2)
      DBHSC0=EPS*(F1+F0(IC)+F0(4))
     1      +1.333333333333333D0*EPS1*(F2+F0(IC)+F0(4)-BCOR)/EPS
      RETURN
      END
C  **************************************************************
      DOUBLE PRECISION FUNCTION DBHSC1(EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      COMMON/BREMS/BCOR,GAM,EPCUT,F0(4),BCB
      EPS1=1.0D0-EPS
      B=EPS/(BCB*GAM*EPS1)
      IF(EPS.LT.EPCUT) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      CALL SCHIFF(B,F1,F2)
      DBHSC1=EPS*EPS*(F1+F0(IC)+F0(4))
     1      +1.333333333333333D0*EPS1*(F2+F0(IC)+F0(4))
      RETURN
      END
C  **************************************************************
      DOUBLE PRECISION FUNCTION DBHSC2(EPS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      COMMON/BREMS/BCOR,GAM,EPCUT,F0(4),BCB
      EPS1=1.0D0-EPS
      B=EPS/(BCB*GAM*EPS1)
      IF(EPS.LT.EPCUT) THEN
        IC=2
      ELSE
        IC=1
      ENDIF
      CALL SCHIFF(B,F1,F2)
      DBHSC2=(EPS*EPS*(F1+F0(IC)+F0(4))
     1      +1.333333333333333D0*EPS1*(F2+F0(IC)+F0(4)))*EPS
      RETURN
      END
C  **************************************************************
      SUBROUTINE SCHIFF(B,F1,F2)
C
C     SCREENING FUNCTIONS F1(B) AND F2(B) IN THE BETHE-HEITLER
C  DIFFERENTIAL CROSS SECTION FOR BREMSSTRAHLUNG EMISSION.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      B2=B*B
      F1=2.0D0-2.0D0*DLOG(1.0D0+B2)
      F2=F1-6.666666666666666D-1
      IF(B.LT.1.0D-10) THEN
        F1=F1-TWOPI*B
      ELSE
        A0=4.0D0*B*DATAN2(1.0D0,B)
        F1=F1-A0
        F2=F2+2.0D0*B2*(4.0D0-A0-3.0D0*DLOG((1.0D0+B2)/B2))
      ENDIF
      F2=0.5D0*(3.0D0*F1-F2)
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE PEPHOT
C  **************************************************************
      SUBROUTINE PEPHOT(IZ,E,CS)
C
C     PHOTOELECTRIC CROSS SECTION (PER ATOM) OBTAINED FROM ANALY-
C  TICAL FITS OF THE LLNL TABLES (UCRL-50400, VOL.6). THE PHOTON
C  ENERGY IS ASSUMED TO BE LARGER THAN 1000 EV.
C
C  INPUT ARGUMENTS:
C     IZ ....... ATOMIC NUMBER,
C     E ........ PHOTON ENERGY (EV).
C
C  OUTPUT ARGUMENTS:
C     CS ....... PHOTOELECTRIC CROSS SECTION (BARN/ATOM).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      COMMON/CPHOT/AK(92),BK(92),CK(92),DK(92),FK(92),AL(11:92),
     1 BL(11:92),CL(11:92),DL(11:92),ALII(28:92),BLII(28:92),
     2 ALIII(30:92),BLIII(30:92),AM(30:92),BM(30:92),CM(30:92),
     3 DM(30:92),AMII(52:92),BMII(52:92),AMIII(54:92),
     4 BMIII(54:92),AMIV(56:92),BMIV(56:92),AMV(60:92),
     5 BMV(60:92),AN(61:92),BN(61:92),CN(61:92),DN(61:92)
      COMMON/CBIND/IB(25,92),NOC(25)
C
C  ****  K-SHELL.
      IF(E.GT.IB(1,IZ)) THEN
        IF(E.LT.5.0D3*(15+IZ)) THEN
          X=DLOG(E)
          CS=DEXP(AK(IZ)-BK(IZ)*X+(CK(IZ)+DK(IZ)/X)/X)
        ELSE
          CS=(((1.6268D-3-2.6830D-6*IZ)/(1.0D0+4.1730D-2*IZ))
     1      +(((1.5274D3-5.1100D-1*IZ)/(1.0D0+1.0270D-2*IZ))
     2      +((1.1330D12-2.1770D9*IZ)/(1.0D0+2.0130D-2*IZ))
     3      /E**1.5D0-9.120D13/E**2.0D0)/E)
     4      *DFLOAT(IZ)**5*FK(IZ)/E
        ENDIF
        RETURN
      ENDIF
C
      IE=DMAX1(E,1.0D3)
      X=DLOG(E)
C  ****  LI-SHELL.
      IF(IE.GT.IB(2,IZ)) THEN
        CS=DEXP(AL(IZ)-BL(IZ)*X+(CL(IZ)+DL(IZ)/X)/X)
        RETURN
      ENDIF
C  ****  LII-SHELL.
      IF(IE.GT.IB(3,IZ)) THEN
        CS=DEXP(ALII(IZ)-BLII(IZ)*X)
        RETURN
      ENDIF
C  ****  LIII-SHELL.
      IF(IE.GT.IB(4,IZ)) THEN
        CS=DEXP(ALIII(IZ)-BLIII(IZ)*X)
        RETURN
      ENDIF
C  ****  MI-SHELL.
      IF(IE.GT.IB(5,IZ)) THEN
        CS=DEXP(AM(IZ)-BM(IZ)*X+(CM(IZ)+DM(IZ)/X)/X)
        RETURN
      ENDIF
C  ****  MII-SHELL.
      IF(IE.GT.IB(6,IZ)) THEN
        CS=DEXP(AMII(IZ)-BMII(IZ)*X)
        RETURN
      ENDIF
C  ****  MIII-SHELL.
      IF(IE.GT.IB(7,IZ)) THEN
        CS=DEXP(AMIII(IZ)-BMIII(IZ)*X)
        RETURN
      ENDIF
C  ****  MIV-SHELL.
      IF(IE.GT.IB(8,IZ)) THEN
        CS=DEXP(AMIV(IZ)-BMIV(IZ)*X)
        RETURN
      ENDIF
C  ****  MV-SHELL.
      IF(IE.GT.IB(9,IZ)) THEN
        CS=DEXP(AMV(IZ)-BMV(IZ)*X)
        RETURN
      ENDIF
C  ****  NI-SHELL.
      IF(IE.GT.IB(10,IZ)) THEN
        CS=DEXP(AN(IZ)-BN(IZ)*X+(CN(IZ)+DN(IZ)/X)/X)
        RETURN
      ENDIF
C  ****  NII-SHELL.
      IF(IE.GT.IB(11,IZ)) THEN
        CS=DEXP(AN(IZ)-BN(IZ)*X+(CN(IZ)+DN(IZ)/X)/X)*9.800D-1
        RETURN
      ENDIF
C  ****  NIII-SHELL.
      IF(IE.GT.IB(12,IZ)) THEN
        CS=DEXP(AN(IZ)-BN(IZ)*X+(CN(IZ)+DN(IZ)/X)/X)*9.665D-1
        RETURN
      ENDIF
C  ****  NIV-SHELL.
        CS=DEXP(AN(IZ)-BN(IZ)*X+(CN(IZ)+DN(IZ)/X)/X)*9.094D-1
        RETURN
      END
C  **************************************************************
C                       SUBROUTINE SCHIFP
C  **************************************************************
      SUBROUTINE SCHIFP(B,G1,G2)
C
C     SCREENING FUNCTIONS G1(B) AND G2(B) IN THE BETHE-HEITLER
C  DIFFERENTIAL CROSS SECTION FOR ELECTRON-POSITRON PAIR PRODUC-
C  TION.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (PI=3.1415926535897932D0, TWOPI=2.0D0*PI)
      B2=B*B
      F1=2.0D0-2.0D0*DLOG(1.0D0+B2)
      F2=F1-6.666666666666666D-1
      IF(B.LT.1.0D-10) THEN
        F1=F1-TWOPI*B
      ELSE
        A0=4.0D0*B*DATAN2(1.0D0,B)
        F1=F1-A0
        F2=F2+2.0D0*B2*(4.0D0-A0-3.0D0*DLOG((1.0D0+B2)/B2))
      ENDIF
      G1=0.5D0*(3.0D0*F1-F2)
      G2=0.25D0*(3.0D0*F1+F2)
      RETURN
      END
C  **************************************************************
C                       BLOCK DATA PENDAT
C  **************************************************************
      BLOCK DATA PENDAT
C
C     PHYSICAL DATA FOR ELEMENTS Z=1-92. (SOME DATA SETS EXTEND
C  TO ELEMENTS WITH Z>92).
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), CHARACTER*2 (L),
     1   INTEGER*4 (I-K,M-N)
C
      COMMON/DELEM/LAS(99)
      COMMON/CATWG/ATW(99)
      COMMON/CMEXP/EPX(99)
      COMMON/CFORM/A1(95),A2(95),A3(95),A4(95),A5(95)
      COMMON/CPHOT/AK(92),BK(92),CK(92),DK(92),FK(92),AL(11:92),
     1 BL(11:92),CL(11:92),DL(11:92),ALII(28:92),BLII(28:92),
     2 ALIII(30:92),BLIII(30:92),AM(30:92),BM(30:92),CM(30:92),
     3 DM(30:92),AMII(52:92),BMII(52:92),AMIII(54:92),
     4 BMIII(54:92),AMIV(56:92),BMIV(56:92),AMV(60:92),
     5 BMV(60:92),AN(61:92),BN(61:92),CN(61:92),DN(61:92)
      COMMON/CPAIR/RS(100),ETA(100)
      COMMON/CBIND/IB(25,92),NOC(25)
      COMMON/CRELX/PRK1(92),PRK2(92),PRK3(92),PRK4(92),PRK5(92),
     1       PRK6(92),PRK7(92)
C
C  ************  CHEMICAL SYMBOLS OF THE ELEMENTS.
C
      DATA LAS /'H ','He','Li','Be','B ','C ','N ','O ','F ',
     1     'Ne','Na','Mg','Al','Si','P ','S ','Cl','Ar','K ',
     2     'Ca','Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu',
     3     'Zn','Ga','Ge','As','Se','Br','Kr','Rb','Sr','Y ',
     4     'Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd','In',
     5     'Sn','Sb','Te','I ','Xe','Cs','Ba','La','Ce','Pr',
     6     'Nd','Pm','Sm','Eu','Gd','Tb','Dy','Ho','Er','Tm',
     7     'Yb','Lu','Hf','Ta','W ','Re','Os','Ir','Pt','Au',
     8     'Hg','Tl','Pb','Bi','Po','At','Rn','Fr','Ra','Ac',
     9     'Th','Pa','U ','Np','Pu','Am','Cm','Bk','Cf','Es'/
C
C  ************  ATOMIC WEIGHTS.
C
      DATA ATW  /1.0079D0,4.0026D0,6.9410D0,9.0122D0,1.0811D1,
     1  1.2011D1,1.4007D1,1.5999D1,1.8998D1,2.0179D1,2.2990D1,
     2  2.4305D1,2.6982D1,2.8086D1,3.0974D1,3.2066D1,3.5453D1,
     3  3.9948D1,3.9098D1,4.0078D1,4.4956D1,4.7880D1,5.0942D1,
     4  5.1996D1,5.4938D1,5.5847D1,5.8933D1,5.8690D1,6.3546D1,
     5  6.5390D1,6.9723D1,7.2610D1,7.4922D1,7.8960D1,7.9904D1,
     6  8.3800D1,8.5468D1,8.7620D1,8.8906D1,9.1224D1,9.2906D1,
     7  9.5940D1,9.7907D1,1.0107D2,1.0291D2,1.0642D2,1.0787D2,
     8  1.1241D2,1.1482D2,1.1871D2,1.2175D2,1.2760D2,1.2690D2,
     9  1.3129D2,1.3291D2,1.3733D2,1.3891D2,1.4012D2,1.4091D2,
     A  1.4424D2,1.4491D2,1.5036D2,1.5196D2,1.5725D2,1.5893D2,
     B  1.6250D2,1.6493D2,1.6726D2,1.6893D2,1.7304D2,1.7497D2,
     C  1.7849D2,1.8095D2,1.8385D2,1.8621D2,1.9020D2,1.9222D2,
     D  1.9508D2,1.9697D2,2.0059D2,2.0438D2,2.0720D2,2.0898D2,
     E  2.0898D2,2.0999D2,2.2202D2,2.2302D2,2.2603D2,2.2703D2,
     F  2.3204D2,2.3104D2,2.3803D2,2.3705D2,2.3905D2,2.4306D2,
     G  2.4707D2,2.4707D2,2.5108D2,2.5208D2/
C
C  ************  MEAN EXCITATION ENERGIES OF THE ELEMENTS.
C
      DATA EPX / 19.2D0, 41.8D0, 40.0D0, 63.7D0, 76.0D0, 81.0D0,
     1   82.0D0, 95.0D0,115.0D0,137.0D0,149.0D0,156.0D0,166.0D0,
     2  173.0D0,173.0D0,180.0D0,174.0D0,188.0D0,190.0D0,191.0D0,
     3  216.0D0,233.0D0,245.0D0,257.0D0,272.0D0,286.0D0,297.0D0,
     4  311.0D0,322.0D0,330.0D0,334.0D0,350.0D0,347.0D0,348.0D0,
     5  343.0D0,352.0D0,363.0D0,366.0D0,379.0D0,393.0D0,417.0D0,
     6  424.0D0,428.0D0,441.0D0,449.0D0,470.0D0,470.0D0,469.0D0,
     7  488.0D0,488.0D0,487.0D0,485.0D0,491.0D0,482.0D0,488.0D0,
     8  491.0D0,501.0D0,523.0D0,535.0D0,546.0D0,560.0D0,574.0D0,
     9  580.0D0,591.0D0,614.0D0,628.0D0,650.0D0,658.0D0,674.0D0,
     A  684.0D0,694.0D0,705.0D0,718.0D0,727.0D0,736.0D0,746.0D0,
     B  757.0D0,790.0D0,790.0D0,800.0D0,810.0D0,823.0D0,823.0D0,
     C  830.0D0,825.0D0,794.0D0,827.0D0,826.0D0,841.0D0,847.0D0,
     D  878.0D0,890.0D0,902.0D0,921.0D0,934.0D0,939.0D0,952.0D0,
     E  966.0D0,980.0D0/
C
C  ************  ATOMIC FORM FACTOR PARAMETERS.
C
      DATA A1 /0.0D0, 3.9265D+0, 4.3100D+1, 5.2757D+1, 2.5021D+1,
     1     1.2211D+1, 9.3229D+0, 3.2455D+0, 2.4197D+0, 1.5985D+0,
     2     3.0926D+1, 1.5315D+1, 7.7061D+0, 3.9493D+0, 2.2042D+0,
     3     1.9453D+1, 1.9354D+1, 8.0374D+0, 8.3779D+1, 5.7370D+1,
     4     5.2310D+1, 4.7514D+1, 4.3785D+1, 4.2048D+1, 3.6972D+1,
     5     3.3849D+1, 3.1609D+1, 2.8763D+1, 2.7217D+1, 2.4263D+1,
     6     2.2403D+1, 1.8606D+1, 1.5143D+1, 1.4226D+1, 1.1792D+1,
     7     9.7574D+0, 1.2796D+1, 1.2854D+1, 1.2368D+1, 1.0208D+1,
     8     8.2823D+0, 7.4677D+0, 7.6028D+0, 6.1090D+0, 5.5346D+0,
     9     4.2340D+0, 4.0444D+0, 4.2905D+0, 4.7950D+0, 5.1112D+0,
     A     5.2407D+0, 5.2153D+0, 5.1639D+0, 4.8814D+0, 5.8054D+0,
     B     6.6724D+0, 6.5104D+0, 6.3364D+0, 6.2889D+0, 6.3028D+0,
     C     6.3853D+0, 6.3475D+0, 6.5779D+0, 6.8486D+0, 7.0993D+0,
     D     7.6122D+0, 7.9681D+0, 8.3481D+0, 6.3875D+0, 8.0042D+0,
     E     8.0820D+0, 7.6940D+0, 7.1927D+0, 6.6751D+0, 6.1623D+0,
     F     5.8335D+0, 5.5599D+0, 4.6551D+0, 4.4327D+0, 4.7601D+0,
     G     5.2872D+0, 5.6084D+0, 5.7680D+0, 5.8041D+0, 5.7566D+0,
     H     5.6541D+0, 6.3932D+0, 6.9313D+0, 7.0027D+0, 6.8796D+0,
     I     6.4739D+0, 6.2405D+0, 6.0081D+0, 5.5708D+0, 5.3680D+0/
      DATA A2 /0.0D0, 1.3426D-1, 9.4875D+1,-1.0896D+2,-4.5494D+1,
     1    -1.9572D+1,-1.2382D+1,-3.6827D+0,-2.4542D+0,-1.4453D+0,
     2     1.3401D+2, 7.9717D+1, 6.2164D+1, 4.0300D+1, 3.1682D+1,
     3    -1.3639D+1,-1.5950D+1,-5.1523D+0, 1.8351D+2, 1.2205D+2,
     4     1.0007D+2, 8.5632D+1, 7.9145D+1, 6.3675D+1, 6.2954D+1,
     5     5.6601D+1, 5.4171D+1, 4.8752D+1, 3.8062D+1, 3.9933D+1,
     6     4.8343D+1, 4.2137D+1, 3.4617D+1, 2.9430D+1, 2.4010D+1,
     7     1.9744D+1, 4.0009D+1, 5.1614D+1, 5.0456D+1, 3.9088D+1,
     8     2.6824D+1, 2.2953D+1, 2.4773D+1, 1.6893D+1, 1.4548D+1,
     9     9.7226D+0, 1.0192D+1, 1.1153D+1, 1.3188D+1, 1.4733D+1,
     A     1.5644D+1, 1.5939D+1, 1.5923D+1, 1.5254D+1, 2.0748D+1,
     B     2.6901D+1, 2.7032D+1, 2.4938D+1, 2.1528D+1, 2.0362D+1,
     C     1.9474D+1, 1.8238D+1, 1.7898D+1, 1.9174D+1, 1.9023D+1,
     D     1.8194D+1, 1.8504D+1, 1.8955D+1, 1.4276D+1, 1.7558D+1,
     E     1.8651D+1, 1.7984D+1, 1.6793D+1, 1.5469D+1, 1.4143D+1,
     F     1.3149D+1, 1.2255D+1, 9.2352D+0, 8.6067D+0, 9.7460D+0,
     G     1.1749D+1, 1.3281D+1, 1.4326D+1, 1.4920D+1, 1.5157D+1,
     H     1.5131D+1, 1.9489D+1, 2.3649D+1, 2.4686D+1, 2.4760D+1,
     I     2.1519D+1, 2.0099D+1, 1.8746D+1, 1.5943D+1, 1.4880D+1/
      DATA A3 /0.0D0, 2.2648D+0, 1.0579D+3, 8.6177D+2, 2.4422D+2,
     1     7.8788D+1, 3.8293D+1, 1.2564D+1, 6.9091D+0, 3.7926D+0,
     2     0.0000D+0, 0.0000D+0, 1.6759D-9, 1.3026D+1, 3.0569D+0,
     3     1.5521D+2, 1.2815D+2, 4.7378D+1, 9.2802D+2, 4.7508D+2,
     4     3.6612D+2, 2.7582D+2, 2.1008D+2, 1.5903D+2, 1.2322D+2,
     5     9.2898D+1, 7.1345D+1, 5.1651D+1, 3.8474D+1, 2.7410D+1,
     6     1.9126D+1, 1.0889D+1, 5.3479D+0, 8.2223D+0, 5.0837D+0,
     7     2.8905D+0, 2.7457D+0, 6.7082D-1, 0.0000D+0, 0.0000D+0,
     8     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     9     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     A     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     B     0.0000D+0, 0.0000D+0, 0.0000D+0, 1.7264D-1, 2.7322D-1,
     C     3.9444D-1, 4.5648D-1, 6.2286D-1, 7.2468D-1, 8.4296D-1,
     D     1.1698D+0, 1.2994D+0, 1.4295D+0, 0.0000D+0, 8.1570D-1,
     E     6.9349D-1, 4.9536D-1, 3.1211D-1, 1.5931D-1, 2.9512D-2,
     F     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     G     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     H     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     I     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0/
      DATA A4 /1.1055D1,6.3519D0,4.7367D+1, 3.9402D+1, 2.2896D+1,
     1     1.3979D+1, 1.0766D+1, 6.5252D+0, 5.1631D+0, 4.0524D+0,
     2     2.7145D+1, 1.8724D+1, 1.4782D+1, 1.1608D+1, 9.7750D+0,
     3     1.6170D+1, 1.5249D+1, 9.1916D+0, 5.4499D+1, 4.1381D+1,
     4     3.7395D+1, 3.3815D+1, 3.1135D+1, 2.8273D+1, 2.6140D+1,
     5     2.3948D+1, 2.2406D+1, 2.0484D+1, 1.8453D+1, 1.7386D+1,
     6     1.7301D+1, 1.5388D+1, 1.3411D+1, 1.2668D+1, 1.1133D+1,
     7     9.8081D+0, 1.3031D+1, 1.4143D+1, 1.3815D+1, 1.2077D+1,
     8     1.0033D+1, 9.2549D+0, 9.5338D+0, 7.9076D+0, 7.3263D+0,
     9     5.9996D+0, 6.0087D+0, 6.2660D+0, 6.7914D+0, 7.1501D+0,
     A     7.3367D+0, 7.3729D+0, 7.3508D+0, 7.1465D+0, 8.2731D+0,
     B     9.3745D+0, 9.3508D+0, 8.9897D+0, 8.4566D+0, 8.2690D+0,
     C     8.1398D+0, 7.9183D+0, 7.9123D+0, 8.1677D+0, 8.1871D+0,
     D     8.1766D+0, 8.2881D+0, 8.4227D+0, 7.0273D+0, 8.0002D+0,
     E     8.1440D+0, 7.9104D+0, 7.5685D+0, 7.1970D+0, 6.8184D+0,
     F     6.5469D+0, 6.3056D+0, 5.4844D+0, 5.2832D+0, 5.5889D+0,
     G     6.0919D+0, 6.4340D+0, 6.6426D+0, 6.7428D+0, 6.7636D+0,
     H     6.7281D+0, 7.5729D+0, 8.2808D+0, 8.4400D+0, 8.4220D+0,
     I     7.8662D+0, 7.5993D+0, 7.3353D+0, 6.7829D+0, 6.5520D+0/
      DATA A5 /0.0D0, 4.9828D+0, 5.5674D+1, 3.0902D+1, 1.1496D+1,
     1     4.8936D+0, 2.5506D+0, 1.2236D+0, 7.4698D-1, 4.7042D-1,
     2     4.7809D+0, 4.6315D+0, 4.3677D+0, 4.9269D+0, 2.6033D+0,
     3     9.6229D+0, 7.2592D+0, 4.1634D+0, 1.3999D+1, 8.6975D+0,
     4     6.9630D+0, 5.4681D+0, 4.2653D+0, 3.2848D+0, 2.7354D+0,
     5     2.1617D+0, 1.7030D+0, 1.2826D+0, 9.7080D-1, 7.2227D-1,
     6     5.0874D-1, 3.1402D-1, 1.6360D-1, 3.2918D-1, 2.3570D-1,
     7     1.5868D-1, 1.5146D-1, 9.7662D-2, 7.3151D-2, 6.4206D-2,
     8     4.8945D-2, 4.3189D-2, 4.4368D-2, 3.3976D-2, 3.0466D-2,
     9     2.4477D-2, 3.7202D-2, 3.7093D-2, 3.8161D-2, 3.8576D-2,
     A     3.8403D-2, 3.7806D-2, 3.4958D-2, 3.6029D-2, 4.3087D-2,
     B     4.7069D-2, 4.6452D-2, 4.2486D-2, 4.1517D-2, 4.1691D-2,
     C     4.2813D-2, 4.2294D-2, 4.5287D-2, 4.8462D-2, 4.9726D-2,
     D     5.5097D-2, 5.6568D-2, 5.8069D-2, 1.2270D-2, 3.8006D-2,
     E     3.5048D-2, 3.0050D-2, 2.5069D-2, 2.0485D-2, 1.6151D-2,
     F     1.4631D-2, 1.4034D-2, 1.1978D-2, 1.1522D-2, 1.2375D-2,
     G     1.3805D-2, 1.4954D-2, 1.5832D-2, 1.6467D-2, 1.6896D-2,
     H     1.7166D-2, 1.9954D-2, 2.2497D-2, 2.1942D-2, 2.1965D-2,
     I     2.0005D-2, 1.8927D-2, 1.8167D-2, 1.6314D-2, 1.5522D-2/
C
C  ************  PHOTOELECTRIC CROSS SECTION PARAMETERS.
C
C  ****  K-SHELL.
      DATA AK /       2.3979D+1, 2.4300D+1, 2.8421D+1, 3.0974D+1,
     1     3.4412D+1, 3.8273D+1, 3.9827D+1, 3.9124D+1, 3.9536D+1,
     2     4.0670D+1, 2.9340D+1, 3.7147D+1, 3.2728D+1, 3.5244D+1,
     3     2.5400D+1, 2.6613D+1, 3.3867D+1, 2.4503D+1, 3.2121D+1,
     4     2.3383D+1, 2.4567D+1, 3.1844D+1, 1.5921D+1, 4.1542D+1,
     5     1.1274D+1, 8.8833D+0, 2.8259D+0,-2.1884D-1,-4.3215D+0,
     6    -4.3310D+0,-1.0659D+1,-9.4605D+0,-1.0901D+1,-2.5348D+1,
     7    -3.2516D+1,-3.7157D+1,-5.3928D+1,-5.0857D+1,-5.4310D+1,
     8    -5.4800D+1,-6.0308D+1,-6.6191D+0,-9.1062D+1,-8.6368D+1,
     9    -9.0798D+1,-1.0317D+2,-1.1344D+2,-1.1536D+2,-1.2078D+2,
     A    -1.0697D+2,-1.3636D+2,-1.3683D+2,-1.6449D+2,-1.5686D+2,
     B    -1.5493D+2,-1.6705D+2,-1.7100D+2,-1.8056D+2,-1.7331D+2,
     C    -1.8190D+2,-1.8464D+2,-1.8832D+2,-1.9458D+2,-2.0113D+2,
     D    -2.3663D+2,-2.4669D+2,-2.5938D+2,-2.6764D+2,-2.4448D+2,
     E    -2.8047D+2,-2.6654D+2,-2.9456D+2,-3.2173D+2,-3.0839D+2,
     F    -3.1714D+2,-3.4126D+2,-2.7761D+2,-1.2817D+2,-3.1376D+2,
     G    -3.2953D+2,-3.7813D+2,-2.7067D+2,-2.3720D+2,-4.4243D+2,
     H    -2.8587D+2,-2.5916D+2,-2.7747D+2,-3.2893D+2,-3.8023D+2,
     I    -3.6323D+2,-3.9107D+2,-3.6863D+2/
      DATA BK /       3.3792D+0, 3.2522D+0, 3.3278D+0, 3.3686D+0,
     1     3.4530D+0, 3.5583D+0, 3.5832D+0, 3.5279D+0, 3.5199D+0,
     2     3.5418D+0, 3.1259D+0, 3.3789D+0, 3.2143D+0, 3.2795D+0,
     3     2.9418D+0, 2.9709D+0, 3.2030D+0, 2.8878D+0, 3.1198D+0,
     4     2.8305D+0, 2.8604D+0, 3.0838D+0, 2.5681D+0, 3.3584D+0,
     5     2.4092D+0, 2.3295D+0, 2.1335D+0, 2.0290D+0, 1.8983D+0,
     6     1.8942D+0, 1.6969D+0, 1.7292D+0, 1.6787D+0, 1.2337D+0,
     7     1.0135D+0, 8.6760D-1, 3.4764D-1, 4.4268D-1, 3.3937D-1,
     8     3.2342D-1, 1.5731D-1, 1.7001D+0,-7.5692D-1,-6.1674D-1,
     9    -7.5358D-1,-1.1184D+0,-1.4189D+0,-1.4791D+0,-1.6419D+0,
     A    -1.2623D+0,-2.0930D+0,-2.1111D+0,-2.9138D+0,-2.6855D+0,
     B    -2.6328D+0,-2.9820D+0,-3.0998D+0,-3.3647D+0,-3.1650D+0,
     C    -3.4162D+0,-3.5025D+0,-3.5984D+0,-3.7806D+0,-3.9703D+0,
     D    -4.9759D+0,-5.2613D+0,-5.6230D+0,-5.8484D+0,-5.1940D+0,
     E    -6.2073D+0,-5.8139D+0,-6.5971D+0,-7.3502D+0,-6.9688D+0,
     F    -7.2153D+0,-7.9074D+0,-6.1435D+0,-2.0756D+0,-7.1433D+0,
     G    -7.5779D+0,-8.9114D+0,-6.0030D+0,-5.0953D+0,-1.0696D+1,
     H    -6.4173D+0,-5.8138D+0,-6.2007D+0,-7.6037D+0,-9.0320D+0,
     I    -8.5693D+0,-9.3240D+0,-8.7213D+0/
      DATA CK /       2.7312D+1, 6.1527D+1, 4.7579D+1, 4.0658D+1,
     1     2.1998D+1,-3.3279D+0,-9.6857D+0, 2.7460D+0, 5.2968D+0,
     2     9.2826D-1, 1.1292D+2, 4.3332D+1, 9.1140D+1, 6.9056D+1,
     3     1.7192D+2, 1.6393D+2, 9.6007D+1, 1.9486D+2, 1.1883D+2,
     4     2.1247D+2, 2.0362D+2, 1.3073D+2, 2.9918D+2, 2.8774D+1,
     5     3.5444D+2, 3.8307D+2, 4.4992D+2, 4.8368D+2, 5.3095D+2,
     6     5.3425D+2, 6.0601D+2, 5.9562D+2, 6.1301D+2, 7.7278D+2,
     7     8.5423D+2, 9.0705D+2, 1.0903D+3, 1.0611D+3, 1.1031D+3,
     8     1.1117D+3, 1.1758D+3, 5.5751D+2, 1.5263D+3, 1.4774D+3,
     9     1.5284D+3, 1.6710D+3, 1.7909D+3, 1.8142D+3, 1.8773D+3,
     A     1.7129D+3, 2.0621D+3, 2.0686D+3, 2.3885D+3, 2.3067D+3,
     B     2.2860D+3, 2.4287D+3, 2.4752D+3, 2.5929D+3, 2.5081D+3,
     C     2.6084D+3, 2.6396D+3, 2.6891D+3, 2.7630D+3, 2.8408D+3,
     D     3.2603D+3, 3.3808D+3, 3.5312D+3, 3.6341D+3, 3.3635D+3,
     E     3.7920D+3, 3.6304D+3, 3.9663D+3, 4.2948D+3, 4.1419D+3,
     F     4.2477D+3, 4.5296D+3, 3.7672D+3, 1.9419D+3, 4.2069D+3,
     G     4.3997D+3, 4.9918D+3, 3.6725D+3, 3.2637D+3, 5.7697D+3,
     H     3.8646D+3, 3.4811D+3, 3.7610D+3, 4.3915D+3, 5.0074D+3,
     I     4.8017D+3, 5.1459D+3, 4.8698D+3/
      DATA DK /      -1.0303D+2,-2.2640D+2,-2.1143D+2,-2.1419D+2,
     1    -1.7912D+2,-1.2048D+2,-1.1587D+2,-1.6450D+2,-1.8615D+2,
     2    -1.8653D+2,-5.4780D+2,-3.4116D+2,-5.0701D+2,-4.4079D+2,
     3    -7.9331D+2,-7.7840D+2,-5.6691D+2,-9.1061D+2,-6.5863D+2,
     4    -9.8894D+2,-9.6927D+2,-7.2627D+2,-1.3162D+3,-3.6777D+2,
     5    -1.5304D+3,-1.6423D+3,-1.8865D+3,-2.0103D+3,-2.1903D+3,
     6    -2.2138D+3,-2.4839D+3,-2.4559D+3,-2.5250D+3,-3.1121D+3,
     7    -3.4197D+3,-3.6198D+3,-4.2852D+3,-4.1965D+3,-4.3659D+3,
     8    -4.4102D+3,-4.6582D+3,-2.2889D+3,-5.9868D+3,-5.8192D+3,
     9    -6.0156D+3,-6.5631D+3,-7.0293D+3,-7.1248D+3,-7.3689D+3,
     A    -6.7171D+3,-8.0997D+3,-8.1296D+3,-9.3614D+3,-9.0732D+3,
     B    -9.0003D+3,-9.5606D+3,-9.7438D+3,-1.0226D+4,-9.8971D+3,
     C    -1.0289D+4,-1.0408D+4,-1.0628D+4,-1.0920D+4,-1.1228D+4,
     D    -1.2880D+4,-1.3361D+4,-1.3955D+4,-1.4381D+4,-1.3330D+4,
     E    -1.5033D+4,-1.4412D+4,-1.5753D+4,-1.7076D+4,-1.6497D+4,
     F    -1.6924D+4,-1.8022D+4,-1.4986D+4,-7.5646D+3,-1.6769D+4,
     G    -1.7556D+4,-1.9960D+4,-1.4571D+4,-1.2910D+4,-2.3099D+4,
     H    -1.5385D+4,-1.3578D+4,-1.4965D+4,-1.7538D+4,-2.0004D+4,
     I    -1.9179D+4,-2.0599D+4,-1.9470D+4/
      DATA FK /     6.1713D-01, 8.9318D-01, 1.0287D+0, 1.0708D+0,
     1     1.0846D+0, 1.0830D+0, 1.0819D+0, 1.0828D+0, 1.0779D+0,
     2     1.0716D+0, 1.0876D+0, 1.0803D+0, 1.0845D+0, 1.0852D+0,
     3     1.0896D+0, 1.0874D+0, 1.0803D+0, 1.0829D+0, 1.0792D+0,
     4     1.0862D+0, 1.0856D+0, 1.0802D+0, 1.0867D+0, 1.0783D+0,
     5     1.0867D+0, 1.0845D+0, 1.0864D+0, 1.0883D+0, 1.0878D+0,
     6     1.0890D+0, 1.0895D+0, 1.0887D+0, 1.0892D+0, 1.0934D+0,
     7     1.0950D+0, 1.0967D+0, 1.1047D+0, 1.1011D+0, 1.1004D+0,
     8     1.1028D+0, 1.1035D+0, 1.1006D+0, 1.1075D+0, 1.1094D+0,
     9     1.1165D+0, 1.1141D+0, 1.1168D+0, 1.1113D+0, 1.1255D+0,
     A     1.1228D+0, 1.1205D+0, 1.1232D+0, 1.1283D+0, 1.1235D+0,
     B     1.1279D+0, 1.1285D+0, 1.1243D+0, 1.1335D+0, 1.1369D+0,
     C     1.1342D+0, 1.1404D+0, 1.1433D+0, 1.1384D+0, 1.1497D+0,
     D     1.1476D+0, 1.1563D+0, 1.1584D+0, 1.1606D+0, 1.1548D+0,
     E     1.1611D+0, 1.1608D+0, 1.1676D+0, 1.1710D+0, 1.1583D+0,
     F     1.1700D+0, 1.1816D+0, 1.1609D+0, 1.1699D+0, 1.1771D+0,
     G     1.1793D+0, 1.1871D+0, 1.1891D+0, 1.1907D+0, 1.2083D+0,
     H     1.1875D+0, 1.2008D+0, 1.1938D+0, 1.2050D+0, 1.2190D+0,
     I     1.2166D+0, 1.2196D+0, 1.2226D+0/
C  ****  LI-SHELL.
      DATA (AL(I),I=11,92)/      1.1318D+2, 2.7985D+1, 4.3794D+1,
     1     2.8274D+1, 3.8072D+1, 3.8696D+1, 3.9829D+1, 4.0486D+1,
     2     3.9801D+1, 4.1351D+1, 4.1936D+1, 2.8470D+1, 2.1363D+1,
     3     9.9023D+0, 2.1411D+1, 1.6946D+1, 1.5250D+1, 2.0236D+1,
     4     1.8027D+1, 1.9087D+1, 2.0916D+1, 2.5765D+1, 2.1418D+1,
     5     1.5925D+1, 3.3806D+1, 7.3843D+0, 1.6764D+1, 2.2579D+1,
     6     3.1371D+1, 3.1997D+1, 2.9376D+1, 3.4968D+0, 3.0032D+1,
     7     3.8248D+1, 2.9685D+1, 1.9393D+1, 4.2924D+1, 1.3968D+1,
     8     3.6216D+1, 1.9151D+1, 1.7211D+1, 1.5977D+1, 1.1789D+1,
     9    -9.7878D-1, 3.4258D+1, 9.0559D+0, 6.2826D+0, 1.8498D+1,
     A     1.9833D+1, 1.2114D+1, 3.2077D+1, 2.7375D+1, 1.2417D+1,
     B    -3.6324D+0,-7.7581D+0, 2.3126D+0,-1.4863D+0, 7.7018D+0,
     C    -7.0476D+0,-4.2139D-1,-7.0912D+1,-4.5775D+1,-8.5437D+1,
     D     7.7575D+0, 5.1485D+0, 1.5243D+1, 2.2209D+1, 9.4167D+0,
     E     2.7464D+1,-4.8969D+1,-5.4966D+1,-1.3966D+2,-1.7286D+1,
     F    -7.2194D+1,-1.4486D+1,-4.8527D+1,-4.7311D+0,-7.5395D+0,
     G     3.9834D+1,-1.8522D+1, 1.2758D-1,-3.9420D+1/
      DATA (BL(I),I=11,92)/      8.7644D+0, 2.6054D+0, 3.6810D+0,
     1     2.5712D+0, 3.2324D+0, 3.2530D+0, 3.3055D+0, 3.3290D+0,
     2     3.2618D+0, 3.3447D+0, 3.3630D+0, 2.7763D+0, 2.4680D+0,
     3     1.9579D+0, 2.4824D+0, 2.3018D+0, 2.2219D+0, 2.4399D+0,
     4     2.3459D+0, 2.3852D+0, 2.4637D+0, 2.6558D+0, 2.4892D+0,
     5     2.2695D+0, 2.9709D+0, 1.9140D+0, 2.2808D+0, 2.5198D+0,
     6     2.8612D+0, 2.8781D+0, 2.7820D+0, 1.7821D+0, 2.8037D+0,
     7     3.0984D+0, 2.7869D+0, 2.3975D+0, 3.2598D+0, 2.2201D+0,
     8     3.0082D+0, 2.3834D+0, 2.3180D+0, 2.2780D+0, 2.1486D+0,
     9     1.6929D+0, 2.9165D+0, 2.0306D+0, 1.9380D+0, 2.3580D+0,
     A     2.4067D+0, 2.1479D+0, 2.8251D+0, 2.6606D+0, 2.1503D+0,
     B     1.6115D+0, 1.4706D+0, 1.8119D+0, 1.6908D+0, 1.9979D+0,
     C     1.5360D+0, 1.7263D+0,-5.2677D-1, 2.6768D-1,-1.0115D+0,
     D     1.9837D+0, 1.9001D+0, 2.2216D+0, 2.4487D+0, 2.0399D+0,
     E     2.6131D+0, 1.8402D-1,-1.4211D-3,-2.6528D+0, 1.1723D+0,
     F    -5.3817D-1, 1.2618D+0, 1.9621D-1, 1.5904D+0, 1.4998D+0,
     G     2.8093D+0, 1.1026D+0, 1.6816D+0, 4.7216D-1/
      DATA (CL(I),I=11,92)/     -2.9368D+2, 3.6962D+0,-5.1754D+1,
     1     4.7326D+0,-2.9353D+1,-3.0791D+1,-3.4371D+1,-3.6143D+1,
     2    -3.3106D+1,-3.8407D+1,-4.0276D+1, 6.6011D+1, 1.2395D+2,
     3     2.1285D+2, 1.3242D+2, 1.7232D+2, 1.8709D+2, 1.5247D+2,
     4     1.7247D+2, 1.6576D+2, 1.5473D+2, 1.1679D+2, 1.5733D+2,
     5     2.0552D+2, 5.6676D+1, 2.7860D+2, 2.0169D+2, 1.5702D+2,
     6     8.4376D+1, 7.9694D+1, 1.0557D+2, 3.3072D+2, 1.0378D+2,
     7     2.9769D+1, 1.1027D+2, 2.0282D+2,-8.4675D+0, 2.6196D+2,
     8     5.5222D+1, 2.1227D+2, 2.3337D+2, 2.4794D+2, 2.9484D+2,
     9     4.1563D+2, 7.9986D+1, 3.2000D+2, 3.5003D+2, 2.3381D+2,
     A     2.2343D+2, 3.0198D+2, 1.0811D+2, 1.5467D+2, 3.0234D+2,
     B     4.6326D+2, 5.0510D+2, 4.0828D+2, 4.4977D+2, 3.6014D+2,
     C     5.1889D+2, 4.4468D+2, 1.1808D+3, 9.1773D+2, 1.3289D+3,
     D     3.6516D+2, 3.9404D+2, 2.9036D+2, 2.2103D+2, 3.5619D+2,
     E     1.6860D+2, 9.7106D+2, 1.0371D+3, 1.9396D+3, 6.3826D+2,
     F     1.2267D+3, 6.1256D+2, 9.7659D+2, 5.2084D+2, 5.5217D+2,
     G    -1.3409D+1, 6.5258D+2, 4.5445D+2, 8.8689D+2/
      DATA (DL(I),I=11,92)/      0.0000D+0, 0.0000D+0, 0.0000D+0,
     1     0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0, 0.0000D+0,
     2     0.0000D+0, 0.0000D+0, 0.0000D+0,-2.7663D+2,-4.3150D+2,
     3    -6.5970D+2,-4.7345D+2,-5.8935D+2,-6.3042D+2,-5.5181D+2,
     4    -6.1002D+2,-5.9549D+2,-5.7602D+2,-4.7662D+2,-6.0044D+2,
     5    -7.3949D+2,-3.2715D+2,-9.4547D+2,-7.3640D+2,-6.2233D+2,
     6    -4.2317D+2,-4.1106D+2,-4.9366D+2,-1.1451D+3,-4.9945D+2,
     7    -2.7672D+2,-5.2745D+2,-8.0369D+2,-1.7265D+2,-1.0120D+3,
     8    -3.7263D+2,-8.5297D+2,-9.2700D+2,-9.8137D+2,-1.1527D+3,
     9    -1.5318D+3,-4.6773D+2,-1.2261D+3,-1.3344D+3,-9.6656D+2,
     A    -9.4050D+2,-1.2060D+3,-5.7955D+2,-7.3287D+2,-1.2176D+3,
     B    -1.7543D+3,-1.8948D+3,-1.5862D+3,-1.7362D+3,-1.4457D+3,
     C    -2.0144D+3,-1.7387D+3,-4.2982D+3,-3.3811D+3,-4.8006D+3,
     D    -1.4820D+3,-1.5882D+3,-1.2342D+3,-1.0053D+3,-1.4815D+3,
     E    -8.3208D+2,-3.6373D+3,-3.8784D+3,-7.0802D+3,-2.4751D+3,
     F    -4.5741D+3,-2.3996D+3,-3.6970D+3,-2.1213D+3,-2.2404D+3,
     G    -2.9461D+0,-2.5333D+3,-1.8340D+3,-3.4097D+3/
C  ****  LII-SHELL.
      DATA (ALII(I),I=28,92)/    3.1206D+1, 2.8909D+1, 2.3549D+1,
     1     2.3801D+1, 2.4926D+1, 2.6556D+1, 2.8434D+1, 2.9456D+1,
     2     3.2526D+1, 3.0036D+1, 3.0762D+1, 3.1198D+1, 3.1263D+1,
     3     3.1711D+1, 3.2634D+1, 3.2799D+1, 3.2007D+1, 3.2922D+1,
     4     3.1649D+1, 3.2919D+1, 3.3080D+1, 3.1259D+1, 3.1599D+1,
     5     3.1540D+1, 3.2133D+1, 3.2810D+1, 3.2195D+1, 3.3978D+1,
     6     3.3617D+1, 3.2106D+1, 3.4646D+1, 3.4647D+1, 3.3548D+1,
     7     3.3769D+1, 3.4282D+1, 3.4449D+1, 3.2987D+1, 3.4672D+1,
     8     3.4539D+1, 3.4037D+1, 3.3861D+1, 3.3439D+1, 3.6258D+1,
     9     3.4887D+1, 3.4548D+1, 3.5388D+1, 3.4006D+1, 3.4304D+1,
     A     3.4748D+1, 3.4419D+1, 3.5618D+1, 3.4622D+1, 3.5173D+1,
     B     3.4874D+1, 3.5060D+1, 3.5378D+1, 3.5472D+1, 3.5641D+1,
     C     3.5377D+1, 3.6352D+1, 3.5085D+1, 3.6316D+1, 3.7573D+1,
     D     3.5850D+1, 3.5924D+1/
      DATA (BLII(I),I=28,92)/    2.5241D+0, 2.1723D+0, 1.4018D+0,
     1     1.4334D+0, 1.5829D+0, 1.8000D+0, 2.0475D+0, 2.1722D+0,
     2     2.5700D+0, 2.2302D+0, 2.3143D+0, 2.3597D+0, 2.3572D+0,
     3     2.4040D+0, 2.5107D+0, 2.5206D+0, 2.4116D+0, 2.5155D+0,
     4     2.3505D+0, 2.4964D+0, 2.5070D+0, 2.2791D+0, 2.3132D+0,
     5     2.2990D+0, 2.3622D+0, 2.4349D+0, 2.3571D+0, 2.5551D+0,
     6     2.5060D+0, 2.3250D+0, 2.6092D+0, 2.6018D+0, 2.4711D+0,
     7     2.4896D+0, 2.5404D+0, 2.5527D+0, 2.3842D+0, 2.5649D+0,
     8     2.5442D+0, 2.4833D+0, 2.4585D+0, 2.4072D+0, 2.7070D+0,
     9     2.5532D+0, 2.5114D+0, 2.5960D+0, 2.4433D+0, 2.4703D+0,
     A     2.5122D+0, 2.4727D+0, 2.5939D+0, 2.4848D+0, 2.5382D+0,
     B     2.5021D+0, 2.5169D+0, 2.5454D+0, 2.5506D+0, 2.5632D+0,
     C     2.5322D+0, 2.6268D+0, 2.4937D+0, 2.6138D+0, 2.7365D+0,
     D     2.5585D+0, 2.5622D+0/
C  ****  LIII-SHELL.
      DATA (ALIII(I),I=30,92)/  -9.8747D+1,-5.1224D+1,-9.0291D+0,
     1     1.4691D+1, 3.3665D+1, 4.5769D+1, 5.6022D+1, 2.9498D+1,
     2     3.4535D+1, 3.6010D+1, 3.6434D+1, 3.4944D+1, 3.4655D+1,
     3     3.4033D+1, 3.3725D+1, 3.2771D+1, 3.1459D+1, 3.1683D+1,
     4     3.1966D+1, 3.0460D+1, 2.7092D+1, 2.8428D+1, 2.9193D+1,
     5     2.9893D+1, 3.3089D+1, 3.3382D+1, 3.2557D+1, 3.3367D+1,
     6     3.2141D+1, 3.1879D+1, 3.4385D+1, 3.4239D+1, 3.4041D+1,
     7     3.4312D+1, 3.4769D+1, 3.4317D+1, 3.3775D+1, 3.4856D+1,
     8     3.5084D+1, 3.5161D+1, 3.4885D+1, 3.5040D+1, 3.4941D+1,
     9     3.4644D+1, 3.5596D+1, 3.5866D+1, 3.5801D+1, 3.6238D+1,
     A     3.5966D+1, 3.6104D+1, 3.6460D+1, 3.6328D+1, 3.6324D+1,
     B     3.6476D+1, 3.6351D+1, 3.6518D+1, 3.6555D+1, 3.6609D+1,
     C     3.6801D+1, 3.6511D+1, 3.6640D+1, 3.6913D+1, 3.6950D+1/
      DATA (BLIII(I),I=30,92)/  -1.6144D+1,-9.1749D+0,-3.1317D+0,
     1     2.0297D-1, 2.8119D+0, 4.4327D+0, 5.7732D+0, 2.2019D+0,
     2     2.8539D+0, 3.0299D+0, 3.0679D+0, 2.8598D+0, 2.8090D+0,
     3     2.7172D+0, 2.6666D+0, 2.5366D+0, 2.3660D+0, 2.3839D+0,
     4     2.4099D+0, 2.2203D+0, 1.8071D+0, 1.9641D+0, 2.0498D+0,
     5     2.1269D+0, 2.4981D+0, 2.5223D+0, 2.4190D+0, 2.5059D+0,
     6     2.3572D+0, 2.3204D+0, 2.6017D+0, 2.5779D+0, 2.5487D+0,
     7     2.5727D+0, 2.6175D+0, 2.5606D+0, 2.4936D+0, 2.6081D+0,
     8     2.6271D+0, 2.6296D+0, 2.5933D+0, 2.6046D+0, 2.5877D+0,
     9     2.5503D+0, 2.6476D+0, 2.6711D+0, 2.6586D+0, 2.6996D+0,
     A     2.6652D+0, 2.6747D+0, 2.7073D+0, 2.6879D+0, 2.6826D+0,
     B     2.6935D+0, 2.6757D+0, 2.6877D+0, 2.6872D+0, 2.6878D+0,
     C     2.7030D+0, 2.6684D+0, 2.6771D+0, 2.7007D+0, 2.7003D+0/
C  ****  MI-SHELL.
      DATA (AM(I),I=30,92)/      4.3696D+4, 1.5986D+3, 5.9747D+2,
     1    -1.2429D+3,-9.8054D+2,-1.7572D+2, 3.7533D+2, 1.4914D+2,
     2    -1.0396D+2, 6.6247D+1, 5.5963D+1, 6.1116D+1, 1.4702D+2,
     3     4.7495D+1, 4.2670D+1, 1.1077D+2, 4.0958D+1,-3.3847D+1,
     4    -4.2259D+0,-1.9229D-1, 4.2862D+0,-4.4277D+0,-2.2799D+1,
     5    -5.7999D+0, 1.2115D+0, 6.7243D+1, 3.4778D+1, 1.6406D+1,
     6     3.2250D+1, 3.2369D+1, 4.7351D+1, 6.2571D+1, 7.3130D+1,
     7    -1.5089D+2, 1.2599D+1, 2.6877D+1, 6.5789D+1, 4.8170D+1,
     8     1.4208D+1, 1.6656D+1,-2.3619D+0, 1.1419D+1, 2.3127D+1,
     9     3.2900D+1, 3.7072D+1, 1.5881D+1,-1.0012D+1,-4.0403D+0,
     A     7.4764D+0, 2.4848D+1, 3.2699D+1, 3.3432D+1, 2.8479D+1,
     B     1.0733D+1, 6.4406D+1, 1.4034D+1,-7.9910D+0,-3.1806D+1,
     C    -4.1957D+1,-1.0685D+2,-4.7101D+1,-1.2630D+2, 2.3973D+1/
      DATA (BM(I),I=30,92)/      2.1140D+3, 7.7565D+1, 2.9716D+1,
     1    -5.8242D+1,-4.5525D+1,-7.4344D+0, 1.8527D+1, 8.3222D+0,
     2    -3.4940D+0, 4.3731D+0, 3.8704D+0, 4.1155D+0, 8.0643D+0,
     3     3.4994D+0, 3.2418D+0, 6.2081D+0, 3.2096D+0,-1.3049D-1,
     4     1.1988D+0, 1.3900D+0, 1.6075D+0, 1.2310D+0, 4.3817D-1,
     5     1.1834D+0, 1.4965D+0, 4.3281D+0, 2.9248D+0, 2.1594D+0,
     6     2.8220D+0, 2.8092D+0, 3.4341D+0, 4.0634D+0, 4.4855D+0,
     7    -4.6756D+0, 2.0067D+0, 2.5907D+0, 4.1762D+0, 3.4617D+0,
     8     2.0917D+0, 2.1971D+0, 1.4307D+0, 1.9656D+0, 2.4353D+0,
     9     2.8263D+0, 2.9739D+0, 2.1422D+0, 1.1220D+0, 1.3530D+0,
     A     1.8027D+0, 2.4676D+0, 2.7848D+0, 2.7975D+0, 2.5762D+0,
     B     1.9850D+0, 3.9644D+0, 2.0154D+0, 1.2168D+0, 3.4698D-1,
     C    -3.4106D-2,-2.3935D+0,-2.2531D-1,-3.1286D+0, 2.3553D+0/
      DATA (CM(I),I=30,92)/     -3.0100D+5,-1.0925D+4,-3.9414D+3,
     1     8.8924D+3, 7.0903D+3, 1.4251D+3,-2.4706D+3,-8.0017D+2,
     2     1.0084D+3,-2.1613D+2,-1.4411D+2,-1.7851D+2,-7.9887D+2,
     3    -7.4736D+1,-4.3471D+1,-5.5985D+2,-2.0291D+1, 5.3892D+2,
     4     3.2112D+2, 2.9476D+2, 2.6609D+2, 3.3496D+2, 4.7841D+2,
     5     3.5147D+2, 3.0094D+2,-2.0981D+2, 4.1708D+1, 1.9015D+2,
     6     6.5509D+1, 6.2733D+1,-5.5300D+1,-1.7635D+2,-2.6277D+2,
     7     1.5615D+3, 2.3199D+2, 1.1699D+2,-1.9912D+2,-5.3211D+1,
     8     2.2845D+2, 2.1100D+2, 3.6929D+2, 2.5269D+2, 1.5691D+2,
     9     7.6874D+1, 3.9309D+1, 2.2046D+2, 4.4026D+2, 3.9017D+2,
     A     2.9332D+2, 1.4344D+2, 8.0413D+1, 7.1760D+1, 1.0876D+2,
     B     2.8505D+2,-1.9811D+2, 2.3657D+2, 4.3923D+2, 6.5746D+2,
     C     7.4865D+2, 1.3442D+3, 7.9727D+2, 1.5180D+3, 1.4781D+2/
      DATA (DM(I),I=30,92)/      6.9158D+5, 2.5335D+4, 9.1004D+3,
     1    -2.0721D+4,-1.6593D+4,-3.3008D+3, 5.8786D+3, 1.7749D+3,
     2    -2.5300D+3, 4.0552D+2, 2.3840D+2, 3.1608D+2, 1.8088D+3,
     3     5.5961D+1,-1.0073D+1, 1.2897D+3,-9.0247D+1,-1.4805D+3,
     4    -9.4718D+2,-8.9072D+2,-8.3089D+2,-1.0114D+3,-1.3836D+3,
     5    -1.0690D+3,-9.4797D+2, 3.6716D+2,-2.8090D+2,-6.7969D+2,
     6    -3.5293D+2,-3.4063D+2,-3.0975D+1, 2.8970D+2, 5.2545D+2,
     7    -4.4170D+3,-8.1961D+2,-5.1062D+2, 3.4310D+2,-5.8629D+1,
     8    -8.3611D+2,-7.9585D+2,-1.2339D+3,-9.0588D+2,-6.4512D+2,
     9    -4.2681D+2,-3.1441D+2,-8.3007D+2,-1.4503D+3,-1.3105D+3,
     A    -1.0396D+3,-6.0877D+2,-4.4199D+2,-4.1013D+2,-4.9922D+2,
     B    -1.0754D+3, 3.7237D+2,-8.7659D+2,-1.4951D+3,-2.1604D+3,
     C    -2.4331D+3,-4.2531D+3,-2.5865D+3,-4.7709D+3,-6.1028D+2/
C  ****  MII-SHELL.
      DATA (AMII(I),I=52,92)/    2.8464D+1, 2.8410D+1, 2.9049D+1,
     1     2.8721D+1, 2.8934D+1, 2.9396D+1, 2.9546D+1, 2.9676D+1,
     2     2.9716D+1, 2.9854D+1, 3.0146D+1, 3.0256D+1, 3.0695D+1,
     3     3.0805D+1, 3.1090D+1, 3.1195D+1, 3.1319D+1, 3.1413D+1,
     4     3.1583D+1, 3.1325D+1, 3.1967D+1, 3.1724D+1, 3.2043D+1,
     5     3.2051D+1, 3.2215D+1, 3.2058D+1, 3.2212D+1, 3.2326D+1,
     6     3.2527D+1, 3.2469D+1, 3.2918D+1, 3.3384D+1, 3.2899D+1,
     7     3.3448D+1, 3.2740D+1, 3.3490D+1, 3.3460D+1, 3.3603D+1,
     8     3.3093D+1, 3.3414D+1, 3.3189D+1/
      DATA (BMII(I),I=52,92)/    2.0364D+0, 2.0190D+0, 2.1012D+0,
     1     2.0451D+0, 2.0663D+0, 2.1227D+0, 2.1353D+0, 2.1447D+0,
     2     2.1422D+0, 2.1535D+0, 2.1852D+0, 2.1931D+0, 2.2440D+0,
     3     2.2511D+0, 2.2816D+0, 2.2883D+0, 2.2976D+0, 2.3029D+0,
     4     2.3181D+0, 2.2781D+0, 2.3542D+0, 2.3166D+0, 2.3505D+0,
     5     2.3450D+0, 2.3595D+0, 2.3339D+0, 2.3471D+0, 2.3554D+0,
     6     2.3748D+0, 2.3616D+0, 2.4107D+0, 2.4622D+0, 2.3975D+0,
     7     2.4582D+0, 2.3681D+0, 2.4522D+0, 2.4433D+0, 2.4553D+0,
     8     2.3903D+0, 2.4236D+0, 2.3925D+0/
C  ****  MIII-SHELL.
      DATA (AMIII(I),I=54,92)/   2.7970D+1, 2.8627D+1, 2.8851D+1,
     1     2.9020D+1, 3.0163D+1, 2.9905D+1, 2.9893D+1, 3.0087D+1,
     2     3.0337D+1, 3.1366D+1, 3.1182D+1, 3.1238D+1, 3.1201D+1,
     3     3.1320D+1, 3.1507D+1, 3.1636D+1, 3.1867D+1, 3.2391D+1,
     4     3.2384D+1, 3.2735D+1, 3.2905D+1, 3.2606D+1, 3.2676D+1,
     5     3.2352D+1, 3.2378D+1, 3.2526D+1, 3.2885D+1, 3.3462D+1,
     6     3.3122D+1, 3.3163D+1, 3.3253D+1, 3.2972D+1, 3.3680D+1,
     7     3.3896D+1, 3.3761D+1, 3.3975D+1, 3.3968D+1, 3.4151D+1,
     8     3.4341D+1/
      DATA (BMIII(I),I=54,92)/   1.9531D+0, 2.0398D+0, 2.0623D+0,
     1     2.0778D+0, 2.2298D+0, 2.1845D+0, 2.1744D+0, 2.1929D+0,
     2     2.2189D+0, 2.3514D+0, 2.3176D+0, 2.3170D+0, 2.3041D+0,
     3     2.3127D+0, 2.3300D+0, 2.3400D+0, 2.3632D+0, 2.4237D+0,
     4     2.4157D+0, 2.4539D+0, 2.4682D+0, 2.4229D+0, 2.4251D+0,
     5     2.3779D+0, 2.3751D+0, 2.3873D+0, 2.4259D+0, 2.4910D+0,
     6     2.4430D+0, 2.4424D+0, 2.4474D+0, 2.4067D+0, 2.4879D+0,
     7     2.5082D+0, 2.4862D+0, 2.5061D+0, 2.5002D+0, 2.5169D+0,
     8     2.5341D+0/
C  ****  MIV-SHELL.
      DATA (AMIV(I),I=56,92)/    3.0122D+1, 3.0955D+1, 3.0487D+1,
     1     3.0207D+1, 2.5951D+1, 2.5029D+1, 2.4489D+1, 2.4349D+1,
     2     2.2077D+1, 2.3319D+1, 2.3001D+1, 2.3993D+1, 2.4974D+1,
     3     2.3049D+1, 2.3260D+1, 2.1549D+1, 2.1066D+1, 2.0432D+1,
     4     1.8834D+1, 2.1163D+1, 1.9081D+1, 1.6610D+1, 1.6876D+1,
     5     1.6895D+1, 1.7888D+1, 2.2040D+1, 2.6269D+1, 3.1848D+1,
     6     3.6211D+1, 4.0775D+1, 4.0274D+1, 3.8657D+1, 3.5375D+1,
     7     3.4248D+1, 3.3686D+1, 3.3973D+1, 3.3725D+1/
      DATA (BMIV(I),I=56,92)/    2.2646D+0, 2.3732D+0, 2.2945D+0,
     1     2.2426D+0, 1.6378D+0, 1.5055D+0, 1.4268D+0, 1.4043D+0,
     2     1.0891D+0, 1.2560D+0, 1.2085D+0, 1.3388D+0, 1.4667D+0,
     3     1.2143D+0, 1.2417D+0, 1.0132D+0, 9.4913D-1, 8.6396D-1,
     4     6.5217D-1, 9.5686D-1, 6.9814D-1, 3.8599D-1, 4.2158D-1,
     5     4.2677D-1, 5.5402D-1, 1.0765D+0, 1.6032D+0, 2.2960D+0,
     6     2.8328D+0, 3.3965D+0, 3.3144D+0, 3.1085D+0, 2.7002D+0,
     7     2.5570D+0, 2.4844D+0, 2.5139D+0, 2.4787D+0/
C  ****  MV-SHELL.
      DATA (AMV(I),I=60,92)/    -5.1263D+1,-1.6317D+2,-1.5573D+2,
     1    -1.6129D+2,-1.7067D+2,-1.4933D+2,-1.4539D+2,-1.4227D+2,
     2    -1.3672D+2,-1.4537D+2,-1.2690D+2,-1.3741D+2,-1.3923D+2,
     3    -1.4425D+2,-1.4337D+2,-1.4439D+2,-1.6290D+2,-1.6561D+2,
     4    -1.5714D+2,-1.5286D+2,-1.2996D+2,-8.8438D+1,-3.6039D+1,
     5     1.2311D+1, 4.9236D+1, 7.3102D+1, 8.4064D+1, 3.6910D+1,
     6     4.9749D+1, 4.0084D+1, 3.4826D+1, 3.6361D+1, 3.4351D+1/
      DATA (BMV(I),I=60,92)/    -9.4871D+0,-2.5479D+1,-2.4235D+1,
     1    -2.4853D+1,-2.5973D+1,-2.2852D+1,-2.2160D+1,-2.1592D+1,
     2    -2.0700D+1,-2.1767D+1,-1.9130D+1,-2.0424D+1,-2.0539D+1,
     3    -2.1085D+1,-2.0840D+1,-2.0855D+1,-2.3179D+1,-2.3397D+1,
     4    -2.2172D+1,-2.1501D+1,-1.8444D+1,-1.3031D+1,-6.2924D+0,
     5    -1.3421D-1, 4.5248D+0, 7.5027D+0, 8.8440D+0, 2.9457D+0,
     6     4.5269D+0, 3.3221D+0, 2.6680D+0, 2.8493D+0, 2.5977D+0/
C  ****  N-SHELL.
      DATA (AN(I),I=61,92)/      5.9530D+1, 1.6834D+4, 2.0291D+3,
     1    -3.9503D+3,-1.8871D+3, 3.5704D+2,-9.5901D+2,-9.6669D+2,
     2    -2.7421D+2,-2.4803D+2, 3.9781D+1, 2.3276D+2, 1.2573D+2,
     3     5.6990D+1,-1.7870D+2, 2.2840D+2, 1.2677D+2,-2.1780D+2,
     4    -5.7463D+1,-5.1141D+1,-9.5450D+1,-3.0531D+1,-9.3082D+1,
     5    -2.2870D+1, 9.9154D+0,-4.1392D+1, 2.2872D+2, 3.8964D+1,
     6     3.0992D+2,-4.2197D+1, 4.7629D+1, 7.9480D+0/
      DATA (BN(I),I=61,92)/      6.9491D+0, 8.1078D+2, 9.8105D+1,
     1    -1.8802D+2,-8.9127D+1, 1.8269D+1,-4.4813D+1,-4.5121D+1,
     2    -1.2073D+1,-1.0749D+1, 3.0324D+0, 1.2004D+1, 7.0792D+0,
     3     4.0179D+0,-6.9443D+0, 1.1663D+1, 7.0720D+0,-8.7078D+0,
     4    -1.3831D+0,-1.0872D+0,-3.0528D+0,-1.1587D-1,-2.9112D+0,
     5     2.3539D-1, 1.7483D+0,-6.0138D-1, 1.1361D+1, 2.9866D+0,
     6     1.4923D+1,-5.8556D-1, 3.4078D+0, 1.6317D+0/
      DATA (CN(I),I=61,92)/      2.6868D+2,-1.1641D+5,-1.3893D+4,
     1     2.7757D+4, 1.3408D+4,-2.2203D+3, 6.9292D+3, 6.9909D+3,
     2     2.1561D+3, 1.9842D+3,-1.7189D+1,-1.3995D+3,-6.2361D+2,
     3    -1.0958D+2, 1.5797D+3,-1.3854D+3,-6.3567D+2, 1.8714D+3,
     4     7.0351D+2, 6.5960D+2, 9.9298D+2, 5.1618D+2, 9.8308D+2,
     5     4.6265D+2, 2.2724D+2, 6.0089D+2,-1.4278D+3, 3.7547D+0,
     6    -2.0431D+3, 6.1975D+2,-5.2203D+1, 2.4361D+2/
      DATA (DN(I),I=61,92)/     -1.7805D+3, 2.6873D+5, 3.2119D+4,
     1    -6.4577D+4,-3.1314D+4, 4.9627D+3,-1.6232D+4,-1.6390D+4,
     2    -5.1394D+3,-4.7619D+3,-1.2460D+2, 3.1757D+3, 1.3033D+3,
     3     2.6509D+1,-4.0063D+3, 3.1866D+3, 1.3463D+3,-4.7285D+3,
     4    -1.8945D+3,-1.7927D+3,-2.6266D+3,-1.4600D+3,-2.6195D+3,
     5    -1.3351D+3,-7.7242D+2,-1.6771D+3, 3.3938D+3,-1.9922D+2,
     6     4.9496D+3,-1.7554D+3,-8.1000D+1,-8.1425D+2/
C
C  ************  PAIR PRODUCTION CROSS SECTION PARAMETERS.
C
C  ****  SCREENING PARAMETER.
      DATA RS    /1.2281D2,7.3167D1,6.9228D1,6.7301D1,6.4696D1,
     1   6.1228D1,5.7524D1,5.4033D1,5.0787D1,4.7851D1,4.6373D1,
     2   4.5401D1,4.4503D1,4.3815D1,4.3074D1,4.2321D1,4.1586D1,
     3   4.0953D1,4.0524D1,4.0256D1,3.9756D1,3.9144D1,3.8462D1,
     4   3.7778D1,3.7174D1,3.6663D1,3.5986D1,3.5317D1,3.4688D1,
     5   3.4197D1,3.3786D1,3.3422D1,3.3068D1,3.2740D1,3.2438D1,
     6   3.2143D1,3.1884D1,3.1622D1,3.1438D1,3.1142D1,3.0950D1,
     7   3.0758D1,3.0561D1,3.0285D1,3.0097D1,2.9832D1,2.9581D1,
     8   2.9411D1,2.9247D1,2.9085D1,2.8930D1,2.8721D1,2.8580D1,
     9   2.8442D1,2.8312D1,2.8139D1,2.7973D1,2.7819D1,2.7675D1,
     A   2.7496D1,2.7285D1,2.7093D1,2.6911D1,2.6705D1,2.6516D1,
     B   2.6304D1,2.6108D1,2.5929D1,2.5730D1,2.5577D1,2.5403D1,
     C   2.5245D1,2.5100D1,2.4941D1,2.4790D1,2.4655D1,2.4506D1,
     D   2.4391D1,2.4262D1,2.4145D1,2.4039D1,2.3922D1,2.3813D1,
     E   2.3712D1,2.3621D1,2.3523D1,2.3430D1,2.3331D1,2.3238D1,
     F   2.3139D1,2.3048D1,2.2967D1,2.2833D1,2.2694D1,2.2624D1,
     G   2.2545D1,2.2446D1,2.2358D1,2.2264D1,2.2178D1/
C  ****  ASYMPTOTIC TRIPLET CONTRIBUTION.
      DATA ETA   /1.1570D0,1.1690D0,1.2190D0,1.2010D0,1.1890D0,
     1   1.1740D0,1.1760D0,1.1690D0,1.1630D0,1.1570D0,1.1740D0,
     2   1.1830D0,1.1860D0,1.1840D0,1.1800D0,1.1780D0,1.1750D0,
     3   1.1700D0,1.1800D0,1.1870D0,1.1840D0,1.1800D0,1.1770D0,
     4   1.1660D0,1.1690D0,1.1660D0,1.1640D0,1.1620D0,1.1540D0,
     5   1.1560D0,1.1570D0,1.1580D0,1.1570D0,1.1580D0,1.1580D0,
     6   1.1580D0,1.1660D0,1.1730D0,1.1740D0,1.1750D0,1.1700D0,
     7   1.1690D0,1.1720D0,1.1690D0,1.1680D0,1.1640D0,1.1670D0,
     8   1.1700D0,1.1720D0,1.1740D0,1.1750D0,1.1780D0,1.1790D0,
     9   1.1800D0,1.1870D0,1.1940D0,1.1970D0,1.1960D0,1.1940D0,
     A   1.1940D0,1.1940D0,1.1940D0,1.1940D0,1.1960D0,1.1970D0,
     B   1.1960D0,1.1970D0,1.1970D0,1.1980D0,1.1980D0,1.2000D0,
     C   1.2010D0,1.2020D0,1.2040D0,1.2050D0,1.2060D0,1.2080D0,
     D   1.2070D0,1.2080D0,1.2120D0,1.2150D0,1.2180D0,1.2210D0,
     E   1.2240D0,1.2270D0,1.2300D0,1.2370D0,1.2430D0,1.2470D0,
     F   1.2500D0,1.2510D0,1.2520D0,1.2550D0,1.2560D0,1.2570D0,
     G   1.2590D0,1.2620D0,1.2620D0,1.2650D0,1.2660D0/
C
C  ************  ATOMIC BINDING ENERGIES, IN EV, AND SUBSHELL
C                OCCUPATION NUMBERS (LEDERER AND SHIRLEY, 1978).
C
      DATA (IB( 1,I),I=1,92)/ 14, 25, 55, 112, 191, 285, 410,
     1  543, 697, 870,1072, 1303, 1558, 1839, 2149, 2472, 2823,
     2  3206, 3608, 4038, 4489, 4966,5466, 5991, 6538, 7111,
     3  7711, 8332, 8980, 9661, 10367, 11104, 11867, 12658,
     4  13474, 14326, 15200, 16105, 17039, 18000, 18983, 20000,
     5  21044, 22117, 23220, 24350, 25514, 26711, 27940, 29200,
     6  30491, 31814, 33170, 34566, 35985, 37441, 38925, 40444,
     7  41991, 43569, 45185, 46835, 48519, 50239, 51996, 53788,
     8  55618, 57486, 59390, 61332, 63314, 65351, 67413, 69523,
     9  71675, 73871, 76111, 78395, 80722, 83103, 85529, 88005,
     A  90526, 93105, 95730, 98400, 101140, 103920, 106760,
     B  109650, 112600, 115610/
      DATA (IB(2,I),I=1,92)/ 2*0, 5, 9, 8, 11, 15, 24, 31, 48,
     1  63, 89, 118, 149, 189, 229, 270, 326, 379, 438, 498, 561,
     2  627, 698, 769, 848, 927, 1010, 1099, 1196, 1298, 1413,
     3  1527, 1654, 1782, 1925, 2065, 2216, 2373, 2534, 2695,
     4  2866, 3042, 3224, 3412, 3605, 3806, 4018, 4238, 4465,
     5  4699, 4939, 5188, 5453, 5713, 5987, 6267, 6549, 6835,
     6  7126, 7430, 7737, 8052, 8376, 8708, 9047, 9395, 9752,
     7  10116, 10487, 10870, 11272, 11680, 12099, 12527, 12968,
     8  13419, 13880, 14353, 14839, 15347, 15861, 16388, 16939,
     9  17493, 18049, 18639, 19237, 19840, 20472, 21105, 21758/
      DATA (IB(3,I),I=1,92)/ 4*0, 8, 11, 14, 14, 17, 22, 31, 49,
     1  73, 100, 136, 165, 202, 251, 297, 350, 404, 461, 521,
     2  585, 651, 721, 796, 871, 953, 1045, 1143, 1249, 1359,
     3  1476, 1596, 1731, 1865, 2007, 2155, 2309, 2462, 2625,
     4  2793, 2967, 3146, 3331, 3524, 3727, 3938, 4156, 4381,
     5  4612, 4852, 5107, 5360, 5624, 5891, 6165, 6441, 6722,
     6  7015, 7312, 7618, 7931, 8252, 8581, 8918, 9265, 9617,
     7  9978, 10349, 10739, 11136, 11542, 11957, 12385, 12824,
     8  13273, 13733, 14209, 14698, 15200, 15709, 16244, 16785,
     9  17337, 17906, 18484, 19083, 19693, 20314, 20948/
      DATA (IB(4,I),I=1,92)/ 6*0, 14, 14, 17, 22, 31, 49, 73,
     1  99, 135, 164, 200, 249, 295, 347, 399, 455, 513, 576,
     2  639, 708, 781, 854, 933, 1022, 1117, 1217, 1323, 1436,
     3  1550, 1678, 1805, 1940, 2080, 2225, 2368, 2520, 2677,
     4  2838, 3004, 3174, 3352, 3538, 3730, 3929, 4132, 4341,
     5  4557, 4787, 5012, 5247, 5483, 5724, 5965, 6208, 6465,
     6  6717, 6977, 7243, 7515, 7790, 8071, 8358, 8648, 8943,
     7  9244, 9561, 9881, 10205, 10535, 10871, 11215, 11564,
     8  11918, 12284, 12657, 13035, 13418, 13814, 14214, 14619,
     9  15031, 15444, 15871, 16300, 16733, 17168/
      DATA (IB(5,I),I=1,92)/ 10*0, 5, 8, 6, 8, 16, 16, 18, 29,
     1  35, 44, 51, 58, 66, 75, 82, 92, 101, 111, 122, 140, 158,
     2  181, 204, 231, 257, 293, 321, 358, 395, 433, 466, 505,
     3  544, 585, 627, 670, 717, 770, 826, 884, 944, 1006, 1072,
     4  1149, 1217, 1293, 1362, 1435, 1511, 1576, 1656, 1728,
     5  1805, 1888, 1970, 2050, 2123, 2211, 2305, 2397, 2491,
     6  2601, 2705, 2817, 2932, 3049, 3174, 3298, 3425, 3562,
     7  3704, 3851, 3999, 4149, 4317, 4482, 4652, 4822, 5002,
     8  5182, 5367, 5548/
      DATA (IB(6,I),I=1,92)/ 12*0, 6, 8, 10, 10, 13, 16, 18, 25,
     1  28, 33, 37, 43, 47, 53, 59, 67, 77, 91, 107, 129, 147,
     2  167, 189, 222, 248, 280, 313, 347, 376, 410, 445, 483,
     3  521, 559, 602, 651, 702, 757, 812, 870, 931, 1002, 1065,
     4  1137, 1205, 1273, 1338, 1403, 1478, 1546, 1619, 1695,
     5  1770, 1845, 1918, 2010, 2088, 2172, 2264, 2365, 2466,
     6  2572, 2682, 2792, 2909, 3027, 3150, 3279, 3416, 3554,
     7  3696, 3854, 4008, 4159, 4327, 4490, 4656, 4831, 5001,
     8  5181/
      DATA (IB(7,I),I=1,92)/ 14*0, 10, 10, 13, 16, 18, 25, 28,
     1  33, 37, 43, 47, 53, 59, 67, 75, 89, 103, 122, 141, 162,
     2  182, 214, 239, 269, 301, 333, 360, 393, 425, 461, 496,
     3  531, 573, 617, 664, 715, 766, 819, 875, 941, 998, 1063,
     4  1124, 1186, 1243, 1298, 1364, 1425, 1486, 1551, 1614,
     5  1679, 1736, 1816, 1883, 1949, 2024, 2108, 2191, 2278,
     6  2367, 2458, 2551, 2646, 2743, 2847, 2957, 3066, 3177,
     7  3302, 3426, 3538, 3663, 3792, 3909, 4046, 4174, 4304/
      DATA (IB(8,I),I=1,92)/ 18*0, 4, 6, 7, 7, 7, 7, 7, 8, 8, 8,
     1  8, 9, 18, 29, 42, 56, 70, 95, 112, 135, 160, 185, 205,
     2  230, 257, 284, 312, 340, 374, 411, 451, 496, 537, 583,
     3  631, 689, 740, 796, 852, 901, 951, 1000, 1060, 1111,
     4  1166, 1225, 1278, 1335, 1386, 1457, 1513, 1576, 1640,
     5  1716, 1790, 1869, 1949, 2031, 2116, 2202, 2291, 2385,
     6  2485, 2586, 2687, 2798, 2909, 3022, 3136, 3248, 3370,
     7  3491, 3611, 3728/
      DATA (IB(9,I),I=1,92)/ 22*0, 7, 7, 7, 8, 8, 8, 8, 9, 18,
     1  29, 42, 56, 69, 94, 111, 133, 158, 182, 202, 226, 253,
     2  279, 307, 335, 368, 405, 444, 485, 528, 573, 620, 677,
     3  724, 781, 835, 883, 931, 978, 1034, 1085, 1136, 1193,
     4  1244, 1298, 1346, 1413, 1466, 1527, 1589, 1662, 1732,
     5  1807, 1883, 1960, 2041, 2121, 2206, 2295, 2390, 2484,
     6  2580, 2683, 2787, 2892, 3000, 3105, 3219, 3332, 3442,
     7  3552/
      DATA (IB(10,I),I=1,92)/ 28*0, 8, 9, 6, 8, 10, 10, 27, 27,
     1  29, 38, 46, 54, 55, 62, 68, 75, 81, 86, 95, 108, 122,
     2  137, 152, 169, 186, 213, 231, 253, 271, 289, 304, 321,
     3  337, 351, 366, 383, 400, 419, 431, 453, 470, 487, 507,
     4  538, 563, 592, 625, 655, 690, 724, 759, 800, 846, 894,
     5  939, 995, 1042, 1097, 1153, 1208, 1269, 1330, 1387, 1442/
      DATA (IB(11,I),I=1,92)/ 30*0, 6, 8, 10, 10, 12, 14, 14,
     1  20, 26, 31, 31, 35, 39, 43, 48, 51, 59, 67, 77, 89, 99,
     2  110, 123, 146, 162, 180, 196, 207, 218, 230, 242, 251,
     3  261, 311, 322, 339, 349, 366, 382, 399, 412, 437, 462,
     4  489, 518, 547, 577, 608, 644, 677, 722, 764, 805, 851,
     5  886, 929, 980, 1058, 1080, 1168, 1224, 1273/
      DATA (IB(12,I),I=1,92)/ 32*0, 10, 10, 12, 14, 14, 20, 26,
     1  31, 31, 35, 39, 43, 48, 51, 59, 67, 77, 89, 99, 110, 123,
     2  146, 162, 180, 196, 207, 218, 230, 242, 251, 261, 272,
     3  284, 297, 309, 320, 333, 346, 359, 380, 402, 423, 445,
     4  469, 495, 520, 546, 571, 609, 645, 679, 705, 740, 768,
     5  810, 879, 890, 968, 1007, 1045/
      DATA (IB(13,I),I=1,92)/ 36*0, 4, 6, 6, 7, 7, 7, 7, 7, 7, 8,
     1  8, 9, 16, 25, 33, 42, 50, 69, 77, 93, 103, 112, 118, 124,
     2  133, 137, 141, 147, 152, 158, 164, 172, 180, 189, 206,
     3  224, 239, 255, 274, 290, 312, 332, 352, 379, 407, 434,
     4  464, 500, 533, 567, 603, 636, 675, 714, 743, 780/
      DATA (IB(14,I),I=1,92)/ 40*0, 7, 7, 7, 7, 7, 8, 8, 9, 16,
     1  24, 32, 40, 50, 68, 75, 90, 103, 108, 115, 121, 129, 132,
     2  136, 142, 148, 155, 161, 169, 176, 185, 196, 214, 227,
     3  243, 260, 273, 296, 315, 334, 360, 386, 412, 441, 473,
     4  507, 541, 577, 603, 639, 677, 708, 738/
      DATA (IB(15,I),I=1,92)/ 46*0, 8, 9, 6, 7, 8, 12, 14, 23,
     1  23, 40, 36, 38, 38, 38, 38, 33, 37, 43, 42, 66, 51, 64,
     2  51, 53, 57, 65, 68, 74, 83, 84, 96, 102, 108, 120, 137,
     3  148, 160, 177, 195, 214, 234, 254, 272, 290, 310, 324/
      DATA (IB(16,I),I=1,92)/ 48*0, 6, 7, 8, 9, 10, 13, 13, 17,
     1  18, 18, 23, 22, 22, 26, 27, 28, 28, 29, 20, 33, 30, 23,
     2  28, 38, 42, 44, 46, 58, 63, 66, 72, 81, 100, 105, 117,
     3  132, 148, 164, 182, 200, 215, 229, 223, 260/
      DATA (IB(17,I),I=1,92)/ 50*0, 8, 9, 10, 12, 11, 15, 18, 18,
     1  23, 22, 22, 26, 27, 28, 28, 29, 20, 33, 30, 23, 28, 31,
     2  34, 34, 35, 46, 51, 52, 54, 58, 76, 86, 93, 104, 115,
     3  127, 140, 153, 167, 182, 223, 195/
      DATA (IB(18,I),I=1,92)/ 54*0, 4, 5, 6, 5, 5, 5, 6, 6, 6, 6,
     1  6, 6, 6, 6, 6, 6, 5, 19, 25, 34, 43, 54, 64, 75, 88, 104,
     2  123, 141, 162, 184, 210, 238, 268, 299, 319, 344, 371,
     3  392/
      DATA (IB(19,I),I=1,92)/ 60*0, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
     1  5, 17, 23, 32, 40, 51, 61, 71, 84, 100, 119, 136, 157,
     2  184, 210, 238, 268, 299, 319, 335, 360, 381/
      DATA (IB(20,I),I=1,92)/ 68*0, 6, 6, 5, 7, 8, 8, 8, 9, 9, 9,
     1  9, 10, 15, 20, 27, 31, 40, 48, 58, 68, 80, 95, 94, 105/
      DATA (IB(21,I),I=1,92)/ 72*0, 8, 8, 8, 9, 9, 9, 9, 10, 12,
     1  18, 25, 31, 40, 48, 58, 68, 80, 88, 94, 96/
      DATA (IB(22,I),I=1,92)/ 78*0, 9, 10, 6, 7, 7, 12, 18, 26,
     1  34, 44, 50, 60, 65, 71/
      DATA (IB(23,I),I=1,92)/ 80*0, 6, 7, 7, 8, 10, 11, 15, 19,
     1  35, 49, 45, 43/
      DATA (IB(24,I),I=1,92)/ 82*0, 7, 8, 10, 11, 15, 19, 31, 43,
     1  38, 33/
      DATA (IB(25,I),I=1,92)/ 86*0, 4, 5, 7, 6, 5, 4/
C
      DATA (NOC(I),I=1,25) /2,2,2,4,2,2,4,4,6,2,2,4,4,6,2,2,4,6,
     1     8,4,6,2,2,4,6/
C
C  ************  K-SHELL RELAXATION DATA.
C
      DATA PRK1 /10*0.0D0,6.690D-3,9.273D-3,1.237D-2,1.598D-2,
     1  2.012D-2,2.478D-2,2.995D-2,3.559D-2,4.186D-2,4.872D-2,
     2  5.638D-2,6.460D-2,7.328D-2,8.258D-2,9.177D-2,1.014D-1,
     3  1.112D-1,1.211D-1,1.311D-1,1.406D-1,1.498D-1,1.585D-1,
     4  1.669D-1,1.747D-1,1.821D-1,1.889D-1,1.953D-1,2.013D-1,
     5  2.068D-1,2.119D-1,2.167D-1,2.213D-1,2.256D-1,2.296D-1,
     6  2.333D-1,2.367D-1,2.400D-1,2.430D-1,2.457D-1,2.482D-1,
     7  2.504D-1,2.524D-1,2.543D-1,2.560D-1,2.576D-1,2.591D-1,
     8  2.605D-1,2.618D-1,2.630D-1,2.641D-1,2.653D-1,2.663D-1,
     9  2.673D-1,2.682D-1,2.692D-1,2.700D-1,2.707D-1,2.717D-1,
     A  2.727D-1,2.735D-1,2.743D-1,2.752D-1,2.760D-1,2.766D-1,
     B  2.773D-1,2.780D-1,2.787D-1,2.794D-1,2.801D-1,2.807D-1,
     C  2.815D-1,2.822D-1,2.828D-1,2.834D-1,2.841D-1,2.848D-1,
     D  2.855D-1,2.861D-1,2.869D-1,2.877D-1,2.884D-1,2.893D-1/
      DATA PRK2 /10*0.0D0,1.330D-2,1.842D-2,2.455D-2,3.171D-2,
     1  3.987D-2,4.906D-2,5.924D-2,7.033D-2,8.265D-2,9.613D-2,
     2  1.111D-1,1.271D-1,1.441D-1,1.621D-1,1.799D-1,1.986D-1,
     3  2.175D-1,2.364D-1,2.557D-1,2.738D-1,2.912D-1,3.077D-1,
     4  3.233D-1,3.379D-1,3.515D-1,3.642D-1,3.757D-1,3.865D-1,
     5  3.963D-1,4.055D-1,4.139D-1,4.217D-1,4.289D-1,4.356D-1,
     6  4.417D-1,4.472D-1,4.523D-1,4.568D-1,4.609D-1,4.643D-1,
     7  4.674D-1,4.699D-1,4.721D-1,4.741D-1,4.757D-1,4.771D-1,
     8  4.784D-1,4.794D-1,4.802D-1,4.810D-1,4.814D-1,4.819D-1,
     9  4.821D-1,4.822D-1,4.825D-1,4.826D-1,4.821D-1,4.821D-1,
     A  4.820D-1,4.819D-1,4.817D-1,4.815D-1,4.811D-1,4.801D-1,
     B  4.795D-1,4.788D-1,4.781D-1,4.774D-1,4.766D-1,4.758D-1,
     C  4.748D-1,4.739D-1,4.729D-1,4.718D-1,4.708D-1,4.696D-1,
     D  4.685D-1,4.673D-1,4.662D-1,4.650D-1,4.639D-1,4.628D-1/
      DATA PRK3 /12*0.0D0,7.559D-5,2.724D-4,6.214D-4,1.116D-3,
     1  1.909D-3,2.913D-3,3.997D-3,5.178D-3,6.223D-3,7.327D-3,
     2  8.484D-3,9.494D-3,1.094D-2,1.221D-2,1.350D-2,1.481D-2,
     3  1.589D-2,1.739D-2,1.893D-2,2.049D-2,2.206D-2,2.360D-2,
     4  2.511D-2,2.657D-2,2.800D-2,2.937D-2,3.070D-2,3.198D-2,
     5  3.318D-2,3.436D-2,3.547D-2,3.655D-2,3.756D-2,3.852D-2,
     6  3.944D-2,4.031D-2,4.113D-2,4.189D-2,4.261D-2,4.327D-2,
     7  4.391D-2,4.450D-2,4.507D-2,4.560D-2,4.611D-2,4.660D-2,
     8  4.706D-2,4.751D-2,4.792D-2,4.829D-2,4.866D-2,4.901D-2,
     9  4.935D-2,4.967D-2,4.993D-2,5.022D-2,5.051D-2,5.079D-2,
     A  5.105D-2,5.125D-2,5.149D-2,5.162D-2,5.182D-2,5.200D-2,
     B  5.214D-2,5.229D-2,5.242D-2,5.256D-2,5.268D-2,5.279D-2,
     C  5.288D-2,5.296D-2,5.305D-2,5.310D-2,5.317D-2,5.322D-2,
     D  5.326D-2,5.330D-2,5.334D-2,5.337D-2/
      DATA PRK4 /12*0.0D0,1.500D-4,5.404D-4,1.232D-3,2.289D-3,
     1  3.778D-3,5.756D-3,7.910D-3,1.025D-2,1.230D-2,1.448D-2,
     2  1.674D-2,1.870D-2,2.152D-2,2.400D-2,2.651D-2,2.903D-2,
     3  3.109D-2,3.400D-2,3.699D-2,4.004D-2,4.309D-2,4.606D-2,
     4  4.900D-2,5.182D-2,5.459D-2,5.725D-2,5.982D-2,6.226D-2,
     5  6.461D-2,6.689D-2,6.903D-2,7.109D-2,7.304D-2,7.487D-2,
     6  7.663D-2,7.830D-2,7.985D-2,8.134D-2,8.266D-2,8.392D-2,
     7  8.515D-2,8.628D-2,8.736D-2,8.835D-2,8.932D-2,9.028D-2,
     8  9.112D-2,9.194D-2,9.273D-2,9.344D-2,9.415D-2,9.479D-2,
     9  9.545D-2,9.602D-2,9.652D-2,9.706D-2,9.762D-2,9.815D-2,
     A  9.863D-2,9.906D-2,9.948D-2,9.976D-2,1.001D-1,1.005D-1,
     B  1.008D-1,1.011D-1,1.014D-1,1.017D-1,1.020D-1,1.023D-1,
     C  1.025D-1,1.028D-1,1.029D-1,1.032D-1,1.033D-1,1.036D-1,
     D  1.037D-1,1.040D-1,1.042D-1,1.044D-1/
      DATA PRK5 /30*0.0D0,1.080D-4,3.342D-4,6.722D-4,1.119D-3,
     1  1.674D-3,2.335D-3,3.051D-3,3.805D-3,4.526D-3,5.236D-3,
     2  5.866D-3,6.341D-3,6.843D-3,7.158D-3,7.470D-3,7.784D-3,
     3  8.016D-3,8.205D-3,8.395D-3,8.585D-3,8.774D-3,8.965D-3,
     4  9.156D-3,9.331D-3,9.483D-3,9.634D-3,9.784D-3,9.920D-3,
     5  1.006D-2,1.020D-2,1.034D-2,1.047D-2,1.060D-2,1.077D-2,
     6  1.083D-2,1.093D-2,1.102D-2,1.112D-2,1.121D-2,1.130D-2,
     7  1.141D-2,1.153D-2,1.164D-2,1.175D-2,1.186D-2,1.197D-2,
     8  1.208D-2,1.219D-2,1.229D-2,1.239D-2,1.249D-2,1.259D-2,
     9  1.268D-2,1.277D-2,1.285D-2,1.293D-2,1.301D-2,1.309D-2,
     A  1.316D-2,1.322D-2,1.329D-2,1.335D-2/
      DATA PRK6 /30*0.0D0,2.060D-4,6.409D-4,1.292D-3,2.154D-3,
     1  3.222D-3,4.492D-3,5.880D-3,7.275D-3,8.714D-3,1.006D-2,
     2  1.122D-2,1.223D-2,1.315D-2,1.378D-2,1.443D-2,1.508D-2,
     3  1.550D-2,1.592D-2,1.634D-2,1.677D-2,1.719D-2,1.762D-2,
     4  1.791D-2,1.820D-2,1.848D-2,1.877D-2,1.905D-2,1.930D-2,
     5  1.956D-2,1.983D-2,2.009D-2,2.033D-2,2.057D-2,2.090D-2,
     6  2.099D-2,2.118D-2,2.134D-2,2.151D-2,2.168D-2,2.185D-2,
     7  2.207D-2,2.231D-2,2.256D-2,2.278D-2,2.304D-2,2.327D-2,
     8  2.353D-2,2.377D-2,2.402D-2,2.427D-2,2.450D-2,2.473D-2,
     9  2.498D-2,2.521D-2,2.544D-2,2.567D-2,2.590D-2,2.612D-2,
     A  2.634D-2,2.657D-2,2.678D-2,2.700D-2/
      DATA PRK7 /30*0.0D0,1.000D-4,1.000D-4,1.000D-4,1.000D-4,
     1  1.000D-4,1.000D-4,2.000D-4,2.000D-4,2.000D-4,3.000D-4,
     2  3.000D-4,4.000D-4,4.000D-4,5.000D-4,6.000D-4,7.000D-4,
     3  7.000D-4,8.000D-4,1.000D-3,1.400D-3,1.900D-3,2.600D-3,
     4  3.300D-3,4.000D-3,4.700D-3,5.400D-3,5.900D-3,6.400D-3,
     5  7.000D-3,7.400D-3,7.900D-3,8.300D-3,8.700D-3,9.100D-3,
     6  9.300D-3,9.700D-3,9.900D-3,1.010D-2,1.030D-2,1.060D-2,
     7  1.080D-2,1.090D-2,1.110D-2,1.130D-2,1.140D-2,1.160D-2,
     8  1.180D-2,1.200D-2,1.220D-2,1.240D-2,1.260D-2,1.280D-2,
     9  1.320D-2,1.370D-2,1.410D-2,1.450D-2,1.500D-2,1.550D-2,
     A  1.590D-2,1.620D-2,1.660D-2,1.680D-2/
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                       NUMERICAL TOOLS.                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  **************************************************************
C                       SUBROUTINE GABQ
C  **************************************************************
      SUBROUTINE GABQ(FCT,XL,XU,SUM,TOL,IER)
C
C     THIS INTEGRATION SUBROUTINE APPLIES THE GAUSS METHOD WITH
C  AN ADAPTIVE-BIPARTITION SCHEME.
C     FCT IS THE (EXTERNAL) FUNCTION BEING INTEGRATED OVER THE
C  INTERVAL (XL,XU). SUM IS THE RESULTANT VALUE OF THE INTEGRAL.
C     TOL IS THE TOLERANCE, I.E. MAXIMUM RELATIVE ERROR REQUIRED
C  ON THE COMPUTED VALUE (SUM). TOL SHOULD NOT EXCEED 1.0D-13.
C     IER IS AN ERROR CONTROL PARAMETER; ITS OUTPUT VALUE IS
C  IER=0 IF THE INTEGRATION ALGORITHM HAS BEEN ABLE TO GET THE
C  REQUIRED ACCURACY AND IER=1 OTHERWISE.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      DIMENSION X(10),W(10),S(128),SN(128),L(128),LN(128)
C  ****  PRINTED OUTPUT OF PARTIAL RESULTS: SET IWR=1.
      DATA IWR/0/
C  ****  COEFFICIENTS FOR GAUSS 20-POINT INTEGRATION.
      DATA NP,NP2,NP4/10,20,40/
C  ****  ABSCISSAS.
      DATA X/7.6526521133497334D-02,2.2778585114164508D-01,
     1       3.7370608871541956D-01,5.1086700195082710D-01,
     2       6.3605368072651503D-01,7.4633190646015079D-01,
     3       8.3911697182221882D-01,9.1223442825132591D-01,
     4       9.6397192727791379D-01,9.9312859918509492D-01/
C  ****  WEIGHTS.
      DATA W/1.5275338713072585D-01,1.4917298647260375D-01,
     1       1.4209610931838205D-01,1.3168863844917663D-01,
     2       1.1819453196151842D-01,1.0193011981724044D-01,
     3       8.3276741576704749D-02,6.2672048334109064D-02,
     4       4.0601429800386941D-02,1.7614007139152118D-02/
C  ****  CORRECTED TOLERANCE.
      CTOL=DMAX1(TOL,1.0D-13)
      PTOL=0.01D0*CTOL
      ERR=1.0D35
      H=XU-XL
C
      IF(IWR.EQ.1) THEN
      WRITE(6,10)
   10 FORMAT(///5X,'GAUSS ADAPTIVE-BIPARTITION QUADRATURE')
      WRITE(6,11) XL,XU,TOL
   11 FORMAT(/5X,'XL = ',1P,E15.8,', XU = ',E15.8,', TOL = ',
     1  E8.1)
      ENDIF
      IER=0
C  ****  GAUSS INTEGRATION FROM XL TO XU.
      A=0.5D0*(XU-XL)
      B=0.5D0*(XL+XU)
      C=A*X(1)
      D=W(1)*(FCT(B+C)+FCT(B-C))
      DO 1 I1=2,NP
      C=A*X(I1)
    1 D=D+W(I1)*(FCT(B+C)+FCT(B-C))
      SUM=D*A
C  ****  ADAPTIVE BIPARTITION SCHEME.
      ICALL=NP2
      LH=1
      S(1)=SUM
      L(1)=1
    2 HO=H
      H=0.5D0*H
      ASUM=SUM
      LHN=0
      DO 5 I=1,LH
      K=L(I)
      SI=S(I)
      XA=XL+(K-1)*HO
      XB=XA+H
      XC=XA+HO
      A=0.5D0*(XB-XA)
      B=0.5D0*(XB+XA)
      C=A*X(1)
      D=W(1)*(FCT(B+C)+FCT(B-C))
      DO 3 I2=2,NP
      C=A*X(I2)
    3 D=D+W(I2)*(FCT(B+C)+FCT(B-C))
      S1=D*A
      A=0.5D0*(XC-XB)
      B=0.5D0*(XC+XB)
      C=A*X(1)
      D=W(1)*(FCT(B+C)+FCT(B-C))
      DO 4 I3=2,NP
      C=A*X(I3)
    4 D=D+W(I3)*(FCT(B+C)+FCT(B-C))
      S2=D*A
      ICALL=ICALL+NP4
      S12=S1+S2
      SUM=SUM+S12-SI
      IF(DABS(S12-SI).LT.DMAX1(PTOL*DABS(S12),1.0D-35)) GO TO 5
      LHN=LHN+2
      IF(LHN.GT.128.OR.ICALL.GT.9999) GO TO 8
      SN(LHN)=S2
      LN(LHN)=K+K
      SN(LHN-1)=S1
      LN(LHN-1)=LN(LHN)-1
    5 CONTINUE
      ERR=DABS(SUM-ASUM)/DMAX1(DABS(SUM),1.0D-35)
      IF(IWR.EQ.1) WRITE(6,12) ICALL,SUM,ERR,LHN
   12 FORMAT(5X,'N = ',I5,', SUM = ',1P,E19.12,', ERR = ',E8.1,
     1  ', LH = ',I3)
      IF(ERR.GT.CTOL.AND.LHN.GT.0) GO TO 6
      IF(IWR.EQ.1) WRITE(6,13)
   13 FORMAT(5X,'END OF GAUSS-BIPARTITION PROCEDURE'///)
      RETURN
    6 LH=LHN
      DO 7 I=1,LH
      S(I)=SN(I)
    7 L(I)=LN(I)
      GO TO 2
C  ****  WARNING (LOW ACCURACY) MESSAGE.
    8 WRITE(6,14)
   14 FORMAT(/5X,'*** LOW ACCURACY IN SUBROUTINE GABQ.')
      WRITE(6,11) XL,XU,TOL
      WRITE(6,15) SUM,ERR
   15 FORMAT(5X,'SUM = ',1P,E19.12,', ERR = ',E8.1//)
      IER=1
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE SPLINE
C  **************************************************************
      SUBROUTINE SPLINE(X,Y,A,B,C,D,S1,SN,N)
C
C     CUBIC SPLINE INTERPOLATION BETWEEN TABULATED DATA.
C
C  INPUT:
C     X(I) (I=1, ...,N) ........ GRID POINTS.
C                     (THE X VALUES MUST BE IN INCREASING ORDER).
C     Y(I) (I=1, ...,N) ........ CORRESPONDING FUNCTION VALUES.
C     S1,SN ..... SECOND DERIVATIVES AT X(1) AND X(N).
C             (THE NATURAL SPLINE CORRESPONDS TO TAKING S1=SN=0).
C     N ........................ NUMBER OF GRID POINTS.
C
C     THE INTERPOLATING POLYNOMIAL IN THE I-TH INTERVAL, FROM
C  X(I) TO X(I+1), IS PI(X)=A(I)+X*(B(I)+X*(C(I)+X*D(I))).
C
C  OUTPUT:
C     A(I),B(I),C(I),D(I) ...... SPLINE COEFFICIENTS.
C
C     REF.: M.J. MARON, 'NUMERICAL ANALYSIS: A PRACTICAL
C           APPROACH', MACMILLAN PUBL. CO., NEW YORK 1982.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      DIMENSION X(N),Y(N),A(N),B(N),C(N),D(N)
      IF(N.LT.4) THEN
      WRITE(6,10) N
   10 FORMAT(5X,'SPLINE INTERPOLATION CANNOT BE PERFORMED WITH',
     1  I4,' POINTS. STOP.')
      STOP
      ENDIF
      N1=N-1
      N2=N-2
C  ****  AUXILIARY ARRAYS H(=A) AND DELTA(=D).
      DO 1 I=1,N1
      IF(X(I+1)-X(I).LT.1.0D-10) THEN
      WRITE(6,11)
   11 FORMAT(5X,'SPLINE X VALUES NOT IN INCREASING ORDER. STOP.')
      STOP
      ENDIF
      A(I)=X(I+1)-X(I)
    1 D(I)=(Y(I+1)-Y(I))/A(I)
C  ****  SYMMETRIC COEFFICIENT MATRIX (AUGMENTED).
      DO 2 I=1,N2
      B(I)=2.0D0*(A(I)+A(I+1))
      K=N1-I+1
    2 D(K)=6.0D0*(D(K)-D(K-1))
      D(2)=D(2)-A(1)*S1
      D(N1)=D(N1)-A(N1)*SN
C  ****  GAUSS SOLUTION OF THE TRIDIAGONAL SYSTEM.
      DO 3 I=2,N2
      R=A(I)/B(I-1)
      B(I)=B(I)-R*A(I)
    3 D(I+1)=D(I+1)-R*D(I)
C  ****  THE SIGMA COEFFICIENTS ARE STORED IN ARRAY D.
      D(N1)=D(N1)/B(N2)
      DO 4 I=2,N2
      K=N1-I+1
    4 D(K)=(D(K)-A(K)*D(K+1))/B(K-1)
      D(N)=SN
C  ****  SPLINE COEFFICIENTS.
      SI1=S1
      DO 5 I=1,N1
      SI=SI1
      SI1=D(I+1)
      H=A(I)
      HI=1.0D0/H
      A(I)=(HI/6.0D0)*(SI*X(I+1)**3-SI1*X(I)**3)
     1    +HI*(Y(I)*X(I+1)-Y(I+1)*X(I))
     2    +(H/6.0D0)*(SI1*X(I)-SI*X(I+1))
      B(I)=(HI/2.0D0)*(SI1*X(I)**2-SI*X(I+1)**2)
     1    +HI*(Y(I+1)-Y(I))+(H/6.0D0)*(SI-SI1)
      C(I)=(HI/2.0D0)*(SI*X(I+1)-SI1*X(I))
    5 D(I)=(HI/6.0D0)*(SI1-SI)
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE INTEG
C  **************************************************************
      SUBROUTINE INTEG(X,A,B,C,D,XL,XU,SUM,N)
C
C     INTEGRAL OF A CUBIC SPLINE FUNCTION.
C
C  INPUT:
C     X(I) (I=1, ...,N) ........ GRID POINTS.
C                     (THE X VALUES MUST BE IN INCREASING ORDER).
C     A(I),B(I),C(I),D(I) ...... SPLINE COEFFICIENTS.
C     N ........................ NUMBER OF GRID POINTS.
C     XL ....................... LOWER LIMIT IN THE INTEGRAL.
C     XU ....................... UPPER LIMIT IN THE INTEGRAL.
C
C  OUTPUT:
C     SUM ...................... VALUE OF THE INTEGRAL.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      DIMENSION X(N),A(N),B(N),C(N),D(N)
C  ****  SET INTEGRATION LIMITS IN INCREASING ORDER.
      SIGN=1.0D0
      IF(XU.LT.XL) THEN
      SUM=XL
      XL=XU
      XU=SUM
      SIGN=-1.0D0
      ENDIF
C  ****  CHECK INTEGRAL LIMITS.
      IWR=0
      IF(XL.LT.X(1).OR.XU.GT.X(N)) IWR=1
C  ****  FIND INVOLVED INTERVALS.
      SUM=0.0D0
      CALL FINDI(X,XL,N,IL)
      CALL FINDI(X,XU,N,IU)
C  ****  ONLY A SINGLE INTERVAL INVOLVED.
      IF(IL.EQ.IU) THEN
      X1=XL
      X2=XU
      SUM=X2*(A(IL)+X2*((B(IL)/2)+X2*((C(IL)/3)+X2*D(IL)/4)))
     1   -X1*(A(IL)+X1*((B(IL)/2)+X1*((C(IL)/3)+X1*D(IL)/4)))
      GO TO 2
      ENDIF
C  ****  CONTRIBUTIONS FROM DIFFERENT INTERVALS.
      X1=XL
      X2=X(IL+1)
      SUM=X2*(A(IL)+X2*((B(IL)/2)+X2*((C(IL)/3)+X2*D(IL)/4)))
     1   -X1*(A(IL)+X1*((B(IL)/2)+X1*((C(IL)/3)+X1*D(IL)/4)))
      IL=IL+1
      DO 1 I=IL,IU
      X1=X(I)
      X2=X(I+1)
      IF(I.EQ.IU) X2=XU
      SUMP=X2*(A(I)+X2*((B(I)/2)+X2*((C(I)/3)+X2*D(I)/4)))
     1    -X1*(A(I)+X1*((B(I)/2)+X1*((C(I)/3)+X1*D(I)/4)))
    1 SUM=SUM+SUMP
    2 SUM=SIGN*SUM
C  ****  INTEGRAL LIMITS OUT OF RANGE.
      IF(IWR.EQ.1) WRITE(6,10)
   10 FORMAT(/'*** WARNING: INTEGRAL LIMITS OUT OF RANGE. ***')
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE FINDI
C  **************************************************************
      SUBROUTINE FINDI(X,XC,N,I)
C
C     FINDS THE INTERVAL (X(I),X(I+1)) CONTAINING THE VALUE XC.
C
C  INPUT:
C     X(I) (I=1, ...,N) ........ GRID POINTS.
C                     (THE X VALUES MUST BE IN INCREASING ORDER).
C     XC ....................... POINT TO BE LOCATED.
C     N ........................ NUMBER OF GRID POINTS.
C
C  OUTPUT:
C     I ........................ INTERVAL INDEX.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      DIMENSION X(N)
      IF(XC.GT.X(N)) THEN
      I=N-1
      RETURN
      ENDIF
      IF(XC.LT.X(1)) THEN
      I=1
      RETURN
      ENDIF
      I=1
      I1=N
    1 IT=(I+I1)/2
      IF(XC.GT.X(IT)) THEN
        I=IT
      ELSE
        I1=IT
      ENDIF
      IF(I1-I.GT.1) GO TO 1
      RETURN
      END
C  **************************************************************
C                         FUNCTION RAND
C  **************************************************************
      DOUBLE PRECISION FUNCTION RAND(DUMMY)
C
C     THIS IS AN ADAPTED VERSION OF SUBROUTINE RANECU WRITTEN BY
C  F. JAMES (COMPUT. PHYS. COMMUN. 60 (1990) 329-344), WHICH HAS
C  BEEN MODIFIED TO GIVE A SINGLE RANDOM NUMBER AT EACH CALL.
C     THE 'SEEDS' ISEED1 AND ISEED2 MUST BE INITIALIZED IN THE
C  MAIN PROGRAM AND TRANSFERRED THROUGH THE NAMED COMMON BLOCK
C  /RSEED/.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
      PARAMETER (USCALE=1.0D0/2.0D0**31)
      COMMON/RSEED/ISEED1,ISEED2
C
      I1=ISEED1/53668
      ISEED1=40014*(ISEED1-I1*53668)-I1*12211
      IF(ISEED1.LT.0) ISEED1=ISEED1+2147483563
C
      I2=ISEED2/52774
      ISEED2=40692*(ISEED2-I2*52774)-I2*3791
      IF(ISEED2.LT.0) ISEED2=ISEED2+2147483399
C
      IZ=ISEED1-ISEED2
      IF(IZ.LT.1) IZ=IZ+2147483562
      RAND=IZ*USCALE
C
      RETURN
      END
C  **************************************************************
C                       SUBROUTINE DIRECT
C  **************************************************************
      SUBROUTINE DIRECT(CDT,DF,U,V,W)
C
C     This subroutine computes the direction cosines of the par-
C  ticle velocity after a collision with given polar and azimu-
C  thal scattering angles.
C
C  Input:  U,V,W ... initial direction cosines,
C          CDT ..... cosine of the polar scattering angle,
C          DF ...... azimuthal scattering angle (rad).
C
C  OUTPUT: U,V,W ... new direction cosines.
C          CDT and DF remain unchanged.
C
C                                 Francesc Salvat. January, 2000.
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z), INTEGER*4 (I-N)
C
      SDF=DSIN(DF)
      CDF=DCOS(DF)
C  ****  Ensure normalization.
      DXY=U*U+V*V
      DXYZ=DXY+W*W
      IF(DABS(DXYZ-1.0D0).GT.1.0D-14) THEN
        FNORM=1.0D0/DSQRT(DXYZ)
        U=FNORM*U
        V=FNORM*V
        W=FNORM*W
        DXY=U*U+V*V
      ENDIF
C
      IF(DXY.GT.1.0D-28) THEN
        SDT=DSQRT((1.0D0-CDT*CDT)/DXY)
        UP=U
        U=U*CDT+SDT*(UP*W*CDF-V*SDF)
        V=V*CDT+SDT*(V*W*CDF+UP*SDF)
        W=W*CDT-DXY*SDT*CDF
      ELSE
        SDT=DSQRT(1.0D0-CDT*CDT)
        V=SDT*SDF
        IF(W.GT.0.0D0) THEN
          U=SDT*CDF
          W=CDT
        ELSE
          U=-SDT*CDF
          W=-CDT
        ENDIF
      ENDIF
C
      RETURN
      END
